@@ -1,4 +1,4 @@
-/* Copyright (c) 2010-2023. The SimGrid Team. All rights reserved.          */
+/* Copyright (c) 2010-2025. The SimGrid Team. All rights reserved.          */
 
 /* This program is free software; you can redistribute it and/or modify it
  * under the terms of the license (GNU LGPL) which comes with this package. */
@@ -46,8 +46,8 @@ int main(int argc, char* argv[])
 
   simgrid_load_platform(argv[1]);
 
-  sg_actor_create("executor", sg_host_by_name("Tremblay"), executor, 0, NULL);
-  sg_actor_create("privileged", sg_host_by_name("Tremblay"), privileged, 0, NULL);
+  sg_actor_create("executor", sg_host_by_name("Tremblay"), &executor, 0, NULL);
+  sg_actor_create("privileged", sg_host_by_name("Tremblay"), &privileged, 0, NULL);
 
   simgrid_run();
 
