@@ -1,4 +1,4 @@
-/* Copyright (c) 2017-2023. The SimGrid Team. All rights reserved.          */
+/* Copyright (c) 2017-2025. The SimGrid Team. All rights reserved.          */
 
 /* This program is free software; you can redistribute it and/or modify it
  * under the terms of the license (GNU LGPL) which comes with this package. */
@@ -26,9 +26,11 @@ static void victimB_fun()
 
 static void killer()
 {
+  auto e = simgrid::s4u::this_actor::get_engine();
+
   XBT_INFO("Hello!"); /* - First start a victim actor */
-  sg4::ActorPtr victimA = sg4::Actor::create("victim A", sg4::Host::by_name("Fafard"), victimA_fun);
-  sg4::ActorPtr victimB = sg4::Actor::create("victim B", sg4::Host::by_name("Jupiter"), victimB_fun);
+  sg4::ActorPtr victimA = e->add_actor("victim A", sg4::Host::by_name("Fafard"), victimA_fun);
+  sg4::ActorPtr victimB = e->add_actor("victim B", sg4::Host::by_name("Jupiter"), victimB_fun);
   sg4::this_actor::sleep_for(10); /* - Wait for 10 seconds */
 
   XBT_INFO("Resume the victim A"); /* - Resume it from its suspended state */
@@ -46,7 +48,7 @@ static void killer()
   sg4::this_actor::sleep_for(1);
 
   XBT_INFO("Start a new actor, and kill it right away");
-  sg4::ActorPtr victimC = sg4::Actor::create("victim C", sg4::Host::by_name("Jupiter"), victimA_fun);
+  sg4::ActorPtr victimC = e->add_actor("victim C", sg4::Host::by_name("Jupiter"), victimA_fun);
   victimC->kill();
 
   sg4::this_actor::sleep_for(1);
@@ -67,7 +69,7 @@ int main(int argc, char* argv[])
 
   e.load_platform(argv[1]); /* - Load the platform description */
   /* - Create and deploy killer actor, that will create the victim actors  */
-  sg4::Actor::create("killer", e.host_by_name("Tremblay"), killer);
+  e.add_actor("killer", e.host_by_name("Tremblay"), killer);
 
   e.run(); /* - Run the simulation */
 
