@@ -1,23 +1,25 @@
-/* Copyright (c) 2003-2023. The SimGrid Team. All rights reserved.          */
+/* Copyright (c) 2003-2025. The SimGrid Team. All rights reserved.          */
 
 /* This program is free software; you can redistribute it and/or modify it
  * under the terms of the license (GNU LGPL) which comes with this package. */
 
-#include "simgrid/plugins/battery.hpp"
-#include "simgrid/plugins/energy.h"
-#include "simgrid/s4u.hpp"
+#include <simgrid/plugins/battery.hpp>
+#include <simgrid/plugins/energy.h>
+#include <simgrid/s4u.hpp>
 #include <simgrid/s4u/Actor.hpp>
 #include <simgrid/s4u/Engine.hpp>
 
 XBT_LOG_NEW_DEFAULT_CATEGORY(battery_energy, "Messages specific for this s4u example");
+namespace sg4 = simgrid::s4u;
 
 static void manager()
 {
   auto battery = simgrid::plugins::Battery::init("Battery", 0.8, -300, 300, 0.9, 0.9, 10, 1000);
 
-  auto* host1 = simgrid::s4u::Engine::get_instance()->host_by_name("MyHost1");
-  auto* host2 = simgrid::s4u::Engine::get_instance()->host_by_name("MyHost2");
-  auto* host3 = simgrid::s4u::Engine::get_instance()->host_by_name("MyHost3");
+  auto e      = simgrid::s4u::this_actor::get_engine();
+  auto* host1 = e->host_by_name("MyHost1");
+  auto* host2 = e->host_by_name("MyHost2");
+  auto* host3 = e->host_by_name("MyHost3");
 
   battery->schedule_handler(
       0.2, simgrid::plugins::Battery::DISCHARGE, simgrid::plugins::Battery::Handler::PERSISTANT,
@@ -44,7 +46,7 @@ static void manager()
   XBT_INFO("Host %s will now execute %f flops", host1->get_cname(), flops);
   host2->execute(flops);
 
-  simgrid::s4u::this_actor::sleep_until(200);
+  sg4::this_actor::sleep_until(200);
   XBT_INFO("Battery state: SoC: %f SoH: %f Energy stored: %fJ Energy provided: %fJ Energy consumed %fJ",
            battery->get_state_of_charge(), battery->get_state_of_health(), battery->get_energy_stored(),
            battery->get_energy_provided(), battery->get_energy_consumed());
@@ -52,12 +54,12 @@ static void manager()
 
 int main(int argc, char* argv[])
 {
-  simgrid::s4u::Engine e(&argc, argv);
+  sg4::Engine e(&argc, argv);
   // if you plan to use Battery::connect_host you have to init the energy plugin at start.
   sg_host_energy_plugin_init();
   e.load_platform(argv[1]);
 
-  simgrid::s4u::Actor::create("manager", e.host_by_name("MyHost1"), manager);
+  e.add_actor("manager", e.host_by_name("MyHost1"), manager);
 
   e.run();
   return 0;
