@@ -1,10 +1,11 @@
-/* Copyright (c) 2007-2023. The SimGrid Team. All rights reserved.          */
+/* Copyright (c) 2007-2025. The SimGrid Team. All rights reserved.          */
 
 /* This program is free software; you can redistribute it and/or modify it
  * under the terms of the license (GNU LGPL) which comes with this package. */
 
-#include "simgrid/s4u.hpp"
 #include "simgrid/plugins/live_migration.h"
+#include "simgrid/s4u.hpp"
+#include "simgrid/s4u/Engine.hpp"
 #include "simgrid/s4u/VirtualMachine.hpp"
 
 XBT_LOG_NEW_DEFAULT_CATEGORY(s4u_cloud_migration, "Messages specific for this example");
@@ -22,7 +23,7 @@ static void vm_migrate(sg4::VirtualMachine* vm, sg4::Host* dst_pm)
 
 static void vm_migrate_async(sg4::VirtualMachine* vm, sg4::Host* dst_pm)
 {
-  sg4::Actor::create("mig_wrk", sg4::Host::current(), vm_migrate, vm, dst_pm);
+  sg4::Engine::get_instance()->add_actor("mig_wrk", sg4::Host::current(), vm_migrate, vm, dst_pm);
 }
 
 static void master_main()
@@ -91,7 +92,7 @@ int main(int argc, char* argv[])
   /* load the platform file */
   e.load_platform(argv[1]);
 
-  sg4::Actor::create("master_", sg4::Host::by_name("Fafard"), master_main);
+  e.add_actor("master_", sg4::Host::by_name("Fafard"), master_main);
 
   e.run();
 
