@@ -1,4 +1,4 @@
-/* Copyright (c) 2007-2023. The SimGrid Team. All rights reserved.          */
+/* Copyright (c) 2007-2025. The SimGrid Team. All rights reserved.          */
 
 /* This program is free software; you can redistribute it and/or modify it
  * under the terms of the license (GNU LGPL) which comes with this package. */
@@ -8,6 +8,22 @@
 XBT_LOG_NEW_DEFAULT_CATEGORY(s4u_test, "Messages specific for this s4u example");
 namespace sg4 = simgrid::s4u;
 
+/* This actor simply starts an activity in a fire and forget (a.k.a. detached) mode and quit.*/
+static void detached()
+{
+  double computation_amount = sg4::this_actor::get_host()->get_speed();
+  XBT_INFO("Execute %g flops, should take 1 second.", computation_amount);
+  sg4::ExecPtr activity = sg4::this_actor::exec_init(computation_amount);
+  // Attach a callback to print some log when the detached activity completes
+  activity->on_this_completion_cb([](sg4::Exec const&) {
+    XBT_INFO("Detached activity is done");
+  });
+
+  activity->detach();
+
+  XBT_INFO("Goodbye now!");
+}
+
 /* This actor simply waits for its activity completion after starting it.
  * That's exactly equivalent to synchronous execution. */
 static void waiter()
@@ -60,10 +76,12 @@ int main(int argc, char* argv[])
   sg4::Host* fafard  = e.host_by_name("Fafard");
   sg4::Host* ginette = e.host_by_name("Ginette");
   sg4::Host* boivin  = e.host_by_name("Boivin");
+  sg4::Host* tremblay  = e.host_by_name("Tremblay");
 
-  sg4::Actor::create("wait", fafard, waiter);
-  sg4::Actor::create("monitor", ginette, monitor);
-  sg4::Actor::create("cancel", boivin, canceller);
+  e.add_actor("wait", fafard, waiter);
+  e.add_actor("monitor", ginette, monitor);
+  e.add_actor("cancel", boivin, canceller);
+  e.add_actor("detach", tremblay, detached);
 
   e.run();
 
