@@ -1,4 +1,4 @@
-/* Copyright (c) 2010-2023. The SimGrid Team. All rights reserved.          */
+/* Copyright (c) 2010-2025. The SimGrid Team. All rights reserved.          */
 
 /* This program is free software; you can redistribute it and/or modify it
  * under the terms of the license (GNU LGPL) which comes with this package. */
@@ -44,17 +44,17 @@ static double cpu_nonlinear(const sg4::Host* host, double capacity, int n)
 }
 
 /** @brief Create a simple 1-host platform */
-static void load_platform()
+static void load_platform(sg4::Engine& e)
 {
-  auto* zone        = sg4::create_empty_zone("Zone1");
-  auto* runner_host = zone->create_host("runner", 1e6);
+  auto* zone        = e.get_netzone_root();
+  auto* runner_host = zone->add_host("runner", 1e6);
   runner_host->set_sharing_policy(sg4::Host::SharingPolicy::NONLINEAR,
                                   std::bind(&cpu_nonlinear, runner_host, std::placeholders::_1, std::placeholders::_2));
   runner_host->seal();
   zone->seal();
 
   /* create actor runner */
-  sg4::Actor::create("runner", runner_host, runner);
+  e.add_actor("runner", runner_host, runner);
 }
 
 /*************************************************************************************************/
@@ -63,7 +63,7 @@ int main(int argc, char* argv[])
   sg4::Engine e(&argc, argv);
 
   /* create platform */
-  load_platform();
+  load_platform(e);
 
   /* runs the simulation */
   e.run();
