@@ -1,4 +1,4 @@
-/* Copyright (c) 2017-2023. The SimGrid Team. All rights reserved.          */
+/* Copyright (c) 2017-2025. The SimGrid Team. All rights reserved.          */
 
 /* This program is free software; you can redistribute it and/or modify it
  * under the terms of the license (GNU LGPL) which comes with this package. */
@@ -87,7 +87,7 @@ int main(int argc, char* argv[])
   xbt_assert(argc == 2, "Usage: %s <platform file>", argv[0]);
 
   e.load_platform(argv[1]);
-  sg4::Actor::create("test", e.host_by_name("MyHost1"), runner);
+  e.add_actor("test", e.host_by_name("MyHost1"), runner);
 
   e.run();
   XBT_INFO("Simulation done.");
