@@ -1,4 +1,4 @@
-/* Copyright (c) 2007-2023. The SimGrid Team. All rights reserved.          */
+/* Copyright (c) 2007-2025. The SimGrid Team. All rights reserved.          */
 
 /* This program is free software; you can redistribute it and/or modify it
  * under the terms of the license (GNU LGPL) which comes with this package. */
@@ -62,7 +62,7 @@ int main(int argc, char* argv[])
 {
   sg4::Engine e(&argc, argv);
   e.load_platform(argv[1]);
-  sg4::Actor::create("test", e.host_by_name("Fafard"), wizard);
+  e.add_actor("test", e.host_by_name("Fafard"), wizard);
 
   e.run();
 
