@@ -1,4 +1,4 @@
-/* Copyright (c) 2010-2023. The SimGrid Team. All rights reserved.          */
+/* Copyright (c) 2010-2025. The SimGrid Team. All rights reserved.          */
 
 /* This program is free software; you can redistribute it and/or modify it
  * under the terms of the license (GNU LGPL) which comes with this package. */
@@ -45,10 +45,10 @@ int main(int argc, char* argv[])
 
   e.load_platform(argv[1]);
 
-  sg4::Actor::create("server", e.host_by_name("HostA"), server);
-  sg4::Actor::create("client", e.host_by_name("HostB"), client, 1);
-  sg4::Actor::create("client", e.host_by_name("HostC"), client, 2);
-  sg4::Actor::create("client", e.host_by_name("HostD"), client, 3);
+  e.add_actor("server", e.host_by_name("HostA"), server);
+  e.add_actor("client", e.host_by_name("HostB"), client, 1);
+  e.add_actor("client", e.host_by_name("HostC"), client, 2);
+  e.add_actor("client", e.host_by_name("HostD"), client, 3);
 
   e.run();
   return 0;
