@@ -1,4 +1,4 @@
-/* Copyright (c) 2017-2023. The SimGrid Team. All rights reserved.          */
+/* Copyright (c) 2017-2025. The SimGrid Team. All rights reserved.          */
 
 /* This program is free software; you can redistribute it and/or modify it
  * under the terms of the license (GNU LGPL) which comes with this package. */
@@ -51,7 +51,7 @@ int main(int argc, char* argv[])
   // Add a new host programmatically, and attach a simple speed profile to it (alternate between full and half speed
   // every two seconds
   e.get_netzone_root()
-      ->create_host("Lilibeth", 25e6)
+      ->add_host("Lilibeth", 25e6)
       ->set_speed_profile(simgrid::kernel::profile::ProfileBuilder::from_string("lilibeth_profile", R"(
 0 1.0
 2 0.5
@@ -60,7 +60,7 @@ int main(int argc, char* argv[])
       ->seal();
 
   // Add a watcher of the changes
-  sg4::Actor::create("watcher", e.host_by_name("Fafard"), watcher);
+  e.add_actor("watcher", e.host_by_name("Fafard"), watcher);
 
   e.run();
 
