@@ -1,4 +1,4 @@
-/* Copyright (c) 2006-2023. The SimGrid Team. All rights reserved.          */
+/* Copyright (c) 2006-2025. The SimGrid Team. All rights reserved.          */
 
 /* This program is free software; you can redistribute it and/or modify it
  * under the terms of the license (GNU LGPL) which comes with this package. */
@@ -24,13 +24,14 @@ static void master(int actor_count)
 
   XBT_INFO("Spawning %d workers", actor_count - 1);
   for (int i = 0; i < actor_count - 1; i++) {
-    sg4::Actor::create("worker", sg4::Host::by_name("Jupiter"), worker, barrier);
+    sg4::this_actor::get_engine()->add_actor("worker", sg4::Host::by_name("Jupiter"), worker, barrier);
   }
 
   XBT_INFO("Waiting on the barrier");
-  barrier->wait();
-
-  XBT_INFO("Bye");
+  if (barrier->wait())
+    XBT_INFO("Bye from the last to enter");
+  else
+    XBT_INFO("Bye");
 }
 
 int main(int argc, char **argv)
@@ -43,7 +44,7 @@ int main(int argc, char **argv)
   xbt_assert(actor_count > 0, "<actor-count> must be greater than 0");
 
   e.load_platform(argc > 2 ? argv[2] : "../../platforms/two_hosts.xml");
-  sg4::Actor::create("master", e.host_by_name("Tremblay"), master, actor_count);
+  e.add_actor("master", e.host_by_name("Tremblay"), master, actor_count);
   e.run();
 
   return 0;
