@@ -1,4 +1,4 @@
-/* Copyright (c) 2010-2023. The SimGrid Team. All rights reserved.          */
+/* Copyright (c) 2010-2025. The SimGrid Team. All rights reserved.          */
 
 /* This program is free software; you can redistribute it and/or modify it
  * under the terms of the license (GNU LGPL) which comes with this package. */
@@ -50,7 +50,7 @@ int main(int argc, char* argv[])
   simgrid::instr::declare_host_variable("HDD_capacity");
   simgrid::instr::declare_host_variable("HDD_utilization", "1 0 0"); // red color
 
-  sg4::Actor::create("master", e.host_by_name("Tremblay"), trace_fun);
+  e.add_actor("master", e.host_by_name("Tremblay"), trace_fun);
 
   e.run();
 
