@@ -1,4 +1,4 @@
-/* Copyright (c) 2010-2023. The SimGrid Team. All rights reserved.          */
+/* Copyright (c) 2010-2025. The SimGrid Team. All rights reserved.          */
 
 /* This program is free software; you can redistribute it and/or modify it
  * under the terms of the license (GNU LGPL) which comes with this package. */
@@ -49,17 +49,19 @@ int main(int argc, char* argv[])
   xbt_assert(argc > 1, "Usage: %s platform_file\n \tExample: %s small_platform.xml\n", argv[0], argv[0]);
 
   e.load_platform(argv[1]);
+  // Have to seal the platform before declaring new tracing variables
+  e.seal_platform();
 
   // declaring link user variables (one without, another with an RGB color)
   simgrid::instr::declare_link_variable("Link_Capacity");
   simgrid::instr::declare_link_variable("Link_Utilization", "0.9 0.1 0.1");
 
-  sg4::Actor::create("master", e.host_by_name("Tremblay"), trace_fun);
-  sg4::Actor::create("worker", e.host_by_name("Tremblay"), trace_fun);
-  sg4::Actor::create("worker", e.host_by_name("Jupiter"), trace_fun);
-  sg4::Actor::create("worker", e.host_by_name("Fafard"), trace_fun);
-  sg4::Actor::create("worker", e.host_by_name("Ginette"), trace_fun);
-  sg4::Actor::create("worker", e.host_by_name("Bourassa"), trace_fun);
+  e.add_actor("master", e.host_by_name("Tremblay"), trace_fun);
+  e.add_actor("worker", e.host_by_name("Tremblay"), trace_fun);
+  e.add_actor("worker", e.host_by_name("Jupiter"), trace_fun);
+  e.add_actor("worker", e.host_by_name("Fafard"), trace_fun);
+  e.add_actor("worker", e.host_by_name("Ginette"), trace_fun);
+  e.add_actor("worker", e.host_by_name("Bourassa"), trace_fun);
 
   e.run();
   return 0;
