@@ -1,4 +1,4 @@
-/* Copyright (c) 2010-2023. The SimGrid Team. All rights reserved.          */
+/* Copyright (c) 2010-2025. The SimGrid Team. All rights reserved.          */
 
 /* This program is free software; you can redistribute it and/or modify it
  * under the terms of the license (GNU LGPL) which comes with this package. */
@@ -48,11 +48,13 @@ int main(int argc, char* argv[])
   xbt_assert(argc > 1, "Usage: %s platform_file\n \tExample: %s small_platform.xml\n", argv[0], argv[0]);
 
   e.load_platform(argv[1]);
+  // Have to seal the platform before declaring new tracing variables
+  e.seal_platform();
 
   simgrid::instr::declare_tracing_category("migration_order");
 
-  sg4::Actor::create("emigrant", e.host_by_name("Fafard"), emigrant);
-  sg4::Actor::create("policeman", e.host_by_name("Tremblay"), policeman);
+  e.add_actor("emigrant", e.host_by_name("Fafard"), emigrant);
+  e.add_actor("policeman", e.host_by_name("Tremblay"), policeman);
 
   e.run();
   return 0;
