@@ -1,4 +1,4 @@
-# Copyright (c) 2017-2023. The SimGrid Team. All rights reserved.
+# Copyright (c) 2017-2025. The SimGrid Team. All rights reserved.
 #
 # This program is free software you can redistribute it and/or modify it
 # under the terms of the license (GNU LGPL) which comes with this package.
@@ -15,11 +15,11 @@ def bob():
   disk = Host.current().get_disks()[0]
 
   this_actor.info("Create my asynchronous activities")
-  exec = this_actor.exec_async(5e9)
+  exe  = this_actor.exec_async(5e9)
   comm = mbox.get_async()
   io   = disk.read_async(300000000)
 
-  pending_activities = ActivitySet([exec, comm])
+  pending_activities = ActivitySet([exe, comm])
   pending_activities.push(io) # Activities can be pushed after creation, too
  
   this_actor.info("Wait for asynchronous activities to complete, all in one shot.")
@@ -38,7 +38,7 @@ if __name__ == '__main__':
   # Load the platform description
   e.load_platform(sys.argv[1])
 
-  Actor.create("bob",   Host.by_name("bob"), bob)
-  Actor.create("alice", Host.by_name("alice"), alice)
+  e.add_actor("bob",   Host.by_name("bob"), bob)
+  e.add_actor("alice", Host.by_name("alice"), alice)
 
   e.run()
