@@ -1,4 +1,4 @@
-# Copyright (c) 2006-2023. The SimGrid Team. All rights reserved.
+# Copyright (c) 2006-2025. The SimGrid Team. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or modify it
 # under the terms of the license (GNU LGPL) which comes with this package.
@@ -15,7 +15,7 @@ you ask SimGrid to start your actors. There is three ways of doing so:
 This example shows all these solutions, even if you obviously should use only one of these solutions to start your
 actors. The most advised solution is to use a deployment file, as it creates a clear separation between your
 application and the settings to test it. This is a better scientific methodology. Actually, starting an actor with
-Actor.create() is mostly useful to start an actor from another actor.
+e.add_actor() is mostly useful to start an actor from another actor.
 """
 
 import sys
@@ -89,15 +89,15 @@ if __name__ == '__main__':
     # The easiest way to do so is to implement the behavior of your actor in a single function,
     # as we do here for the receiver actors. This function can take any kind of parameters, as
     # long as the last parameters of Actor::create() match what your function expects.
-    Actor.create("receiver", Host.by_name("Fafard"), receiver, "mb42")
+    e.add_actor("receiver", Host.by_name("Fafard"), receiver, "mb42")
 
     # If your actor is getting more complex, you probably want to implement it as a class instead,
     # as we do here for the sender actors. The main behavior goes into operator()() of the class.
     #
     # You can then directly start your actor, as follows:
-    Actor.create("sender1", Host.by_name("Tremblay"), Sender())
+    e.add_actor("sender1", Host.by_name("Tremblay"), Sender())
     # If you want to pass parameters to your class, that's very easy: just use your constructors
-    Actor.create("sender2", Host.by_name("Jupiter"), Sender("GloubiBoulga"))
+    e.add_actor("sender2", Host.by_name("Jupiter"), Sender("GloubiBoulga"))
 
     # But starting actors directly is considered as a bad experimental habit, since it ties the code
     # you want to test with the experimental scenario. Starting your actors from an external deployment
