@@ -1,4 +1,4 @@
-/* Copyright (c) 2006-2023. The SimGrid Team. All rights reserved.          */
+/* Copyright (c) 2006-2025. The SimGrid Team. All rights reserved.          */
 
 /* This program is free software; you can redistribute it and/or modify it
  * under the terms of the license (GNU LGPL) which comes with this package. */
@@ -19,7 +19,9 @@
 #include <xbt/signal.hpp>
 #include <xbt/utility.hpp>
 
+#ifndef DOXYGEN
 XBT_LOG_EXTERNAL_CATEGORY(s4u_activity);
+#endif
 
 namespace simgrid {
 
@@ -33,6 +35,7 @@ namespace s4u {
  * That is, activities are all the things that do take time to the actor in the simulated world.
  */
 class XBT_PUBLIC Activity : public xbt::Extendable<Activity> {
+
 #ifndef DOXYGEN
   friend ActivitySet;
   friend Comm;
@@ -165,6 +168,8 @@ public:
   /** Blocks the current actor until the activity is terminated, or until the timeout is elapsed\n
    *  Raises: timeout exception.*/
   Activity* wait_for(double timeout);
+  /** Just like wait_for(), but the activity is first canceled if a timeout exception is raised.*/
+  Activity* wait_for_or_cancel(double timeout);
   /** Blocks the current actor until the activity is terminated, or until the time limit is reached\n
    * Raises: timeout exception. */
   void wait_until(double time_limit);
@@ -173,9 +178,15 @@ public:
   Activity* cancel();
   /** Retrieve the current state of the activity */
   Activity::State get_state() const { return state_; }
-  /** Return a string representation of the activity's state (one of INITED, STARTING, STARTED, CANCELED, FINISHED) */
+  /** Return a string representation of the activity's state (one of INITED, STARTING, STARTED, CANCELED, FINISHED,
+   * DONE) */
   const char* get_state_str() const;
+  bool is_canceled() const { return state_ == State::CANCELED; }
+  bool is_failed() const { return state_ == State::FAILED; }
+  bool is_done() const { return state_ == State::FINISHED; }
   void set_state(Activity::State state) { state_ = state; }
+  void set_detached (bool detached) { detached_ = detached; }
+  bool is_detached() const { return detached_;}
 
   /** Blocks the progression of this activity until it gets resumed */
   virtual Activity* suspend();
@@ -199,8 +210,8 @@ public:
   kernel::activity::ActivityImpl* get_impl() const { return pimpl_.get(); }
 
 #ifndef DOXYGEN
-  static ssize_t deprecated_wait_any_for(const std::vector<ActivityPtr>& activities, double timeout); // XBT_ATTRIB_DEPRECATED_v339
-  XBT_ATTRIB_DEPRECATED_v339("Please use ActivitySet instead") static ssize_t test_any(const std::vector<ActivityPtr>& activities);
+  static ssize_t deprecated_wait_any_for(const std::vector<ActivityPtr>& activities, double timeout); // XBT_ATTRIB_DEPRECATED_v401
+  XBT_ATTRIB_DEPRECATED_v401("Please use ActivitySet instead") static ssize_t test_any(const std::vector<ActivityPtr>& activities);
 
   friend void intrusive_ptr_release(Activity* a)
   {
@@ -224,6 +235,8 @@ private:
   double remains_                          = 0;
   bool suspended_                          = false;
   bool marked_                             = false;
+  bool detached_                           = false;
+
   std::vector<ActivityPtr> successors_;
   std::set<ActivityPtr> dependencies_;
   std::atomic_int_fast32_t refcount_{0};
@@ -232,60 +245,58 @@ private:
 template <class AnyActivity> class Activity_T : public Activity {
   std::string name_             = "unnamed";
   std::string tracing_category_ = "";
+  std::function<void(void*)> clean_fun_; // used if detached
 
-  inline static xbt::signal<void(AnyActivity const&)> on_start;
+  static xbt::signal<void(AnyActivity const&)> on_start;
   xbt::signal<void(AnyActivity const&)> on_this_start;
-  inline static xbt::signal<void(AnyActivity const&)> on_completion;
+  static xbt::signal<void(AnyActivity const&)> on_completion;
   xbt::signal<void(AnyActivity const&)> on_this_completion;
-  inline static xbt::signal<void(AnyActivity const&)> on_suspend;
+  static xbt::signal<void(AnyActivity const&)> on_suspend;
   xbt::signal<void(AnyActivity const&)> on_this_suspend;
-  inline static xbt::signal<void(AnyActivity const&)> on_resume;
+  static xbt::signal<void(AnyActivity const&)> on_resume;
   xbt::signal<void(AnyActivity const&)> on_this_resume;
-  inline static xbt::signal<void(AnyActivity&)> on_veto;
+  static xbt::signal<void(AnyActivity&)> on_veto;
   xbt::signal<void(AnyActivity&)> on_this_veto;
 
 protected:
-  void fire_on_start() const override { on_start(static_cast<const AnyActivity&>(*this)); }
+  void fire_on_start() const override;
   void fire_on_this_start() const override { on_this_start(static_cast<const AnyActivity&>(*this)); }
-  void fire_on_completion() const override { on_completion(static_cast<const AnyActivity&>(*this)); }
+  void fire_on_completion() const override;
   void fire_on_this_completion() const override { on_this_completion(static_cast<const AnyActivity&>(*this)); }
-  void fire_on_suspend() const override { on_suspend(static_cast<const AnyActivity&>(*this)); }
+  void fire_on_suspend() const override;
   void fire_on_this_suspend() const override { on_this_suspend(static_cast<const AnyActivity&>(*this)); }
-  void fire_on_resume() const override { on_resume(static_cast<const AnyActivity&>(*this)); }
+  void fire_on_resume() const override;
   void fire_on_this_resume() const override { on_this_resume(static_cast<const AnyActivity&>(*this)); }
-  void fire_on_veto() override { on_veto(static_cast<AnyActivity&>(*this)); }
+  void fire_on_veto() override;
   void fire_on_this_veto() override { on_this_veto(static_cast<AnyActivity&>(*this)); }
 
 public:
   /*! \static Add a callback fired when any activity starts (no veto) */
-  static void on_start_cb(const std::function<void(AnyActivity const&)>& cb) { on_start.connect(cb); }
+  static void on_start_cb(const std::function<void(AnyActivity const&)>& cb);
   /*!  Add a callback fired when this specific activity starts (no veto) */
   void on_this_start_cb(const std::function<void(AnyActivity const&)>& cb) { on_this_start.connect(cb); }
   /*! \static Add a callback fired when any activity completes (either normally, cancelled or failed) */
-  static void on_completion_cb(const std::function<void(AnyActivity const&)>& cb) { on_completion.connect(cb); }
+  static void on_completion_cb(const std::function<void(AnyActivity const&)>& cb);
   /*! Add a callback fired when this specific activity completes (either normally, cancelled or failed) */
   void on_this_completion_cb(const std::function<void(AnyActivity const&)>& cb) { on_this_completion.connect(cb); }
   /*! \static Add a callback fired when any activity is suspended */
-  static void on_suspend_cb(const std::function<void(AnyActivity const&)>& cb) { on_suspend.connect(cb); }
+  static void on_suspend_cb(const std::function<void(AnyActivity const&)>& cb);
   /*! Add a callback fired when this specific activity is suspended */
   void on_this_suspend_cb(const std::function<void(AnyActivity const&)>& cb) { on_this_suspend.connect(cb); }
   /*! \static Add a callback fired when any activity is resumed after being suspended */
-  static void on_resume_cb(const std::function<void(AnyActivity const&)>& cb) { on_resume.connect(cb); }
+  static void on_resume_cb(const std::function<void(AnyActivity const&)>& cb);
   /*! Add a callback fired when this specific activity is resumed after being suspended */
   void on_this_resume_cb(const std::function<void(AnyActivity const&)>& cb) { on_this_resume.connect(cb); }
   /*! \static Add a callback fired each time that any activity fails to start because of a veto (e.g., unsolved
    *  dependency or no resource assigned) */
-  static void on_veto_cb(const std::function<void(AnyActivity&)>& cb) { on_veto.connect(cb); }
+  static void on_veto_cb(const std::function<void(AnyActivity&)>& cb);
   /*! Add a callback fired each time that this specific activity fails to start because of a veto (e.g., unsolved
    *  dependency or no resource assigned) */
   void on_this_veto_cb(const std::function<void(AnyActivity&)>& cb) { on_this_veto.connect(cb); }
 
 #ifndef DOXYGEN
-  XBT_ATTRIB_DEPRECATED_v338("Please use on_suspend_cb() instead") static void on_suspended_cb(const std::function<void(Activity const&)>& cb) { on_suspend.connect(cb); }
-  XBT_ATTRIB_DEPRECATED_v338("Please use on_resume_cb() instead") static void on_resumed_cb(const std::function<void(Activity const&)>& cb) { on_resume.connect(cb); }
-
-  XBT_ATTRIB_DEPRECATED_v339("Please use ActivitySet instead") static ssize_t wait_any(const std::vector<ActivityPtr>& activities) { return deprecated_wait_any_for(activities, -1); }
-  XBT_ATTRIB_DEPRECATED_v339("Please use ActivitySet instead") static ssize_t wait_any_for(const std::vector<ActivityPtr>& activities, double timeout) { return deprecated_wait_any_for(activities, timeout); }
+  XBT_ATTRIB_DEPRECATED_v401("Please use ActivitySet instead") static ssize_t wait_any(const std::vector<ActivityPtr>& activities) { return deprecated_wait_any_for(activities, -1); }
+  XBT_ATTRIB_DEPRECATED_v401("Please use ActivitySet instead") static ssize_t wait_any_for(const std::vector<ActivityPtr>& activities, double timeout) { return deprecated_wait_any_for(activities, timeout); }
 #endif
 
   AnyActivity* add_successor(ActivityPtr a)
@@ -314,12 +325,28 @@ public:
     return static_cast<AnyActivity*>(this);
   }
   const std::string& get_tracing_category() const { return tracing_category_; }
+  const std::function<void(void*)>& get_clean_function() const { return clean_fun_; }
 
   AnyActivity* start()
   {
     Activity::start();
     return static_cast<AnyActivity*>(this);
   }
+   /** Start the activity, and ignore its result. It can be completely forgotten after that. */
+  AnyActivity* detach()
+  {
+    xbt_assert(get_state() == State::INITED || get_state() == State::STARTING,
+               "You cannot use %s() once your activity is %s (not implemented)", __func__, get_state_str());
+
+    set_detached(true);
+    return start();
+  }
+
+  AnyActivity* detach(const std::function<void(void*)>& clean_function)
+  {
+    clean_fun_ = clean_function;
+    return detach();
+  }
 
   AnyActivity* cancel() { return static_cast<AnyActivity*>(Activity::cancel()); }
   AnyActivity* wait() { return wait_for(-1.0); }
