@@ -1,4 +1,4 @@
-/* Copyright (c) 2006-2023. The SimGrid Team. All rights reserved.          */
+/* Copyright (c) 2006-2025. The SimGrid Team. All rights reserved.          */
 
 /* This program is free software; you can redistribute it and/or modify it
  * under the terms of the license (GNU LGPL) which comes with this package. */
@@ -7,6 +7,7 @@
 #define SIMGRID_S4U_ACTOR_HPP
 
 #include <simgrid/forward.h>
+#include <simgrid/s4u/Engine.hpp>
 
 #include <simgrid/chrono.hpp>
 #include <xbt/Extendable.hpp>
@@ -119,6 +120,7 @@ XBT_PUBLIC ExecPtr exec_init(double flops_amounts);
 XBT_PUBLIC ExecPtr exec_init(const std::vector<s4u::Host*>& hosts, const std::vector<double>& flops_amounts,
                              const std::vector<double>& bytes_amounts);
 
+/** Initialize and start a sequential execution */
 XBT_PUBLIC ExecPtr exec_async(double flops_amounts);
 
 /** @brief Returns the actor ID of the current actor. */
@@ -127,24 +129,30 @@ XBT_PUBLIC aid_t get_pid();
 /** @brief Returns the ancestor's actor ID of the current actor. */
 XBT_PUBLIC aid_t get_ppid();
 
+/** Returns the engine instance we're running in */
+XBT_PUBLIC Engine* get_engine();
+
 /** @brief Returns the name of the current actor. */
 XBT_PUBLIC std::string get_name();
 /** @brief Returns the name of the current actor as a C string. */
 XBT_PUBLIC const char* get_cname();
 
-/** @brief Returns the name of the host on which the current actor is running. */
-XBT_PUBLIC Host* get_host();
+/** Returns the name of the host on which the current actor is running. */
+XBT_PUBLIC s4u::Host* get_host();
+/** Migrate the current actor to a new host. */
+XBT_PUBLIC void set_host(s4u::Host* new_host);
 
-/** @brief Suspend the current actor, that is blocked until resume()ed by another actor. */
+/** Suspend the current actor, that is blocked until resume()ed by another actor. */
 XBT_PUBLIC void suspend();
 
-/** @brief Yield the current actor. */
+/** Yield the current actor, give the control to the other actors. It will be executed again in the next scheduling
+ * round */
 XBT_PUBLIC void yield();
 
-/** @brief kill the current actor. */
+/** kill the current actor. */
 XBT_ATTRIB_NORETURN XBT_PUBLIC void exit();
 
-/** @brief Add a function to the list of "on_exit" functions of the current actor.
+/** Add a function to the list of "on_exit" functions of the current actor.
  *
  * The on_exit functions are the functions executed when your actor is killed. You should use them to free the data used
  * by your actor.
@@ -160,8 +168,6 @@ XBT_ATTRIB_NORETURN XBT_PUBLIC void exit();
 
 XBT_PUBLIC void on_exit(const std::function<void(bool)>& fun);
 
-/** @brief Migrate the current actor to a new host. */
-XBT_PUBLIC void set_host(Host* new_host);
 } // namespace this_actor
 
 /** An actor is an independent stream of execution in your distributed application.
@@ -181,7 +187,8 @@ XBT_PUBLIC void set_host(Host* new_host);
  *
  * @endrst
  */
-class XBT_PUBLIC Actor : public xbt::Extendable<Actor> {
+class XBT_PUBLIC Actor : public simgrid::xbt::Extendable<s4u::Actor> {
+
 #ifndef DOXYGEN
   friend Exec;
   friend Mailbox;
@@ -208,7 +215,6 @@ public:
   /** Retrieve the amount of references on that object. Useful to debug the automatic refcounting */
   int get_refcount() const;
 
-  // ***** Actor creation *****
   /** \static
    * Retrieve a reference to myself
    */
@@ -283,11 +289,8 @@ public:
   /** Add a callback fired when this specific actor is about to disappear (its destructor was called). */
   void on_this_destruction_cb(const std::function<void(Actor const&)>& cb) { on_this_destruction.connect(cb); }
 
-  /** \static
-   *  Create an actor from a @c std::function<void()>.
-   *  If the actor is restarted, it gets a fresh copy of the function.
-   *  @verbatim embed:rst:inline See the :ref:`example <s4u_ex_actors_create>`. @endverbatim */
-  static ActorPtr create(const std::string& name, s4u::Host* host, const std::function<void()>& code);
+  XBT_ATTRIB_DEPRECATED_v401("Please use Engine::add_actor() instead") static ActorPtr
+      create(const std::string& name, s4u::Host* host, const std::function<void()>& code);
   /** \static
    *  Create an actor, but don't start it yet.
    *
@@ -299,10 +302,11 @@ public:
 
   template <class F> ActorPtr start(F code) { return start(std::function<void()>(std::move(code))); }
 
-  template <class F, class... Args,
+  template <class F, class... Args
   // This constructor is enabled only if the call code(args...) is valid:
 #ifndef DOXYGEN /* breathe seem to choke on function signatures in template parameter, see breathe#611 */
-            typename = typename std::result_of_t<F(Args...)>
+            ,
+            typename = typename std::invoke_result_t<F, Args...>
 #endif
             >
   ActorPtr start(F code, Args... args)
@@ -312,36 +316,26 @@ public:
 
   ActorPtr start(const std::function<void()>& code, std::vector<std::string> args);
 
-  /** \static
-   * Create an actor from a callable thing.
-   *  @verbatim embed:rst:inline See the :ref:`example <s4u_ex_actors_create>`. @endverbatim */
-  template <class F> static ActorPtr create(const std::string& name, s4u::Host* host, F code)
+  template <class F>
+  XBT_ATTRIB_DEPRECATED_v401("Please use Engine::add_actor() instead") static ActorPtr
+      create(const std::string& name, s4u::Host* host, F code)
   {
-    return create(name, host, std::function<void()>(std::move(code)));
+    return s4u::Engine::get_instance()->add_actor(name, host, std::function<void()>(std::move(code)));
   }
-
-  /** \static
-   * Create an actor using a callable thing and its arguments.
-   *
-   * Note that the arguments will be copied, so move-only parameters are forbidden.
-   * @verbatim embed:rst:inline See the :ref:`example <s4u_ex_actors_create>`. @endverbatim */
-
-  template <class F, class... Args,
-            // This constructor is enabled only if the call code(args...) is valid:
+  template <class F, class... Args // This constructor is enabled only if calling code(args...) is valid
 #ifndef DOXYGEN /* breathe seem to choke on function signatures in template parameter, see breathe#611 */
-            typename = typename std::result_of_t<F(Args...)>
+            ,
+            typename = typename std::invoke_result_t<F, Args...>
 #endif
             >
-  static ActorPtr create(const std::string& name, s4u::Host* host, F code, Args... args)
+  XBT_ATTRIB_DEPRECATED_v401("Please use Engine::add_actor() instead") static ActorPtr
+      create(const std::string& name, s4u::Host* host, F code, Args... args)
   {
     return create(name, host, std::bind(std::move(code), std::move(args)...));
   }
 
-  /** \static
-   *  Create actor from function name and a vector of strings as arguments.
-   *  @verbatim embed:rst:inline See the :ref:`example <s4u_ex_actors_create>`. @endverbatim */
-  static ActorPtr create(const std::string& name, s4u::Host* host, const std::string& function,
-                         std::vector<std::string> args);
+  XBT_ATTRIB_DEPRECATED_v401("Please use Engine::add_actor() instead") static ActorPtr
+      create(const std::string& name, s4u::Host* host, const std::string& function, std::vector<std::string> args);
 
   // ***** Methods *****
   /** This actor will be automatically terminated when the last non-daemon actor finishes.
@@ -361,7 +355,7 @@ public:
   /** Retrieves the name of that actor as a C string */
   const char* get_cname() const;
   /** Retrieves the host on which that actor is running */
-  Host* get_host() const;
+  s4u::Host* get_host() const;
   /** Retrieves the actor ID of that actor */
   aid_t get_pid() const;
   /** Retrieves the actor ID of that actor's creator */
@@ -418,7 +412,7 @@ public:
    * Asynchronous activities started by the actor are not migrated automatically, so you have
    * to take care of this yourself (only you knows which ones should be migrated).
    */
-  void set_host(Host* new_host);
+  void set_host(s4u::Host* new_host);
 
   /** Ask the actor to die.
    *
@@ -466,7 +460,6 @@ public:
   /** Set a property (old values will be overwritten) */
   void set_property(const std::string& key, const std::string& value);
 };
-
 }} // namespace simgrid::s4u
 
 
