@@ -1,4 +1,4 @@
-/* Copyright (c) 2006-2023. The SimGrid Team. All rights reserved.          */
+/* Copyright (c) 2006-2025. The SimGrid Team. All rights reserved.          */
 
 /* This program is free software; you can redistribute it and/or modify it
  * under the terms of the license (GNU LGPL) which comes with this package. */
@@ -31,9 +31,6 @@ class XBT_PUBLIC Comm : public Activity_T<Comm> {
   size_t src_buff_size_               = sizeof(void*);
 
   /* FIXME: expose these elements in the API */
-  bool detached_                                                          = false;
-  std::function<bool(void*, void*, kernel::activity::CommImpl*)> match_fun_;
-  std::function<void(void*)> clean_fun_;
   std::function<void(kernel::activity::CommImpl*, void*, size_t)> copy_data_function_;
 
   Comm() = default;
@@ -56,8 +53,8 @@ protected:
      */
   }
   /* These ensure that the on_completion signals are really thrown */
-  void fire_on_completion_for_real() const { Activity_T<Comm>::fire_on_completion(); }
-  void fire_on_this_completion_for_real() const { Activity_T<Comm>::fire_on_this_completion(); }
+  void fire_on_completion_for_real() const;
+  void fire_on_this_completion_for_real() const;
 
 public:
   /*! \static Add a callback fired when the send of any Comm is posted  */
@@ -70,19 +67,15 @@ public:
   void on_this_recv_cb(const std::function<void(Comm const&)>& cb) { on_this_recv.connect(cb); }
 
   CommPtr set_copy_data_callback(const std::function<void(kernel::activity::CommImpl*, void*, size_t)>& callback);
-  XBT_ATTRIB_DEPRECATED_v338("Please manifest if you actually need this function") static void copy_buffer_callback(
-      kernel::activity::CommImpl*, void*, size_t);
-  XBT_ATTRIB_DEPRECATED_v338("Please manifest if you actually need this function") static void copy_pointer_callback(
-      kernel::activity::CommImpl*, void*, size_t);
 
   ~Comm() override;
 
-  static void send(kernel::actor::ActorImpl* sender, const Mailbox* mbox, double task_size, double rate, void* src_buff,
-                   size_t src_buff_size,
+  static void send(kernel::actor::ActorImpl* sender, const s4u::Mailbox* mbox, double task_size, double rate,
+                   void* src_buff, size_t src_buff_size,
                    const std::function<bool(void*, void*, simgrid::kernel::activity::CommImpl*)>& match_fun,
                    const std::function<void(simgrid::kernel::activity::CommImpl*, void*, size_t)>& copy_data_fun,
                    void* data, double timeout);
-  static void recv(kernel::actor::ActorImpl* receiver, const Mailbox* mbox, void* dst_buff, size_t* dst_buff_size,
+  static void recv(kernel::actor::ActorImpl* receiver, const s4u::Mailbox* mbox, void* dst_buff, size_t* dst_buff_size,
                    const std::function<bool(void*, void*, simgrid::kernel::activity::CommImpl*)>& match_fun,
                    const std::function<void(simgrid::kernel::activity::CommImpl*, void*, size_t)>& copy_data_fun,
                    void* data, double timeout, double rate);
@@ -94,19 +87,19 @@ public:
    * involved hosts! Enjoy the comfort of the simulator :)
    */
   static CommPtr sendto_init(); /* Source and Destination hosts have to be set before the communication can start */
-  static CommPtr sendto_init(Host* from, Host* to);
-  static CommPtr sendto_async(Host* from, Host* to, uint64_t simulated_size_in_bytes);
-  static void sendto(Host* from, Host* to, uint64_t simulated_size_in_bytes);
+  static CommPtr sendto_init(s4u::Host* from, s4u::Host* to);
+  static CommPtr sendto_async(s4u::Host* from, s4u::Host* to, uint64_t simulated_size_in_bytes);
+  static void sendto(s4u::Host* from, s4u::Host* to, uint64_t simulated_size_in_bytes);
 
-  CommPtr set_source(Host* from);
-  Host* get_source() const;
-  CommPtr set_destination(Host* to);
-  Host* get_destination() const;
+  CommPtr set_source(s4u::Host* from);
+  s4u::Host* get_source() const;
+  CommPtr set_destination(s4u::Host* to);
+  s4u::Host* get_destination() const;
 
   /* Mailbox-based communications */
-  CommPtr set_mailbox(Mailbox* mailbox);
-  /** Retrieve the mailbox on which this comm acts */
-  Mailbox* get_mailbox() const { return mailbox_; }
+  CommPtr set_mailbox(s4u::Mailbox* mailbox);
+  /** Retrieve the mailbox on which this comm acts. This is nullptr if the comm was created through sendto() */
+  s4u::Mailbox* get_mailbox() const { return mailbox_; }
 
   /** Specify the data to send.
    *
@@ -152,7 +145,7 @@ public:
   /** Retrieve the size of the received data. Not to be mixed with @ref Activity::get_remaining()  */
   size_t get_dst_data_size() const { return dst_buff_size_; }
   /** Retrieve the payload associated to the communication. You can only do that once the comm is (gracefully)
-   * terminated, and it is only setup by the default copy_data callback (not the SMPI one) */
+   * terminated. */
   void* get_payload() const;
 
   /* Common functions */
@@ -173,27 +166,18 @@ public:
   Actor* get_receiver() const;
 
   /* Comm life cycle */
-  /** Start the comm, and ignore its result. It can be completely forgotten after that. */
-  Comm* detach();
-  /** Start the comm, and ignore its result. It can be completely forgotten after that. */
-  Comm* detach(const std::function<void(void*)>& clean_function)
-  {
-    clean_fun_ = clean_function;
-    return detach();
-  }
-
   Comm* wait_for(double timeout) override;
 
 #ifndef DOXYGEN
-  XBT_ATTRIB_DEPRECATED_v339("Please use ActivitySet instead") static ssize_t wait_any(const std::vector<CommPtr>& comms) { return deprecated_wait_any_for(comms, -1); }
-  XBT_ATTRIB_DEPRECATED_v339("Please use ActivitySet instead") static ssize_t wait_any_for(const std::vector<CommPtr>& comms, double timeout) { return deprecated_wait_any_for(comms, timeout); }
+  XBT_ATTRIB_DEPRECATED_v401("Please use ActivitySet instead") static ssize_t wait_any(const std::vector<CommPtr>& comms) { return deprecated_wait_any_for(comms, -1); }
+  XBT_ATTRIB_DEPRECATED_v401("Please use ActivitySet instead") static ssize_t wait_any_for(const std::vector<CommPtr>& comms, double timeout) { return deprecated_wait_any_for(comms, timeout); }
 
   static ssize_t deprecated_wait_any_for(const std::vector<CommPtr>& comms,
-                                         double timeout); // XBT_ATTRIB_DEPRECATED_v339
+                                         double timeout); // XBT_ATTRIB_DEPRECATED_v401
 
-  XBT_ATTRIB_DEPRECATED_v339("Please use ActivitySet instead") static ssize_t test_any(const std::vector<CommPtr>& comms);
-  XBT_ATTRIB_DEPRECATED_v339("Please use ActivitySet instead") static void wait_all(const std::vector<CommPtr>& comms);
-  XBT_ATTRIB_DEPRECATED_v339("Please use ActivitySet instead") static size_t
+  XBT_ATTRIB_DEPRECATED_v401("Please use ActivitySet instead") static ssize_t test_any(const std::vector<CommPtr>& comms);
+  XBT_ATTRIB_DEPRECATED_v401("Please use ActivitySet instead") static void wait_all(const std::vector<CommPtr>& comms);
+  XBT_ATTRIB_DEPRECATED_v401("Please use ActivitySet instead") static size_t
       wait_all_for(const std::vector<CommPtr>& comms, double timeout);
 #endif
 };
