@@ -1,4 +1,4 @@
-/* Copyright (c) 2019-2023. The SimGrid Team. All rights reserved.          */
+/* Copyright (c) 2019-2025. The SimGrid Team. All rights reserved.          */
 
 /* This program is free software; you can redistribute it and/or modify it
  * under the terms of the license (GNU LGPL) which comes with this package. */
@@ -7,6 +7,7 @@
 #define INCLUDE_SIMGRID_S4U_DISK_HPP_
 
 #include <simgrid/disk.h>
+#include <simgrid/kernel/resource/Action.hpp>
 #include <simgrid/forward.h>
 #include <simgrid/s4u/Io.hpp>
 #include <xbt/Extendable.hpp>
@@ -30,6 +31,7 @@ namespace s4u {
  */
 
 class XBT_PUBLIC Disk : public xbt::Extendable<Disk> {
+
 #ifndef DOXYGEN
   friend Engine;
   friend Io;
@@ -42,6 +44,11 @@ class XBT_PUBLIC Disk : public xbt::Extendable<Disk> {
   // The private implementation, that never changes
   kernel::resource::DiskImpl* const pimpl_;
 
+protected:
+#ifndef DOXYGEN
+  friend kernel::resource::DiskAction; // signal on_io_state_change
+#endif
+
 public:
 #ifndef DOXYGEN
   kernel::resource::DiskImpl* get_impl() const { return pimpl_; }
@@ -68,13 +75,21 @@ public:
    */
   Disk* set_readwrite_bandwidth(double bw);
 
+  /** Turns that disk on if it was previously off. This call does nothing if the disk is already on. */
+  void turn_on();
+  /** Turns that disk off. All I/O activites are forcefully stopped. */
+  void turn_off();
+  /** Returns if that disk is currently up and running */
+  bool is_on() const;
+
+
   const std::unordered_map<std::string, std::string>* get_properties() const;
   const char* get_property(const std::string& key) const;
   Disk* set_property(const std::string&, const std::string& value);
   Disk* set_properties(const std::unordered_map<std::string, std::string>& properties);
 
-  Disk* set_host(Host* host);
-  Host* get_host() const;
+  Disk* set_host(s4u::Host* host);
+  s4u::Host* get_host() const;
 
   Disk* set_state_profile(kernel::profile::Profile* profile);
   Disk* set_read_bandwidth_profile(kernel::profile::Profile* profile);
@@ -83,6 +98,8 @@ public:
    * @brief Set the max amount of operations (either read or write) that can take place on this disk at the same time
    *
    * Use -1 to set no limit.
+   *
+   * @param limit  Number of concurrent I/O operations
    */
   Disk* set_concurrency_limit(int limit);
   int get_concurrency_limit() const;
@@ -135,10 +152,6 @@ public:
   /* The signals */
   /** @brief \static Add a callback fired when a new Disk is created */
   static void on_creation_cb(const std::function<void(Disk&)>& cb) { on_creation.connect(cb); }
-  /** @brief \static Add a callback fired when any Disk is destroyed */
-  static void on_destruction_cb(const std::function<void(Disk const&)>& cb) { on_destruction.connect(cb); }
-  /** @brief Add a callback fired when this specific Disk is destroyed */
-  void on_this_destruction_cb(const std::function<void(Disk const&)>& cb) { on_this_destruction.connect(cb); }
   /** @brief \static Add a callback fired when any Disk is turned on or off */
   static void on_onoff_cb(const std::function<void(Disk const&)>& cb)
   {
@@ -149,19 +162,51 @@ public:
   {
     on_this_onoff.connect(cb);
   }
-
-  XBT_ATTRIB_DEPRECATED_v338("Please use on_onoff_cb() instead") static void on_state_change_cb(
-      const std::function<void(Disk const&)>& cb)
+  /** \static @brief Add a callback fired when the read bandwidth of any Disk changes */
+  static void on_read_bandwidth_change_cb(const std::function<void(Disk const&)>& cb)
   {
-    on_onoff.connect(cb);
+    on_read_bandwidth_change.connect(cb);
+  }
+  /** @brief Add a callback fired when the read bandwidth of this specific Disk changes */
+  void on_this_read_bandwidth_change_cb(const std::function<void(Disk const&)>& cb)
+  {
+    on_this_read_bandwidth_change.connect(cb);
+  }
+  /** \static @brief Add a callback fired when the write bandwidth of any Disk changes */
+  static void on_write_bandwidth_change_cb(const std::function<void(Disk const&)>& cb)
+  {
+    on_write_bandwidth_change.connect(cb);
+  }
+  /** @brief Add a callback fired when the read bandwidth of this specific Disk changes */
+  void on_this_write_bandwidth_change_cb(const std::function<void(Disk const&)>& cb)
+  {
+    on_this_write_bandwidth_change.connect(cb);
+  }
+  /** \static @brief Add a callback fired when an I/O changes it state (ready/done/cancel) */
+  static void on_io_state_change_cb(
+      const std::function<void(kernel::resource::DiskAction&, kernel::resource::Action::State)>& cb)
+  {
+    on_io_state_change.connect(cb);
   }
 
+  /** @brief \static Add a callback fired when any Disk is destroyed */
+  static void on_destruction_cb(const std::function<void(Disk const&)>& cb) { on_destruction.connect(cb); }
+  /** @brief Add a callback fired when this specific Disk is destroyed */
+  void on_this_destruction_cb(const std::function<void(Disk const&)>& cb) { on_this_destruction.connect(cb); }
+
 private:
+#ifndef DOXYGEN
   static xbt::signal<void(Disk&)> on_creation;
-  static xbt::signal<void(Disk const&)> on_destruction;
-  xbt::signal<void(Disk const&)> on_this_destruction;
   static xbt::signal<void(Disk const&)> on_onoff;
   xbt::signal<void(Disk const&)> on_this_onoff;
+  static xbt::signal<void(Disk const&)> on_read_bandwidth_change;
+  xbt::signal<void(Disk const&)> on_this_read_bandwidth_change;
+  static xbt::signal<void(Disk const&)> on_write_bandwidth_change;
+  xbt::signal<void(Disk const&)> on_this_write_bandwidth_change;
+  static xbt::signal<void(kernel::resource::DiskAction&, kernel::resource::Action::State)> on_io_state_change;
+  static xbt::signal<void(Disk const&)> on_destruction;
+  xbt::signal<void(Disk const&)> on_this_destruction;
+#endif
 };
 
 } // namespace s4u
