@@ -1,4 +1,4 @@
-/* Copyright (c) 2017-2023. The SimGrid Team. All rights reserved.          */
+/* Copyright (c) 2017-2025. The SimGrid Team. All rights reserved.          */
 
 /* This program is free software; you can redistribute it and/or modify it
  * under the terms of the license (GNU LGPL) which comes with this package. */
@@ -53,9 +53,15 @@ public:
   /** @brief On sequential executions, returns the amount of flops that remain to be done; This cannot be used on
    * parallel executions. */
   double get_remaining() const override;
+  /** @brief Returns the ratio of elements that are still to do
+   *
+   * The returned value is between 0 (completely done) and 1 (nothing done yet). */
   double get_remaining_ratio() const;
-  ExecPtr set_host(Host* host);
-  ExecPtr set_hosts(const std::vector<Host*>& hosts);
+  /** @brief Change the host on which this activity takes place.
+   *
+   * This cannot be done once the activity is terminated, but it can be done on started executions. */
+  ExecPtr set_host(s4u::Host* host);
+  ExecPtr set_hosts(const std::vector<s4u::Host*>& hosts);
   ExecPtr unset_host();
   ExecPtr unset_hosts() { return unset_host(); }
 
@@ -65,11 +71,22 @@ public:
 
   ExecPtr set_thread_count(int thread_count);
 
+  /** @brief change the execution bound
+   * This means changing the maximal amount of flops per second that it may consume, regardless of what the host may
+   * deliver. Currently, this cannot be changed once the exec started. See also the "cloud-capping" example.  */
   ExecPtr set_bound(double bound);
+
+  /** @brief  Change the execution priority, don't you think?
+   * An execution with twice the priority will get twice the amount of flops when the resource is shared.
+   * The default priority is 1.
+   *
+   * Currently, this cannot be changed once the exec started. */
   ExecPtr set_priority(double priority);
   ExecPtr update_priority(double priority);
 
-  Host* get_host() const;
+  /** @brief Retrieve the host on which this activity takes place.
+   *  If it runs on more than one host, only the first host is returned. */
+  s4u::Host* get_host() const;
   unsigned int get_host_number() const;
   int get_thread_count() const;
   double get_cost() const;
@@ -77,11 +94,11 @@ public:
   bool is_assigned() const override;
 
 #ifndef DOXYGEN
-  static ssize_t deprecated_wait_any_for(const std::vector<ExecPtr>& execs, double timeout); // XBT_ATTRIB_DEPRECATED_v339
+  static ssize_t deprecated_wait_any_for(const std::vector<ExecPtr>& execs, double timeout); // XBT_ATTRIB_DEPRECATED_v401
 
-  XBT_ATTRIB_DEPRECATED_v339("Please use ActivitySet instead") static ssize_t
+  XBT_ATTRIB_DEPRECATED_v401("Please use ActivitySet instead") static ssize_t
       wait_any(const std::vector<ExecPtr>& execs) { return deprecated_wait_any_for(execs, -1); }
-  XBT_ATTRIB_DEPRECATED_v339("Please use ActivitySet instead") static ssize_t
+  XBT_ATTRIB_DEPRECATED_v401("Please use ActivitySet instead") static ssize_t
       wait_any_for(const std::vector<ExecPtr>& execs, double timeout) { return deprecated_wait_any_for(execs, timeout); }
 #endif
 };
