@@ -1,4 +1,4 @@
-/* Copyright (c) 2006-2023. The SimGrid Team. All rights reserved.          */
+/* Copyright (c) 2006-2025. The SimGrid Team. All rights reserved.          */
 
 /* This program is free software; you can redistribute it and/or modify it
  * under the terms of the license (GNU LGPL) which comes with this package. */
@@ -36,20 +36,19 @@ namespace s4u {
  * @endrst
  */
 class XBT_PUBLIC Host : public xbt::Extendable<Host> {
+
 #ifndef DOXYGEN
   friend kernel::resource::VMModel;            // Use the pimpl_cpu to compute the VM sharing
   friend kernel::resource::VirtualMachineImpl; // creates the the pimpl_cpu
   friend kernel::routing::NetZoneImpl;
   friend kernel::resource::HostImpl; // call destructor from private implementation
+  friend kernel::resource::CpuAction; // signal exec_state_changed
 
   // The private implementation, that never changes
   kernel::resource::HostImpl* const pimpl_;
 
   kernel::resource::CpuImpl* pimpl_cpu_      = nullptr;
   kernel::routing::NetPoint* pimpl_netpoint_ = nullptr;
-#ifndef DOXYGEN
-  friend kernel::resource::CpuAction; // signal exec_state_changed
-#endif
 
   static xbt::signal<void(Host&)> on_creation;
   static xbt::signal<void(Host const&)> on_destruction;
@@ -68,20 +67,12 @@ public:
   xbt::signal<void(Host const&)> on_this_speed_change;
   static xbt::signal<void(Host const&)> on_onoff;
   xbt::signal<void(Host const&)> on_this_onoff;
-
 #endif
+
   /** \static Add a callback fired on each newly created host */
   static void on_creation_cb(const std::function<void(Host&)>& cb) { on_creation.connect(cb); }
   /** \static Add a callback fired when any machine is turned on or off (called AFTER the change) */
-  static void on_onoff_cb(const std::function<void(Host const&)>& cb)
-  {
-    on_onoff.connect(cb);
-  }
-  XBT_ATTRIB_DEPRECATED_v338("Please use on_onoff_cb() instead") static void on_state_change_cb(
-      const std::function<void(Host const&)>& cb)
-  {
-    on_onoff.connect(cb);
-  }
+  static void on_onoff_cb(const std::function<void(Host const&)>& cb) { on_onoff.connect(cb); }
   /** Add a callback fired when this specific machine is turned on or off (called AFTER the change) */
   void on_this_onoff_cb(const std::function<void(Host const&)>& cb)
   {
@@ -132,19 +123,18 @@ public:
   Host* set_cpu(kernel::resource::CpuImpl* cpu);
   kernel::resource::CpuImpl* get_cpu() const { return pimpl_cpu_; }
   kernel::routing::NetPoint* get_netpoint() const { return pimpl_netpoint_; }
-  /**
-   * @brief Callback to set CPU factor
+
+  /** Configure the factor callback to the CPU associated to this host
    *
-   * This callback offers a flexible way to create variability in CPU executions
-   *
-   * @param flops Execution size in flops
-   * @return Multiply factor
+   * This callback takes the execution size in flops and returns the multiply factor
    */
+  Host* set_cpu_factor_cb(const std::function<double(Host&, double)>& cb);
+
+#ifndef DOXYGEN
   using CpuFactorCb = double(double flops);
-  /**
-   * @brief Configure the factor callback to the CPU associated to this host
-   */
-  Host* set_factor_cb(const std::function<CpuFactorCb>& cb);
+  XBT_ATTRIB_DEPRECATED_v401("Please use set_cpu_factor_cb() instead") Host* set_factor_cb(
+      const std::function<CpuFactorCb>& cb);
+#endif
 
   size_t get_actor_count() const;
   std::vector<ActorPtr> get_all_actors() const;
@@ -176,6 +166,8 @@ public:
    * @brief Set the max amount of executions that can take place on this host at the same time
    *
    * Use -1 to set no limit.
+   *
+   * @param limit  Number of concurrent executions
    */
   Host* set_concurrency_limit(int limit);
   int get_concurrency_limit() const;
@@ -254,21 +246,32 @@ public:
   Host* set_coordinates(const std::string& coords);
 
   std::vector<Disk*> get_disks() const;
-  /**
-   * @brief Create and add disk in the host
+  Disk* get_disk_by_name(const std::string& name) const;
+
+  #ifndef DOXYGEN
+   XBT_ATTRIB_DEPRECATED_v401("Please use Host::add_disk")
+   Disk* create_disk(const std::string& name, double read_bandwidth, double write_bandwidth)
+   { return add_disk(name, read_bandwidth, write_bandwidth); }
+   XBT_ATTRIB_DEPRECATED_v401("Please use Host::add_disk")
+   Disk* create_disk(const std::string& name, const std::string& read_bandwidth, const std::string& write_bandwidth)
+   { return add_disk(name, read_bandwidth, write_bandwidth); }
+  #endif 
+
+   /**
+   * @brief Add a disk to a host
    *
    * @param name Disk name
    * @param read_bandwidth Reading speed of the disk
    * @param write_bandwidth Writing speed of the disk
    */
-  Disk* create_disk(const std::string& name, double read_bandwidth, double write_bandwidth);
+  Disk* add_disk(const std::string& name, double read_bandwidth, double write_bandwidth);
   /**
-   * @brief Human-friendly version of create_disk function.
+   * @brief Human-friendly version of add_disk function.
    *
    * @throw std::invalid_argument if read/write speeds are incorrect
    */
-  Disk* create_disk(const std::string& name, const std::string& read_bandwidth, const std::string& write_bandwidth);
-  void add_disk(const Disk* disk);
+  Disk* add_disk(const std::string& name, const std::string& read_bandwidth, const std::string& write_bandwidth);
+  void register_disk(const Disk* disk);
   void remove_disk(const std::string& disk_name);
 
   VirtualMachine* create_vm(const std::string& name, int core_amount);
