@@ -1,4 +1,4 @@
-/* Copyright (c) 2004-2023. The SimGrid Team. All rights reserved.          */
+/* Copyright (c) 2004-2025. The SimGrid Team. All rights reserved.          */
 
 /* This program is free software; you can redistribute it and/or modify it
  * under the terms of the license (GNU LGPL) which comes with this package. */
@@ -30,6 +30,7 @@ namespace s4u {
  * @endrst
  */
 class XBT_PUBLIC Link : public xbt::Extendable<Link> {
+
 #ifndef DOXYGEN
   friend kernel::resource::StandardLinkImpl;
 #endif
@@ -41,7 +42,7 @@ protected:
   // The implementation that never changes
   kernel::resource::LinkImpl* const pimpl_;
 #ifndef DOXYGEN
-  friend kernel::resource::NetworkAction; // signal comm_state_changed
+  friend kernel::resource::NetworkAction; // signal on_communication_state_change
 #endif
 
 public:
@@ -146,10 +147,6 @@ public:
   /** @brief Returns the current load (in bytes per second) */
   double get_load() const;
 
-#ifndef DOXYGEN
-  XBT_ATTRIB_DEPRECATED_v338("Please use get_load() instead") double get_usage() const { return get_load(); }
-#endif
-
   /** @brief Check if the Link is used (at least one flow uses the link) */
   bool is_used() const;
 
@@ -208,16 +205,7 @@ public:
   /** \static @brief Add a callback fired when any Link is destroyed */
   static void on_destruction_cb(const std::function<void(Link const&)>& cb) { on_destruction.connect(cb); }
   /** @brief Add a callback fired when this specific Link is destroyed */
-  void on_this_destruction_cb(const std::function<void(Link const&)>& cb)
-  {
-    on_this_destruction.connect(cb);
-  }
-
-  XBT_ATTRIB_DEPRECATED_v338("Please use on_onoff_cb() instead") static void on_state_change_cb(
-      const std::function<void(Link const&)>& cb)
-  {
-    on_onoff.connect(cb);
-  }
+  void on_this_destruction_cb(const std::function<void(Link const&)>& cb) { on_this_destruction.connect(cb); }
 };
 
 /**
