@@ -1,4 +1,4 @@
-/* Copyright (c) 2015-2023. The SimGrid Team. All rights reserved.          */
+/* Copyright (c) 2015-2025. The SimGrid Team. All rights reserved.          */
 
 /* This program is free software; you can redistribute it and/or modify it
  * under the terms of the license (GNU LGPL) which comes with this package. */
@@ -31,6 +31,7 @@ public:
  *
  */
 class XBT_PUBLIC VirtualMachine : public s4u::Host {
+
   kernel::resource::VirtualMachineImpl* const pimpl_vm_;
 
   /* Signals about the life cycle of the VM */
@@ -59,11 +60,6 @@ class XBT_PUBLIC VirtualMachine : public s4u::Host {
 #endif
 
 public:
-  XBT_ATTRIB_DEPRECATED_v336("Please use s4u::Host::create_vm") explicit VirtualMachine(const std::string& name,
-                                                                                        Host* physical_host,
-                                                                                        int core_amount,
-                                                                                        size_t ramsize = 1024);
-
 #ifndef DOXYGEN
   // No copy/move
   VirtualMachine(VirtualMachine const&) = delete;
