@@ -1,4 +1,4 @@
-/* Copyright (c) 2010-2023. The SimGrid Team. All rights reserved.          */
+/* Copyright (c) 2010-2025. The SimGrid Team. All rights reserved.          */
 
 /* This program is free software; you can redistribute it and/or modify it
  * under the terms of the license (GNU LGPL) which comes with this package. */
@@ -17,6 +17,7 @@
 #include "src/kernel/activity/ExecImpl.hpp"
 #include "src/kernel/resource/CpuImpl.hpp"
 #include "src/kernel/resource/NetworkModel.hpp"
+#include "xbt/log.h"
 
 #include <fstream>
 
@@ -278,21 +279,26 @@ static void on_netzone_creation(s4u::NetZone const& netzone)
     auto* root = new NetZoneContainer(id, 0, nullptr);
     xbt_assert(Container::get_root() == root);
 
-    if (TRACE_smpi_is_enabled()) {
-      auto* mpi = root->get_type()->by_name_or_create<ContainerType>("MPI");
-      if (not TRACE_smpi_is_grouped())
-        mpi->by_name_or_create<StateType>("MPI_STATE");
-      root->get_type()->by_name_or_create("MPI_LINK", mpi, mpi);
-      root->get_type()->by_name_or_create("MIGRATE_LINK", mpi, mpi);
-      mpi->by_name_or_create<StateType>("MIGRATE_STATE");
-    }
-
     if (TRACE_needs_platform()) {
       currentContainer.push_back(root);
     }
     return;
   }
 
+  // This static is ugly, but I fail to make it work otherwise
+  static bool inited = false;
+  if (TRACE_smpi_is_enabled() && not inited) {
+    inited = true;
+
+    auto root = Container::get_root();
+    auto* mpi = root->get_type()->by_name_or_create<ContainerType>("MPI");
+    if (not TRACE_smpi_is_grouped())
+      mpi->by_name_or_create<StateType>("MPI_STATE");
+    root->get_type()->by_name_or_create("MPI_LINK", mpi, mpi);
+    root->get_type()->by_name_or_create("MIGRATE_LINK", mpi, mpi);
+    mpi->by_name_or_create<StateType>("MIGRATE_STATE");
+  }
+
   if (TRACE_needs_platform()) {
     auto level      = static_cast<unsigned>(currentContainer.size());
     auto* container = new NetZoneContainer(id, level, currentContainer.back());
@@ -302,7 +308,8 @@ static void on_netzone_creation(s4u::NetZone const& netzone)
 
 static void on_link_creation(s4u::Link const& link)
 {
-  if (currentContainer.empty()) // No ongoing parsing. Are you creating the loopback?
+  if (currentContainer.empty() ||
+      link.get_name() == "__loopback__") // No ongoing parsing. Are you creating the loopback?
     return;
 
   auto* container = new Container(link.get_name(), "LINK", currentContainer.back());
@@ -383,7 +390,7 @@ static void on_activity_suspend_resume(s4u::Activity const& activity)
   on_action_state_change(*activity.get_impl()->model_action_, /*ignored*/ kernel::resource::Action::State::STARTED);
 }
 
-static void on_platform_created()
+static void on_platform_sealed()
 {
   currentContainer.clear();
   std::set<std::string, std::less<>> filter;
@@ -453,7 +460,7 @@ void define_callbacks()
   // always need the callbacks to zones (we need only the root zone), to create the rootContainer and the rootType
   // properly
   if (TRACE_needs_platform()) {
-    s4u::Engine::on_platform_created_cb(on_platform_created);
+    s4u::Engine::on_platform_sealed_cb(on_platform_sealed);
     s4u::Host::on_creation_cb(on_host_creation);
     s4u::Host::on_speed_change_cb([](s4u::Host const& host) {
       Container::by_name(host.get_name())
@@ -476,6 +483,9 @@ void define_callbacks()
   }
 
   s4u::NetZone::on_creation_cb(on_netzone_creation);
+  // The root netzone is created earlier when we are parsing an XML file, so handle it now
+  for (auto nz : s4u::Engine::get_instance()->get_all_netzones())
+    on_netzone_creation(*nz);
 
   s4u::Host::on_exec_state_change_cb(on_action_state_change);
   s4u::Link::on_communication_state_change_cb(on_action_state_change);
@@ -519,7 +529,7 @@ void define_callbacks()
       if (c.get_sender()) {
         Container::by_name(instr_pid(*c.get_sender()))->get_state("ACTOR_STATE")->pop_event();
         Container::by_name(instr_pid(*c.get_receiver()))->get_state("ACTOR_STATE")->pop_event();
-      } else {
+      } else if(c.get_source()) {
         Container::by_name(c.get_source()->get_name())->get_state("HOST_STATE")->pop_event();
         Container::by_name(c.get_destination()->get_name())->get_state("HOST_STATE")->pop_event();
       }
