@@ -1,15 +1,21 @@
-/* Copyright (c) 2016-2023. The SimGrid Team. All rights reserved.          */
+/* Copyright (c) 2016-2025. The SimGrid Team. All rights reserved.          */
 
 /* This program is free software; you can redistribute it and/or modify it
  * under the terms of the license (GNU LGPL) which comes with this package. */
 
+#include <csignal>
 #include <simgrid/Exception.hpp>
 #include <simgrid/kernel/Timer.hpp>
 #include <simgrid/kernel/routing/NetPoint.hpp>
 #include <simgrid/kernel/routing/NetZoneImpl.hpp>
 #include <simgrid/s4u/Host.hpp>
 
+#include "simgrid/s4u/Engine.hpp"
 #include "src/kernel/EngineImpl.hpp"
+#include "src/kernel/activity/ActivityImpl.hpp"
+#include "src/kernel/activity/BarrierImpl.hpp"
+#include "src/kernel/activity/MutexImpl.hpp"
+#include "src/kernel/activity/SemaphoreImpl.hpp"
 #include "src/kernel/resource/StandardLinkImpl.hpp"
 #include "src/kernel/resource/profile/Profile.hpp"
 #include "src/kernel/xml/platf.hpp"
@@ -20,6 +26,7 @@
 #include "src/simgrid/math_utils.h"
 #include "src/simgrid/sg_config.hpp"
 #include "src/smpi/include/smpi_actor.hpp"
+#include "xbt/log.h"
 
 #if SIMGRID_HAVE_MC
 #include "src/mc/remote/AppSide.hpp"
@@ -29,6 +36,7 @@
 
 #include <boost/algorithm/string/predicate.hpp>
 #include <dlfcn.h>
+#include <string>
 
 XBT_LOG_NEW_DEFAULT_CATEGORY(ker_engine, "Logging specific to Engine (kernel)");
 
@@ -117,8 +125,14 @@ static void segvhandler(int signum, siginfo_t* siginfo, void* /*context*/)
   std::raise(signum);
 }
 
+// Allow java and others to request to not install our signal handlers (Java's ones are better)
+bool do_install_signal_handlers = true;
+
 static void install_signal_handlers()
 {
+  if (not do_install_signal_handlers)
+    return;
+
   /* Install signal handler for SIGINT */
   std::signal(SIGINT, inthandler);
 
@@ -182,6 +196,10 @@ void EngineImpl::initialize(int* argc, char** argv)
   simgrid::mc::AppSide::get(); // To ensure that it's initialized
 #endif
 
+#if HAVE_SMPI
+  smpi_init_options_internal(true);
+#endif
+
   if (static bool inited = false; not inited) {
     inited = true;
     xbt_log_init(argc, argv);
@@ -222,6 +240,11 @@ void EngineImpl::context_mod_init() const
   }
 #endif
 
+  /* select the context factory that will create the contexts */
+  if (context::ContextFactory::initializer) { // Give Java a chance to hijack the factory mechanism
+    instance_->set_context_factory(context::ContextFactory::initializer());
+    return;
+  }
   /* use the factory specified by --cfg=contexts/factory:value */
   for (auto const& [factory_name, factory] : context_factories)
     if (context_factory_name == factory_name) {
@@ -256,13 +279,14 @@ void EngineImpl::shutdown()
 {
   if (EngineImpl::instance_ == nullptr)
     return;
+  s4u::Engine::shutdown_ongoing_ = true;
   XBT_DEBUG("EngineImpl::shutdown() called. Simulation's over.");
 #if HAVE_SMPI
   if (not instance_->actor_list_.empty()) {
     if (smpi_process() && smpi_process()->initialized()) {
       xbt_die("Process exited without calling MPI_Finalize - Killing simulation");
     } else {
-      XBT_WARN("Process called exit when leaving - Skipping cleanups");
+      fprintf(stderr, "Process called exit when leaving - Skipping cleanups\n");
       return;
     }
   }
@@ -297,6 +321,9 @@ void EngineImpl::seal_platform() const
 
   /* seal netzone root, recursively seal children netzones, hosts and disks */
   netzone_root_->seal();
+
+  // Now the platform is officially sealed and thus completely created
+  simgrid::s4u::Engine::on_platform_sealed();
 }
 
 void EngineImpl::load_platform(const std::string& platf)
@@ -441,44 +468,65 @@ void EngineImpl::empty_trash()
   }
 }
 
+static const char* activity_type_name(activity::ActivityImpl* acti)
+{
+  if (dynamic_cast<kernel::activity::ExecImpl*>(acti) != nullptr)
+    return "an execution";
+
+  if (dynamic_cast<kernel::activity::CommImpl*>(acti) != nullptr)
+    return "a communication";
+
+  if (dynamic_cast<kernel::activity::MessImpl*>(acti) != nullptr)
+    return "a message";
+
+  if (dynamic_cast<kernel::activity::SleepImpl*>(acti) != nullptr)
+    return "a sleeping";
+
+  if (dynamic_cast<kernel::activity::IoImpl*>(acti) != nullptr)
+    return "an I/O";
+
+  if (dynamic_cast<kernel::activity::SemAcquisitionImpl*>(acti) != nullptr)
+    return "a semaphore acquisition";
+
+  if (dynamic_cast<kernel::activity::MutexAcquisitionImpl*>(acti) != nullptr)
+    return "a mutex lock";
+
+  if (dynamic_cast<kernel::activity::BarrierAcquisitionImpl*>(acti) != nullptr)
+    return "a barrier acquisition";
+
+  return "an unknown";
+}
+
 void EngineImpl::display_all_actor_status() const
 {
-  XBT_INFO("%zu actors are still running, waiting for something.", actor_list_.size());
+  if (actor_list_.size() == 1)
+    XBT_INFO("1 actor is still active, awaiting something. Here is its status:");
+  else
+    XBT_INFO("%zu actors are still active, awaiting something. Here is their status:", actor_list_.size());
   /*  List the actors and their state */
-  XBT_INFO("Legend of the following listing: \"Actor <pid> (<name>@<host>): <status>\"");
   for (auto const& [_, actor] : actor_list_) {
-    if (actor->waiting_synchro_) {
-      const char* synchro_description = "unknown";
-
-      if (boost::dynamic_pointer_cast<kernel::activity::ExecImpl>(actor->waiting_synchro_) != nullptr)
-        synchro_description = "execution";
-
-      if (boost::dynamic_pointer_cast<kernel::activity::CommImpl>(actor->waiting_synchro_) != nullptr)
-        synchro_description = "communication";
-
-      if (boost::dynamic_pointer_cast<kernel::activity::MessImpl>(actor->waiting_synchro_) != nullptr)
-        synchro_description = "message";
-
-      if (boost::dynamic_pointer_cast<kernel::activity::SleepImpl>(actor->waiting_synchro_) != nullptr)
-        synchro_description = "sleeping";
-
-      if (boost::dynamic_pointer_cast<kernel::activity::SynchroImpl>(actor->waiting_synchro_) != nullptr)
-        synchro_description = "synchronization";
-
-      if (boost::dynamic_pointer_cast<kernel::activity::IoImpl>(actor->waiting_synchro_) != nullptr)
-        synchro_description = "I/O";
-
-      XBT_INFO("Actor %ld (%s@%s): waiting for %s activity %#zx (%s) in state %s to finish %s", actor->get_pid(),
-               actor->get_cname(), actor->get_host()->get_cname(), synchro_description,
-               (xbt_log_no_loc ? (size_t)0xDEADBEEF : (size_t)actor->waiting_synchro_.get()),
-               actor->waiting_synchro_->get_cname(), actor->waiting_synchro_->get_state_str(),
+    if (actor->waiting_synchros_.empty()) {
+      XBT_INFO(" - pid %ld (%s@%s) simcall %s", actor->get_pid(), actor->get_cname(), actor->get_host()->get_cname(),
+               (actor->simcall_.observer_ != nullptr ? actor->simcall_.observer_->to_string().c_str()
+                                                     : actor->simcall_.get_cname()));
+    } else if (actor->waiting_synchros_.size() == 1) {
+      auto synchro = actor->waiting_synchros_[0];
+      XBT_INFO(" - pid %ld (%s@%s): waiting for %s activity %#zx (%s) in state %s", actor->get_pid(),
+               actor->get_cname(), actor->get_host()->get_cname(), activity_type_name(synchro.get()),
+               (xbt_log_no_loc ? (size_t)0xDEADBEEF : (size_t)synchro.get()), synchro->get_cname(),
+               synchro->get_state_str());
+    } else {
+      XBT_INFO(" - pid %ld (%s@%s): waiting for %zu activit%s to finish %s", actor->get_pid(), actor->get_cname(),
+               actor->get_host()->get_cname(), actor->waiting_synchros_.size(),
+               (actor->waiting_synchros_.size() == 1 ? "y" : "ies"),
                (actor->simcall_.observer_ != nullptr && not xbt_log_no_loc
                     ? actor->simcall_.observer_->to_string().c_str()
                     : ""));
-    } else {
-      XBT_INFO("Actor %ld (%s@%s) simcall %s", actor->get_pid(), actor->get_cname(), actor->get_host()->get_cname(),
-               (actor->simcall_.observer_ != nullptr ? actor->simcall_.observer_->to_string().c_str()
-                                                     : actor->simcall_.get_cname()));
+      for (auto const& synchro : actor->waiting_synchros_) {
+        XBT_INFO("     - %s activity %#zx (%s) in state %s", activity_type_name(synchro.get()),
+                 (xbt_log_no_loc ? (size_t)0xDEADBEEF : (size_t)synchro.get()), synchro->get_cname(),
+                 synchro->get_state_str());
+      }
     }
   }
 }
