@@ -1,4 +1,4 @@
-/* Copyright (c) 2007-2023. The SimGrid Team. All rights reserved.          */
+/* Copyright (c) 2007-2025. The SimGrid Team. All rights reserved.          */
 
 /* This program is free software; you can redistribute it and/or modify it
  * under the terms of the license (GNU LGPL) which comes with this package. */
@@ -8,7 +8,7 @@
 
 #include "simgrid/forward.h"
 #include "simgrid/kernel/resource/Action.hpp"
-#include "simgrid/simix.hpp"
+#include "simgrid/simcall.hpp"
 #include "src/kernel/actor/Simcall.hpp"
 #include "xbt/utility.hpp"
 
@@ -20,7 +20,7 @@
 namespace simgrid::kernel::activity {
 
 XBT_DECLARE_ENUM_CLASS(State, WAITING, READY, RUNNING, DONE, CANCELED, FAILED, SRC_HOST_FAILURE, DST_HOST_FAILURE,
-                       TIMEOUT, SRC_TIMEOUT, DST_TIMEOUT, LINK_FAILURE);
+                       LINK_FAILURE);
 
 class XBT_PUBLIC ActivityImpl : public kernel::actor::ObjectAccessSimcallItem {
   std::atomic_int_fast32_t refcount_{0};
@@ -37,6 +37,7 @@ public:
   std::list<actor::Simcall*> simcalls_; /* List of simcalls waiting for this activity */
   s4u::Activity* piface_          = nullptr;
   resource::Action* model_action_ = nullptr; /* The resource consumption in the used model */
+  bool detached_                  = false;   /* If detached or not */
 
 protected:
   void inline set_name(std::string_view name)
@@ -49,6 +50,7 @@ protected:
   void clear_hosts() { hosts_.clear(); }
   void add_host(s4u::Host* host) { hosts_.push_back(host); }
   void set_hosts(const std::vector<s4u::Host*>& hosts) { hosts_ = hosts; }
+  void set_detached(bool detached) { detached_ = detached; }
 
 public:
   const std::string& get_name() const { return name_; }
@@ -68,6 +70,9 @@ public:
   void set_finish_time(double finish_time) { finish_time_ = finish_time; }
   double get_finish_time() const { return finish_time_; }
 
+  bool is_detached() const { return detached_; }
+
+  void detach();
   virtual bool test(actor::ActorImpl* issuer);
   static ssize_t test_any(actor::ActorImpl* issuer, const std::vector<ActivityImpl*>& activities);
 
@@ -79,15 +84,19 @@ public:
   virtual void resume();
   virtual void cancel();
 
-  virtual void set_exception(actor::ActorImpl* issuer) = 0; // Raising exceptions and stuff
-  virtual void finish() = 0; // Setups the status, clean things up, unlock all simcalls blocked on that activity.
+  virtual void finish() = 0; // Setups the status, clean things up, unlock all simcalls blocked on that activity and set
+                             // relevant exceptions.
 
   s4u::Host* get_host() const { return hosts_.front(); }
   const std::vector<s4u::Host*>& get_hosts() const { return hosts_; };
 
+  // Connect this activity to that actor
   void register_simcall(actor::Simcall* simcall);
+  // Disconnect this activity from that given actor
   void unregister_simcall(actor::Simcall* simcall);
-  void handle_activity_waitany(actor::Simcall* simcall);
+  // Disconnect this activity from the first actor waiting on it.
+  // Returns the actor to be notified of the activity state.
+  actor::ActorImpl* unregister_first_simcall();
   void clean_action();
   virtual double get_remaining() const;
   // Support for the boost::intrusive_ptr<ActivityImpl> datatype
@@ -115,6 +124,12 @@ public:
     return static_cast<AnyActivityImpl&>(*this);
   }
   const std::string& get_tracing_category() const { return tracing_category_; }
+
+  AnyActivityImpl& detach()
+  {
+    ActivityImpl::detach();
+    return static_cast<AnyActivityImpl&>(*this);
+  }
 };
 
 } // namespace simgrid::kernel::activity
