@@ -1,4 +1,4 @@
-/* Copyright (c) 2012-2023. The SimGrid Team. All rights reserved.          */
+/* Copyright (c) 2012-2025. The SimGrid Team. All rights reserved.          */
 
 /* This program is free software; you can redistribute it and/or modify it
  * under the terms of the license (GNU LGPL) which comes with this package. */
@@ -27,15 +27,12 @@ class XBT_PUBLIC BarrierAcquisitionImpl : public ActivityImpl_T<BarrierAcquisiti
 
 public:
   BarrierAcquisitionImpl(actor::ActorImpl* issuer, BarrierImpl* bar) : issuer_(issuer), barrier_(bar) {}
-  BarrierImplPtr get_barrier() { return barrier_; }
-  actor::ActorImpl* get_issuer() { return issuer_; }
+  BarrierImplPtr get_barrier() const { return barrier_; }
+  actor::ActorImpl* get_issuer() const { return issuer_; }
 
   bool test(actor::ActorImpl* issuer = nullptr) override;
   void wait_for(actor::ActorImpl* issuer, double timeout) override;
   void finish() override;
-  void set_exception(actor::ActorImpl* issuer) override
-  { /* nothing to do */
-  }
 };
 
 class XBT_PUBLIC BarrierImpl {
@@ -56,6 +53,7 @@ public:
 
   BarrierAcquisitionImplPtr acquire_async(actor::ActorImpl* issuer);
   unsigned get_id() const { return id_; }
+  bool was_last() const { return ongoing_acquisitions_.empty(); }
 
   friend void intrusive_ptr_add_ref(BarrierImpl* barrier)
   {
