@@ -1,4 +1,4 @@
-/* Copyright (c) 2007-2023. The SimGrid Team. All rights reserved.          */
+/* Copyright (c) 2007-2025. The SimGrid Team. All rights reserved.          */
 
 /* This program is free software; you can redistribute it and/or modify it
  * under the terms of the license (GNU LGPL) which comes with this package. */
@@ -16,6 +16,7 @@
 #include "src/kernel/resource/NetworkModel.hpp"
 #include "src/kernel/resource/StandardLinkImpl.hpp"
 #include "src/mc/mc_replay.hpp"
+#include "xbt/asserts.h"
 
 XBT_LOG_NEW_DEFAULT_SUBCATEGORY(ker_network, kernel, "Kernel network-related synchronization");
 
@@ -28,7 +29,6 @@ std::function<void(CommImpl*, void*, size_t)> CommImpl::copy_data_callback_ = []
   xbt_assert((buff_size == sizeof(void*)), "Cannot copy %zu bytes: must be sizeof(void*)", buff_size);
   if (comm->dst_buff_ != nullptr) // get_async provided a buffer
     *(void**)(comm->dst_buff_) = buff;
-  comm->payload_ = buff; // Setup what will be retrieved by s4u::Comm::get_payload()
 };
 
 void CommImpl::set_copy_data_callback(const std::function<void(CommImpl*, void*, size_t)>& callback)
@@ -91,13 +91,6 @@ CommImpl& CommImpl::set_dst_buff(unsigned char* buff, size_t* size)
   return *this;
 }
 
-CommImpl& CommImpl::detach()
-{
-  detached_ = true;
-  EngineImpl::get_instance()->get_maestro()->activities_.insert(this);
-  return *this;
-}
-
 CommImpl::~CommImpl()
 {
   XBT_DEBUG("Really free communication %p in state %s (detached = %d)", this, get_state_str(), detached_);
@@ -136,8 +129,8 @@ CommImpl* CommImpl::start()
     set_start_time(model_action_->get_start_time());
     set_state(State::RUNNING);
 
-    XBT_DEBUG("Starting communication %p from '%s' to '%s' (model action: %p; state: %s)", this, from_->get_cname(),
-              to_->get_cname(), model_action_, get_state_str());
+    XBT_DEBUG("Starting communication %p (s4u:%p) from '%s' to '%s' (model action: %p; state: %s)", this, piface_,
+              from_->get_cname(), to_->get_cname(), model_action_, get_state_str());
 
     /* If a link is failed, detect it immediately */
     if (model_action_->get_state() == resource::Action::State::FAILED) {
@@ -196,6 +189,8 @@ void CommImpl::copy_data()
     *dst_buff_size_ = buff_size;
   }
 
+  payload_ = src_buff_; // Setup what will be retrieved by s4u::Comm::get_payload()
+
   if (buff_size > 0) {
     if (copy_data_fun)
       copy_data_fun(this, src_buff_, buff_size);
@@ -222,7 +217,7 @@ ActivityImplPtr CommImpl::isend(actor::CommIsendSimcall* observer)
    *
    * If it is not found then push our communication into the rendez-vous point */
   CommImplPtr other_comm =
-      mbox->find_matching_comm(CommImplType::RECEIVE, observer->get_match_fun(), observer->get_payload(), this_comm,
+      mbox->find_matching_comm(CommImplType::RECEIVE, observer->get_match_fun(), observer->get_match_data(), this_comm,
                                /*done*/ false, /*remove_matching*/ true);
 
   if (not other_comm) {
@@ -255,13 +250,13 @@ ActivityImplPtr CommImpl::isend(actor::CommIsendSimcall* observer)
 
   /* Setup the communication synchro */
   other_comm->src_actor_ = observer->get_issuer();
-  other_comm->src_data_  = observer->get_payload();
   (*other_comm)
       .set_src_buff(observer->get_src_buff(), observer->get_src_buff_size())
       .set_size(observer->get_payload_size())
       .set_rate(observer->get_rate());
 
-  other_comm->match_fun     = observer->get_match_fun();
+  other_comm->match_fun       = observer->get_match_fun();
+  other_comm->src_match_data_ = observer->get_match_data();
   other_comm->copy_data_fun = observer->get_copy_data_fun();
 
   if (MC_is_active() || MC_record_replay_is_active())
@@ -285,7 +280,7 @@ ActivityImplPtr CommImpl::irecv(actor::CommIrecvSimcall* observer)
   if (mbox->is_permanent() && mbox->has_some_done_comm()) {
     XBT_DEBUG("We have a comm that has probably already been received, trying to match it, to skip the communication");
     // find a match in the list of already received comms
-    other_comm = mbox->find_matching_comm(CommImplType::SEND, observer->get_match_fun(), observer->get_payload(),
+    other_comm = mbox->find_matching_comm(CommImplType::SEND, observer->get_match_fun(), observer->get_match_data(),
                                           this_synchro, /*done*/ true, /*remove_matching*/ true);
     if (other_comm && other_comm->model_action_ && other_comm->get_remaining() < 1e-12) {
       XBT_DEBUG("comm %p has been already sent, and is finished, destroy it", other_comm.get());
@@ -308,7 +303,7 @@ ActivityImplPtr CommImpl::irecv(actor::CommIrecvSimcall* observer)
      * ourself so that the other side also gets a chance of choosing if it wants to match with us.
      *
      * If it is not found then push our communication into the rendez-vous point */
-    other_comm = mbox->find_matching_comm(CommImplType::SEND, observer->get_match_fun(), observer->get_payload(),
+    other_comm = mbox->find_matching_comm(CommImplType::SEND, observer->get_match_fun(), observer->get_match_data(),
                                           this_synchro, /*done*/ false, /*remove_matching*/ true);
 
     if (other_comm == nullptr) {
@@ -326,14 +321,14 @@ ActivityImplPtr CommImpl::irecv(actor::CommIrecvSimcall* observer)
 
   /* Setup communication synchro */
   other_comm->dst_actor_ = observer->get_issuer();
-  other_comm->dst_data_  = observer->get_payload();
   other_comm->set_dst_buff(observer->get_dst_buff(), observer->get_dst_buff_size());
 
   if (observer->get_rate() > -1.0 && (other_comm->get_rate() < 0.0 || observer->get_rate() < other_comm->get_rate()))
     other_comm->set_rate(observer->get_rate());
 
-  other_comm->match_fun     = observer->get_match_fun();
-  other_comm->copy_data_fun = observer->get_copy_data_fun();
+  other_comm->match_fun       = observer->get_match_fun();
+  other_comm->dst_match_data_ = observer->get_match_data();
+  other_comm->copy_data_fun   = observer->get_copy_data_fun();
 
   if (MC_is_active() || MC_record_replay_is_active()) {
     other_comm->set_state(State::RUNNING);
@@ -351,27 +346,6 @@ bool CommImpl::test(actor::ActorImpl* issuer)
   return ActivityImpl::test(issuer);
 }
 
-void CommImpl::wait_for(actor::ActorImpl* issuer, double timeout)
-{
-  XBT_DEBUG("CommImpl::wait_for(%g), %p, state %s", timeout, this, get_state_str());
-
-  /* Associate this simcall to the wait synchro */
-  register_simcall(&issuer->simcall_);
-  if (MC_is_active() || MC_record_replay_is_active()) {
-    // FIXME: what about timeouts?
-    set_state(State::DONE);
-    finish();
-    return;
-  }
-  ActivityImpl::wait_for(issuer, timeout);
-}
-
-void CommImpl::wait_any_for(actor::ActorImpl* issuer, const std::vector<CommImpl*>& comms, double timeout)
-{
-  std::vector<ActivityImpl*> activities(comms.begin(), comms.end());
-  ActivityImpl::wait_any_for(issuer, activities, timeout);
-}
-
 void CommImpl::suspend()
 {
   /* FIXME: shall we suspend also the timeout synchro? */
@@ -400,71 +374,11 @@ void CommImpl::cancel()
              && not MC_record_replay_is_active() && (get_state() == State::READY || get_state() == State::RUNNING)) {
     model_action_->cancel();
   }
-}
-
-void CommImpl::set_exception(actor::ActorImpl* issuer)
-{
-  switch (get_state()) {
-    case State::FAILED:
-      issuer->exception_ = std::make_exception_ptr(NetworkFailureException(XBT_THROW_POINT, "Remote peer failed"));
-      break;
-    case State::SRC_TIMEOUT:
-      issuer->exception_ =
-          std::make_exception_ptr(TimeoutException(XBT_THROW_POINT, "Communication timeouted because of the sender"));
-      break;
-
-    case State::DST_TIMEOUT:
-      issuer->exception_ =
-          std::make_exception_ptr(TimeoutException(XBT_THROW_POINT, "Communication timeouted because of the receiver"));
-      break;
-
-    case State::SRC_HOST_FAILURE:
-      if (issuer == src_actor_)
-        issuer->set_wannadie();
-      else {
-        set_state(State::FAILED);
-        issuer->exception_ = std::make_exception_ptr(NetworkFailureException(XBT_THROW_POINT, "Remote peer failed"));
-      }
-      break;
-
-    case State::DST_HOST_FAILURE:
-      if (issuer == dst_actor_)
-        issuer->set_wannadie();
-      else {
-        set_state(State::FAILED);
-        issuer->exception_ = std::make_exception_ptr(NetworkFailureException(XBT_THROW_POINT, "Remote peer failed"));
-      }
-      break;
-
-    case State::LINK_FAILURE:
-      XBT_DEBUG("Link failure in synchro %p between '%s' and '%s': posting an exception to the issuer: %s (%p) "
-                "detached:%d",
-                this, src_actor_ ? src_actor_->get_host()->get_cname() : nullptr,
-                dst_actor_ ? dst_actor_->get_host()->get_cname() : nullptr, issuer->get_cname(), issuer, detached_);
-      if (src_actor_ == issuer) {
-        XBT_DEBUG("I'm source");
-      } else if (dst_actor_ == issuer) {
-        XBT_DEBUG("I'm dest");
-      } else {
-        XBT_DEBUG("I'm neither source nor dest");
-      }
-      set_state(State::FAILED);
-      issuer->throw_exception(std::make_exception_ptr(NetworkFailureException(XBT_THROW_POINT, "Link failure")));
-      break;
-
-    case State::CANCELED:
-      if (issuer == dst_actor_)
-        issuer->exception_ =
-            std::make_exception_ptr(CancelException(XBT_THROW_POINT, "Communication canceled by the sender"));
-      else
-        issuer->exception_ =
-            std::make_exception_ptr(CancelException(XBT_THROW_POINT, "Communication canceled by the receiver"));
-      break;
-
-    default:
-      xbt_assert(get_state() == State::DONE, "Internal error in CommImpl::finish(): unexpected synchro state %s",
-                 get_state_str());
-  }
+  CommImplPtr tmp = this; // Make sure the object does not disappear until after we check whether we need to remove it
+  if (tmp->src_actor_)
+    tmp->src_actor_->activities_.erase(this); // The actor does not need to cancel the activity when it dies
+  if (tmp->dst_actor_)
+    tmp->dst_actor_->activities_.erase(this); // The actor does not need to cancel the activity when it dies
 }
 
 void CommImpl::finish()
@@ -472,22 +386,10 @@ void CommImpl::finish()
   XBT_DEBUG("CommImpl::finish() comm %p, state %s, src_proc %p, dst_proc %p, detached: %d", this, get_state_str(),
             src_actor_.get(), dst_actor_.get(), detached_);
 
-  if (get_iface()) {
-    const auto& piface = static_cast<const s4u::Comm&>(*get_iface());
-    set_iface(nullptr); // reset iface to protect against multiple trigger of the on_completion signals
-    piface.fire_on_completion_for_real();
-    piface.fire_on_this_completion_for_real();
-  }
-
   /* Update synchro state */
-  if (src_timeout_ && src_timeout_->get_state() == resource::Action::State::FINISHED)
-    set_state(State::SRC_TIMEOUT);
-  else if (dst_timeout_ && dst_timeout_->get_state() == resource::Action::State::FINISHED)
-    set_state(State::DST_TIMEOUT);
-  else if ((from_ && not from_->is_on()) ||
-           (src_timeout_ && src_timeout_->get_state() == resource::Action::State::FAILED))
+  if (from_ && not from_->is_on())
     set_state(State::SRC_HOST_FAILURE);
-  else if ((to_ && not to_->is_on()) || (dst_timeout_ && dst_timeout_->get_state() == resource::Action::State::FAILED))
+  else if (to_ && not to_->is_on())
     set_state(State::DST_HOST_FAILURE);
   else if (model_action_ && model_action_->get_state() == resource::Action::State::FAILED) {
     set_state(State::LINK_FAILURE);
@@ -496,8 +398,17 @@ void CommImpl::finish()
     xbt_assert(to_ && to_->is_on());
     set_state(State::DONE);
   }
-  src_timeout_ = nullptr;
-  dst_timeout_ = nullptr;
+
+  if (get_iface()) {
+    auto& piface = static_cast<s4u::Comm&>(*get_iface());
+    set_iface(nullptr); // reset iface to protect against multiple trigger of the on_completion signals
+    piface.fire_on_completion_for_real();
+    piface.fire_on_this_completion_for_real();
+    if (detached_ && get_state() == State::DONE){
+      s4u::CommPtr keepalive(&piface);
+      piface.release_dependencies();
+    }
+  }
 
   /* destroy the model actions associated with the communication activity */
   clean_action();
@@ -513,36 +424,66 @@ void CommImpl::finish()
     EngineImpl::get_instance()->get_maestro()->activities_.erase(this);
 
   while (not simcalls_.empty()) {
-    actor::Simcall* simcall = simcalls_.front();
-    simcalls_.pop_front();
 
-    /* If a waitany simcall is waiting for this synchro to finish, then remove it from the other synchros in the waitany
-     * list. Afterwards, get the position of the actual synchro in the waitany list and return it as the result of the
-     * simcall */
+    auto issuer = unregister_first_simcall();
+    if (issuer == nullptr) /* don't answer exiting and dying actors */
+      continue;
 
-    if (simcall->call_ == actor::Simcall::Type::NONE) // FIXME: maybe a better way to handle this case
-      continue;                                       // if actor handling comm is killed
+    issuer->activities_.erase(this);
 
-    handle_activity_waitany(simcall);
+    /* Check out for errors, and answer the simcall */
+    switch (get_state()) {
+      case State::FAILED: // FIXME: probably useless nowadays
+        issuer->exception_ = std::make_exception_ptr(NetworkFailureException(XBT_THROW_POINT, "Remote peer failed"));
+        break;
 
-    /* Check out for errors */
+      case State::SRC_HOST_FAILURE:
+        xbt_assert(issuer != src_actor_);
+        set_state(State::FAILED);
+        issuer->exception_ = std::make_exception_ptr(NetworkFailureException(XBT_THROW_POINT, "Remote peer failed"));
+        break;
 
-    if (not simcall->issuer_->get_host()->is_on()) {
-      simcall->issuer_->set_wannadie();
-    } else {
-      // Do not answer to dying actors
-      if (not simcall->issuer_->wannadie()) {
-        set_exception(simcall->issuer_);
-        simcall->issuer_->simcall_answer();
-      }
+      case State::DST_HOST_FAILURE:
+        xbt_assert(issuer != dst_actor_);
+        set_state(State::FAILED);
+        issuer->exception_ = std::make_exception_ptr(NetworkFailureException(XBT_THROW_POINT, "Remote peer failed"));
+        break;
+
+      case State::LINK_FAILURE:
+        XBT_DEBUG("Link failure in synchro %p between '%s' and '%s': posting an exception to the issuer: %s (%p) "
+                  "detached:%d",
+                  this, src_actor_ ? src_actor_->get_host()->get_cname() : nullptr,
+                  dst_actor_ ? dst_actor_->get_host()->get_cname() : nullptr, issuer->get_cname(), issuer, detached_);
+        if (src_actor_ == issuer) {
+          XBT_DEBUG("I'm source");
+        } else if (dst_actor_ == issuer) {
+          XBT_DEBUG("I'm dest");
+        } else {
+          XBT_DEBUG("I'm neither source nor dest");
+        }
+        set_state(State::FAILED);
+        issuer->throw_exception(std::make_exception_ptr(NetworkFailureException(XBT_THROW_POINT, "Link failure")));
+        break;
+
+      case State::CANCELED:
+        if (issuer == dst_actor_)
+          issuer->exception_ =
+              std::make_exception_ptr(CancelException(XBT_THROW_POINT, "Communication canceled by the sender"));
+        else
+          issuer->exception_ =
+              std::make_exception_ptr(CancelException(XBT_THROW_POINT, "Communication canceled by the receiver"));
+        break;
+
+      default:
+        xbt_assert(get_state() == State::DONE, "Internal error in CommImpl::finish(): unexpected synchro state %s",
+                   get_state_str());
     }
+    issuer->simcall_answer();
 
-    simcall->issuer_->waiting_synchro_ = nullptr;
-    simcall->issuer_->activities_.erase(this);
     if (detached_) {
-      if (simcall->issuer_ != dst_actor_ && dst_actor_ != nullptr)
+      if (issuer != dst_actor_ && dst_actor_ != nullptr)
         dst_actor_->activities_.erase(this);
-      if (simcall->issuer_ != src_actor_ && src_actor_ != nullptr)
+      if (issuer != src_actor_ && src_actor_ != nullptr)
         src_actor_->activities_.erase(this);
     }
   }
