@@ -1,4 +1,4 @@
-/* Copyright (c) 2007-2023. The SimGrid Team. All rights reserved.          */
+/* Copyright (c) 2007-2025. The SimGrid Team. All rights reserved.          */
 
 /* This program is free software; you can redistribute it and/or modify it
  * under the terms of the license (GNU LGPL) which comes with this package. */
@@ -14,8 +14,6 @@ namespace simgrid::kernel::activity {
 
 enum class CommImplType { SEND, RECEIVE };
 
-using timeout_action_type = std::unique_ptr<resource::Action, std::function<void(resource::Action*)>>;
-
 class XBT_PUBLIC CommImpl : public ActivityImpl_T<CommImpl> {
   ~CommImpl() override;
 
@@ -23,7 +21,6 @@ class XBT_PUBLIC CommImpl : public ActivityImpl_T<CommImpl> {
 
   double rate_       = -1.0;
   double size_       = 0.0;
-  bool detached_     = false;   /* If detached or not */
   bool copied_       = false;   /* whether the data were already copied */
   MailboxImpl* mbox_ = nullptr; /* Rendez-vous where the comm is queued. nullptr once the comm is matched with both a
                                    sender and receiver */
@@ -51,13 +48,12 @@ public:
   CommImpl& set_dst_buff(unsigned char* buff, size_t* size);
   CommImpl& set_rate(double rate);
   CommImpl& set_mailbox(MailboxImpl* mbox);
-  CommImpl& detach();
+
 
   double get_rate() const { return rate_; }
   MailboxImpl* get_mailbox() const { return mbox_; }
   unsigned get_mailbox_id() const { return mbox_id_; }
   unsigned get_id() const { return id_; }
-  bool is_detached() const { return detached_; }
   bool is_assigned() const { return (to_ != nullptr && from_ != nullptr); }
 
   std::vector<s4u::Link*> get_traversed_links() const;
@@ -67,26 +63,21 @@ public:
   static ActivityImplPtr irecv(actor::CommIrecvSimcall* observer);
 
   bool test(actor::ActorImpl* issuer) override;
-  void wait_for(actor::ActorImpl* issuer, double timeout) override;
-  static void wait_any_for(actor::ActorImpl* issuer, const std::vector<CommImpl*>& comms, double timeout);
 
   CommImpl* start();
   void suspend() override;
   void resume() override;
   void cancel() override;
-  void set_exception(actor::ActorImpl* issuer) override;
   void finish() override;
 
   std::function<void(void*)> clean_fun; /* Function to clean the detached src_buf if something goes wrong */
   std::function<bool(void*, void*, CommImpl*)> match_fun; /* Filter function used by the other side. It is used when
 looking if a given communication matches my needs. For that, myself must match the
 expectations of the other side, too. See  */
+  void* src_match_data_ = nullptr;                        /* User data associated to the communication */
+  void* dst_match_data_ = nullptr;
   std::function<void(CommImpl*, void*, size_t)> copy_data_fun;
 
-  /* Model actions */
-  timeout_action_type src_timeout_{nullptr, [](resource::Action* a) { a->unref(); }}; /* timeout set by the sender */
-  timeout_action_type dst_timeout_{nullptr, [](resource::Action* a) { a->unref(); }}; /* timeout set by the receiver */
-
   actor::ActorImplPtr src_actor_ = nullptr;
   actor::ActorImplPtr dst_actor_ = nullptr;
 
@@ -96,9 +87,6 @@ expectations of the other side, too. See  */
   size_t src_buff_size_    = 0;
   size_t* dst_buff_size_   = nullptr;
   void* payload_           = nullptr; // If dst_buff_ is NULL, the default copy callback puts the data here
-
-  void* src_data_ = nullptr; /* User data associated to the communication */
-  void* dst_data_ = nullptr;
 };
 } // namespace simgrid::kernel::activity
 
