@@ -1,4 +1,4 @@
-/* Copyright (c) 2012-2023. The SimGrid Team. All rights reserved.          */
+/* Copyright (c) 2012-2025. The SimGrid Team. All rights reserved.          */
 
 /* This program is free software; you can redistribute it and/or modify it
  * under the terms of the license (GNU LGPL) which comes with this package. */
@@ -11,6 +11,7 @@
 #include "src/kernel/actor/ActorImpl.hpp"
 #include "xbt/asserts.h"
 #include <boost/intrusive/list.hpp>
+#include <string>
 
 namespace simgrid::kernel::activity {
 
@@ -60,15 +61,12 @@ public:
   bool test(actor::ActorImpl* issuer = nullptr) override;
   void wait_for(actor::ActorImpl* issuer, double timeout) override;
   void finish() override;
-  void set_exception(actor::ActorImpl* issuer) override
-  { /* nothing to do */
-  }
 };
 
 class XBT_PUBLIC MutexImpl {
   std::atomic_int_fast32_t refcount_{1};
   s4u::Mutex piface_;
-  actor::ActorImpl* owner_ = nullptr;
+  actor::ActorImplPtr owner_ = nullptr;
   std::deque<MutexAcquisitionImplPtr> ongoing_acquisitions_;
   static unsigned next_id_;
   unsigned id_ = next_id_++;
@@ -87,7 +85,7 @@ public:
   void unlock(actor::ActorImpl* issuer);
   unsigned get_id() const { return id_; }
 
-  actor::ActorImpl* get_owner() const { return owner_; }
+  actor::ActorImpl* get_owner() const { return owner_.get(); }
 
   // boost::intrusive_ptr<Mutex> support:
   friend void intrusive_ptr_add_ref(MutexImpl* mutex)
@@ -100,12 +98,38 @@ public:
   {
     if (mutex->refcount_.fetch_sub(1) == 1) {
       xbt_assert(mutex->ongoing_acquisitions_.empty(), "The destroyed mutex still had ongoing acquisitions");
-      xbt_assert(mutex->owner_ == nullptr, "The destroyed mutex is still owned by actor %s",
-                 mutex->owner_->get_cname());
+      // Sanity check, when the simulation is ending, only issue a small warning, not a big fat xbt_assert
+      if (mutex->owner_ != nullptr) {
+        if ((not s4u::Engine::has_instance()) || mutex->owner_->wannadie()) {
+          static bool shown = false;
+          if (not shown)
+            ::fprintf(stderr,
+                      "Actor %s is dying, but it still has a locked mutex. That's OK as no other actor waits for this "
+                      "mutex.\n",
+                      mutex->owner_->get_cname());
+          shown = true;
+        } else
+          xbt_die("The destroyed mutex %u is still owned by actor %s", mutex->id_, mutex->owner_->get_cname());
+      }
       delete mutex;
     }
   }
 
+  std::string to_string() const
+  {
+    std::string out = std::string("mutex(id: ") + std::to_string(id_);
+    if (is_recursive_) {
+      out += ", recusive with recdepth=" + std::to_string(recursive_depth);
+    } else {
+      out += ", non-recursive";
+    }
+    out += std::string(", owner: ") + (owner_ ? std::to_string(owner_->get_pid()) : "none");
+    out += ongoing_acquisitions_.empty() ? " + no" : std::to_string(ongoing_acquisitions_.size());
+    out += " queued acquisitions";
+    out += ")";
+    return out;
+  }
+
   s4u::Mutex& get_iface() { return piface_; }
 };
 } // namespace simgrid::kernel::activity
