@@ -1,4 +1,4 @@
-/* Copyright (c) 2019-2023. The SimGrid Team. All rights reserved.          */
+/* Copyright (c) 2019-2025. The SimGrid Team. All rights reserved.          */
 
 /* This program is free software; you can redistribute it and/or modify it
  * under the terms of the license (GNU LGPL) which comes with this package. */
@@ -9,6 +9,7 @@
 #include "simgrid/s4u/Semaphore.hpp"
 #include "src/kernel/actor/ActorImpl.hpp"
 #include "src/kernel/actor/SynchroObserver.hpp"
+#include "xbt/ex.h"
 
 #include <atomic>
 #include <boost/intrusive/list.hpp>
@@ -29,7 +30,7 @@ class XBT_PUBLIC SemAcquisitionImpl : public ActivityImpl_T<SemAcquisitionImpl>
   friend actor::SemaphoreAcquisitionObserver;
 
 public:
-  SemAcquisitionImpl(actor::ActorImpl* issuer, SemaphoreImpl* sem) : issuer_(issuer), semaphore_(sem) {}
+  SemAcquisitionImpl(actor::ActorImpl* issuer, SemaphoreImpl* sem);
   SemaphoreImplPtr get_semaphore() { return semaphore_; }
   actor::ActorImpl* get_issuer() { return issuer_; }
 
@@ -37,9 +38,6 @@ public:
   void wait_for(actor::ActorImpl* issuer, double timeout) override;
   void finish() override;
   void cancel() override;
-  void set_exception(actor::ActorImpl* issuer) override
-  { /* nothing to do */
-  }
 };
 
 class XBT_PUBLIC SemaphoreImpl {
