@@ -1,4 +1,4 @@
-/* Copyright (c) 2007-2023. The SimGrid Team. All rights reserved.          */
+/* Copyright (c) 2007-2025. The SimGrid Team. All rights reserved.          */
 
 /* This program is free software; you can redistribute it and/or modify it
  * under the terms of the license (GNU LGPL) which comes with this package. */
@@ -10,6 +10,7 @@
 
 #include "src/internal_config.h"
 #include "src/kernel/EngineImpl.hpp"
+#include "src/sthread/sthread.h"
 #if HAVE_SMPI
 #include "src/smpi/include/private.hpp"
 #endif
@@ -19,6 +20,8 @@
 #include <typeinfo>
 #include <utility>
 
+#include <mutex> // To terminate the actors in a thread-safe manner
+
 XBT_LOG_NEW_DEFAULT_SUBCATEGORY(ker_actor, kernel, "Logging specific to Actor's kernel side");
 
 namespace simgrid::kernel::actor {
@@ -152,10 +155,19 @@ void ActorImpl::cleanup_from_self()
     on_exit.reset();
   }
 
-  /* cancel non-blocking activities */
-  for (auto activity : activities_)
-    activity->cancel();
-  activities_.clear();
+  /* cancel non-blocking activities, in a thread-safe way. We cannot protect this state modification in a simcall
+   * because the actor is dying, but we need to take care of sthread. */
+  static std::mutex destruction_mutex;
+  sthread_disable();
+  destruction_mutex.lock();
+  sthread_enable();
+
+  while (not activities_.empty())
+    activities_.begin()->get()->cancel(); // cancel() removes the activity from this collection
+
+  sthread_disable();
+  destruction_mutex.unlock();
+  sthread_enable();
 
   XBT_DEBUG("%s@%s(%ld) should not run anymore", get_cname(), get_host()->get_cname(), get_pid());
 
@@ -181,19 +193,19 @@ void ActorImpl::exit()
   suspended_ = false;
   exception_ = nullptr;
 
-  if (waiting_synchro_ != nullptr) {
+  while (not waiting_synchros_.empty()) {
     /* Take an extra reference on the activity object that may be unref by Comm::finish() or friends */
-    activity::ActivityImplPtr activity = waiting_synchro_;
+    activity::ActivityImplPtr activity = waiting_synchros_.back();
+    waiting_synchros_.pop_back();
+    
     activity->cancel();
     activity->set_state(activity::State::FAILED);
     activity->finish();
 
-    activities_.erase(waiting_synchro_);
-    waiting_synchro_ = nullptr;
+    activities_.erase(activity);
   }
-  for (auto const& activity : activities_)
-    activity->cancel();
-  activities_.clear();
+  while (not activities_.empty())
+    activities_.begin()->get()->cancel();
 
   // Forcefully kill the actor if its host is turned off. Not a HostFailureException because you should not survive that
   this->throw_exception(std::make_exception_ptr(ForcefulKillException(host_->is_on() ? "exited" : "host failed")));
@@ -213,9 +225,9 @@ void ActorImpl::kill(ActorImpl* actor) const
 
   actor->exit();
 
-  if (actor == this) {
-    XBT_DEBUG("Go on, this is a suicide,");
-  } else
+  if (actor == this)
+    XBT_DEBUG("Damn, this is a suicide :(");
+  else
     EngineImpl::get_instance()->add_actor_to_run_list(actor);
 }
 
@@ -345,7 +357,7 @@ void ActorImpl::resume()
   /* resume the activities that were blocked when suspending the actor. */
   for (auto const& activity : activities_)
     activity->resume();
-  if (not waiting_synchro_) // Reschedule the actor if it was forcefully unscheduled in yield()
+  if (waiting_synchros_.empty()) // Reschedule the actor if it was forcefully unscheduled in yield()
     EngineImpl::get_instance()->add_actor_to_run_list_no_check(this);
 
   XBT_OUT();
@@ -384,11 +396,11 @@ void ActorImpl::throw_exception(std::exception_ptr e)
   if (suspended_)
     resume();
 
-  /* cancel the blocking synchro if any */
-  if (waiting_synchro_) {
-    waiting_synchro_->cancel();
-    activities_.erase(waiting_synchro_);
-    waiting_synchro_ = nullptr;
+  /* cancel the blocking synchros if any */
+  while (not waiting_synchros_.empty()) {
+    waiting_synchros_.back()->cancel();
+    activities_.erase(waiting_synchros_.back());
+    waiting_synchros_.pop_back();
   }
 }
 
