@@ -1,4 +1,4 @@
-/* Copyright (c) 2019-2023. The SimGrid Team. All rights reserved.          */
+/* Copyright (c) 2019-2025. The SimGrid Team. All rights reserved.          */
 
 /* This program is free software; you can redistribute it and/or modify it
  * under the terms of the license (GNU LGPL) which comes with this package. */
@@ -7,8 +7,10 @@
 #define SIMGRID_MC_SIMCALL_OBSERVER_HPP
 
 #include "simgrid/forward.h"
+#include "src/mc/remote/Channel.hpp"
 #include "src/mc/transition/Transition.hpp"
 #include "xbt/asserts.h"
+#include "xbt/ex.h"
 
 #include <string>
 #include <unordered_map>
@@ -54,7 +56,7 @@ public:
   }
 
   /** Serialize to the given string buffer, to send over the network */
-  virtual void serialize(std::stringstream& stream) const = 0;
+  virtual void serialize(simgrid::mc::Channel& channel) const = 0;
 
   /** Used to debug (to display the simcall on which each actor is blocked when displaying it */
   virtual std::string to_string() const = 0;
@@ -65,17 +67,30 @@ public:
   virtual bool is_visible() const { return true; }
 };
 
-template <class T> class ResultingSimcall : public SimcallObserver {
+/** This is the ancestor class of all observers for simcalls that do not answer immediately but only through
+ * simcall_blocking */
+template <class T> class DelayedSimcallObserver : public SimcallObserver {
   T result_;
 
 protected:
-  ~ResultingSimcall() = default;
+  ~DelayedSimcallObserver() = default;
 
 public:
-  ResultingSimcall(ActorImpl* actor, T default_result) : SimcallObserver(actor), result_(default_result) {}
+  DelayedSimcallObserver(ActorImpl* actor, T default_result) : SimcallObserver(actor), result_(default_result) {}
   void set_result(T res) { result_ = res; }
   T get_result() const { return result_; }
 };
+/* Specialization of the above template for the void return type, that is used for pure synchro simcalls */
+template <> class DelayedSimcallObserver<void> : public SimcallObserver {
+protected:
+  ~DelayedSimcallObserver() = default;
+
+public:
+  explicit DelayedSimcallObserver(ActorImpl* actor) : SimcallObserver(actor) {}
+  void get_result() const
+  { /* Nothing to do */
+  }
+};
 
 class RandomSimcall final : public SimcallObserver {
   const int min_;
@@ -87,35 +102,54 @@ public:
   {
     xbt_assert(min <= max);
   }
-  void serialize(std::stringstream& stream) const override;
+  void serialize(mc::Channel& channel) const override;
   std::string to_string() const override;
   int get_max_consider() const override;
   void prepare(int times_considered) override;
   int get_value() const { return next_value_; }
 };
 
-class ActorJoinSimcall final : public SimcallObserver {
+class ActorJoinSimcall final : public DelayedSimcallObserver<void> {
   s4u::ActorPtr const other_; // We need a Ptr to ensure access to the actor after its end, but Ptr requires s4u
   const double timeout_;
 
 public:
   ActorJoinSimcall(ActorImpl* actor, ActorImpl* other, double timeout = -1.0);
-  void serialize(std::stringstream& stream) const override;
+  void serialize(mc::Channel& channel) const override;
   std::string to_string() const override;
   bool is_enabled() override;
 
   s4u::ActorPtr get_other_actor() const { return other_; }
   double get_timeout() const { return timeout_; }
 };
+/* Dummy observer, not visible by the checker but needed for the template to compile */
+class ActorSuspendSimcall final : public DelayedSimcallObserver<void> {
+public:
+  explicit ActorSuspendSimcall(ActorImpl* actor) : DelayedSimcallObserver<void>(actor) {}
+  void serialize(mc::Channel& channel) const override { THROW_UNIMPLEMENTED; }
+  std::string to_string() const override { THROW_UNIMPLEMENTED; }
+  bool is_visible() const override { return false; }
+};
 
-class ActorSleepSimcall final : public SimcallObserver {
+class ActorSleepSimcall final : public DelayedSimcallObserver<void> {
 
 public:
-  explicit ActorSleepSimcall(ActorImpl* actor) : SimcallObserver(actor) {}
-  void serialize(std::stringstream& stream) const override;
+  explicit ActorSleepSimcall(ActorImpl* actor) : DelayedSimcallObserver<void>(actor) {}
+  void serialize(mc::Channel& channel) const override;
   std::string to_string() const override;
 };
 
+class ActorCreateSimcall final : public SimcallObserver {
+  aid_t child_ = -1;
+
+public:
+  explicit ActorCreateSimcall(ActorImpl* actor) : SimcallObserver(actor) {}
+  void serialize(mc::Channel& channel) const override;
+  std::string to_string() const override;
+
+  void set_child(aid_t child) { child_ = child; }
+};
+
 class ObjectAccessSimcallObserver final : public SimcallObserver {
   ObjectAccessSimcallItem* const object_;
 
@@ -124,7 +158,7 @@ public:
       : SimcallObserver(actor), object_(object)
   {
   }
-  void serialize(std::stringstream& stream) const override;
+  void serialize(mc::Channel& channel) const override;
   std::string to_string() const override;
   bool is_visible() const override;
   bool is_enabled() override { return true; }
