@@ -1,4 +1,4 @@
-/* Copyright (c) 2009-2023. The SimGrid Team. All rights reserved.          */
+/* Copyright (c) 2009-2025. The SimGrid Team. All rights reserved.          */
 
 /* This program is free software; you can redistribute it and/or modify it
  * under the terms of the license (GNU LGPL) which comes with this package. */
@@ -87,6 +87,7 @@ void ThreadContext::wrapper(ThreadContext* context)
   // Tell the caller (normally the maestro) we are starting, and wait for its green light
   context->end_.release();
   context->start();
+  context->initialized();
 
   try {
     (*context)();
@@ -105,6 +106,7 @@ void ThreadContext::wrapper(ThreadContext* context)
   install_sigsegv_stack(false);
   XBT_DEBUG("Terminating");
   Context::set_current(nullptr);
+  context->finalizing();
 }
 
 void ThreadContext::release()
