@@ -1,4 +1,4 @@
-/* Copyright (c) 2009-2023. The SimGrid Team. All rights reserved.          */
+/* Copyright (c) 2009-2025. The SimGrid Team. All rights reserved.          */
 
 /* This program is free software; you can redistribute it and/or modify it
  * under the terms of the license (GNU LGPL) which comes with this package. */
@@ -14,7 +14,7 @@
 #include <vector>
 
 #include "src/xbt/parmap.hpp"
-#include <simgrid/simix.hpp>
+#include <simgrid/simcall.hpp>
 
 #include "src/internal_config.h"
 #include "src/kernel/context/ContextSwapped.hpp"
@@ -33,6 +33,7 @@ private:
 
 class UContextFactory : public SwappedContextFactory {
 public:
+  const char* get_name() const override { return "ucontext"; }
   UContext* create_context(std::function<void()>&& code, actor::ActorImpl* actor) override;
 };
 } // namespace simgrid::kernel::context
