@@ -1,4 +1,4 @@
-/* Copyright (c) 2013-2023. The SimGrid Team. All rights reserved.          */
+/* Copyright (c) 2013-2025. The SimGrid Team. All rights reserved.          */
 
 /* This program is free software; you can redistribute it and/or modify it
  * under the terms of the license (GNU LGPL) which comes with this package. */
@@ -14,6 +14,7 @@
 #include "src/simgrid/math_utils.h"
 #include "src/simgrid/module.hpp"
 #include "src/simgrid/sg_config.hpp"
+#include "xbt/config.hpp"
 
 #include <algorithm>
 #include <numeric>
@@ -27,6 +28,22 @@ static simgrid::config::Flag<std::string> cfg_network_solver("network/solver",
                                                              "Set linear equations solver used by network model",
                                                              "maxmin", &simgrid::kernel::lmm::System::validate_solver);
 
+SIMGRID_REGISTER_NETWORK_MODEL(raw,
+                               "Simplest network model with `time = size/bw + lat` and fair sharing. "
+                               "No cross traffic, no TCP gamma, no correction factors, just these plain equations.",
+                               []() {
+                                 auto net_model =
+                                     std::make_shared<simgrid::kernel::resource::NetworkCm02Model>("Network_Raw");
+                                 auto* engine = simgrid::kernel::EngineImpl::get_instance();
+                                 engine->add_model(net_model);
+                                 engine->get_netzone_root()->set_network_model(net_model);
+
+                                 simgrid::config::set_default<std::string>("network/latency-factor", "1.0");
+                                 simgrid::config::set_default<std::string>("network/bandwidth-factor", "1.0");
+                                 simgrid::config::set_default<double>("network/weight-S", 0.0);
+                                 simgrid::config::set_default<bool>("network/crosstraffic", false);
+                               });
+
 /******************************************************************************/
 /* Network model based on optimizations discussed during Pedro Velho's thesis */
 /******************************************************************************/
