@@ -1,4 +1,4 @@
-/* Copyright (c) 2006-2023. The SimGrid Team. All rights reserved.          */
+/* Copyright (c) 2006-2025. The SimGrid Team. All rights reserved.          */
 
 /* This program is free software; you can redistribute it and/or modify it
  * under the terms of the license (GNU LGPL) which comes with this package. */
@@ -20,10 +20,15 @@
 #include "src/simgrid/module.hpp"
 #include "src/simgrid/sg_config.hpp"
 #include "xbt/asserts.hpp"
+#include "xbt/log.h"
 
 XBT_LOG_NEW_DEFAULT_SUBCATEGORY(ker_platform, kernel, "Kernel platform-related information");
 
-namespace simgrid::kernel::routing {
+namespace simgrid {
+
+template class xbt::Extendable<kernel::routing::NetZoneImpl>;
+
+namespace kernel::routing {
 
 xbt::signal<void(bool symmetrical, kernel::routing::NetPoint* src, kernel::routing::NetPoint* dst,
                  kernel::routing::NetPoint* gw_src, kernel::routing::NetPoint* gw_dst,
@@ -33,32 +38,11 @@ xbt::signal<void(bool symmetrical, kernel::routing::NetPoint* src, kernel::routi
 NetZoneImpl::NetZoneImpl(const std::string& name) : piface_(this), name_(name)
 {
   auto* engine = s4u::Engine::get_instance();
-  /* workaroud: first netzoneImpl will be the root netzone.
-   * Without globals and with current model description init functions (see module.hpp), we need
-   * the root netzone to exist when creating the models.
-   * This is usually done at sg_platf.cpp, during XML parsing */
-  if (not engine->get_netzone_root()) {
-    engine->set_netzone_root(&piface_);
-    /* root netzone set, initialize models */
-    simgrid::s4u::Engine::on_platform_creation();
-
-    /* Initialize the models. That must be done after we got all config, and before we need the models.
-     * That is, after the last <config> tag, if any, and before the first of cluster|peer|zone|trace|trace_cb
-     *
-     * I'm not sure for <trace> and <trace_cb>, there may be a bug here
-     * (FIXME: check it out by creating a file beginning with one of these tags)
-     * but cluster and peer come down to zone creations, so putting this verification here is correct.
-     */
-    simgrid_host_models().init_from_flag_value();
-    simgrid_vm_model_init_HL13();
-  }
 
   xbt_enforce(nullptr == engine->netpoint_by_name_or_null(get_name()),
              "Refusing to create a second NetZone called '%s'.", get_cname());
   netpoint_ = new NetPoint(name_, NetPoint::Type::NetZone);
   XBT_DEBUG("NetZone '%s' created with the id '%lu'", get_cname(), netpoint_->id());
-  _sg_cfg_init_status = 2; /* HACK: direct access to the global controlling the level of configuration to prevent
-                            * any further config now that we created some real content */
   simgrid::s4u::NetZone::on_creation(piface_); // notify the signal
 }
 
@@ -165,7 +149,7 @@ size_t NetZoneImpl::get_link_count() const
   return total;
 }
 
-s4u::Host* NetZoneImpl::create_host(const std::string& name, const std::vector<double>& speed_per_pstate)
+s4u::Host* NetZoneImpl::add_host(const std::string& name, const std::vector<double>& speed_per_pstate)
 {
   xbt_enforce(cpu_model_pm_,
              "Impossible to create host: %s. Invalid CPU model: nullptr. Have you set the parent of this NetZone: %s?",
@@ -185,7 +169,7 @@ resource::StandardLinkImpl* NetZoneImpl::do_create_link(const std::string& name,
   return network_model_->create_link(name, bandwidths);
 }
 
-s4u::Link* NetZoneImpl::create_link(const std::string& name, const std::vector<double>& bandwidths)
+s4u::Link* NetZoneImpl::add_link(const std::string& name, const std::vector<double>& bandwidths)
 {
   xbt_enforce(
       network_model_,
@@ -196,8 +180,8 @@ s4u::Link* NetZoneImpl::create_link(const std::string& name, const std::vector<d
   return links_[name]->get_iface();
 }
 
-s4u::SplitDuplexLink* NetZoneImpl::create_split_duplex_link(const std::string& name,
-                                                            const std::vector<double>& bandwidths)
+s4u::SplitDuplexLink* NetZoneImpl::add_split_duplex_link(const std::string& name, const std::vector<double>& bw_up,
+                                                            const std::vector<double>& bw_down)
 {
   xbt_enforce(
       network_model_,
@@ -205,13 +189,13 @@ s4u::SplitDuplexLink* NetZoneImpl::create_split_duplex_link(const std::string& n
       name.c_str(), get_cname());
   xbt_enforce(not sealed_, "Impossible to create link: %s. NetZone %s already sealed", name.c_str(), get_cname());
 
-  auto* link_up             = create_link(name + "_UP", bandwidths)->get_impl()->set_englobing_zone(this);
-  auto* link_down           = create_link(name + "_DOWN", bandwidths)->get_impl()->set_englobing_zone(this);
+  auto* link_up             = add_link(name + "_UP", bw_up)->get_impl()->set_englobing_zone(this);
+  auto* link_down           = add_link(name + "_DOWN", bw_down)->get_impl()->set_englobing_zone(this);
   split_duplex_links_[name] = std::make_unique<resource::SplitDuplexLinkImpl>(name, link_up, link_down);
   return split_duplex_links_[name]->get_iface();
 }
 
-s4u::Disk* NetZoneImpl::create_disk(const std::string& name, double read_bandwidth, double write_bandwidth)
+s4u::Disk* NetZoneImpl::add_disk(const std::string& name, double read_bandwidth, double write_bandwidth)
 {
   xbt_enforce(disk_model_,
              "Impossible to create disk: %s. Invalid disk model: nullptr. Have you set the parent of this NetZone: %s?",
@@ -222,7 +206,7 @@ s4u::Disk* NetZoneImpl::create_disk(const std::string& name, double read_bandwid
   return l->get_iface();
 }
 
-NetPoint* NetZoneImpl::create_router(const std::string& name)
+NetPoint* NetZoneImpl::add_router(const std::string& name)
 {
   xbt_enforce(nullptr == s4u::Engine::get_instance()->netpoint_by_name_or_null(name),
              "Refusing to create a router named '%s': this name already describes a node.", name.c_str());
@@ -597,6 +581,14 @@ void NetZoneImpl::get_global_route_with_netzones(const NetPoint* src, const NetP
   /* If src and dst are in the same netzone, life is good */
   if (src_ancestor == dst_ancestor) { /* ROUTING_BASE */
     route.link_list_ = std::move(links);
+
+    xbt_assert(src->get_englobing_zone() == common_ancestor,
+               "Source host %s is not part of the zone %s but of zone %s, please report that bug.", src->get_cname(),
+               common_ancestor->get_cname(), src->get_englobing_zone()->get_cname());
+    xbt_assert(dst->get_englobing_zone() == common_ancestor,
+               "Destination host %s is not part of the zone %s but of zone %s, please report that bug.",
+               dst->get_cname(), common_ancestor->get_cname(), dst->get_englobing_zone()->get_cname());
+
     common_ancestor->get_local_route(src, dst, &route, latency);
     links = std::move(route.link_list_);
     return;
@@ -714,7 +706,7 @@ void NetZoneImpl::seal()
   s4u::NetZone::on_seal(piface_);
 }
 
-void NetZoneImpl::set_parent(NetZoneImpl* parent)
+NetZoneImpl* NetZoneImpl::set_parent(NetZoneImpl* parent)
 {
   xbt_enforce(not sealed_, "Impossible to set parent to an already sealed NetZone(%s)", this->get_cname());
   parent_ = parent;
@@ -729,6 +721,7 @@ void NetZoneImpl::set_parent(NetZoneImpl* parent)
     set_disk_model(parent->get_disk_model());
     set_host_model(parent->get_host_model());
   }
+  return this;
 }
 
 void NetZoneImpl::set_network_model(std::shared_ptr<resource::NetworkModel> netmodel)
@@ -787,4 +780,6 @@ bool NetZoneImpl::is_component_recursive(const NetPoint* netpoint) const
   return std::any_of(begin(children_), end(children_),
                      [netpoint](const auto* child) { return child->is_component_recursive(netpoint); });
 }
-} // namespace simgrid::kernel::routing
+
+} // namespace kernel::routing
+} // namespace simgrid
