@@ -1,4 +1,4 @@
-/* Copyright (c) 2007-2023. The SimGrid Team. All rights reserved.          */
+/* Copyright (c) 2007-2025. The SimGrid Team. All rights reserved.          */
 
 /* This program is free software; you can redistribute it and/or modify it
  * under the terms of the license (GNU LGPL) which comes with this package. */
@@ -7,10 +7,12 @@
 #define SIMGRID_MC_PATTERN_H
 
 #include "src/kernel/activity/CommImpl.hpp"
-#include "src/mc/remote/RemotePtr.hpp"
+#include "src/mc/mc_config.hpp"
 
 #include <algorithm>
 #include <exception>
+#include <memory>
+#include <numeric>
 #include <vector>
 
 namespace simgrid::mc {
@@ -56,13 +58,16 @@ class ActorState {
    */
   std::vector<std::shared_ptr<Transition>> pending_transitions_;
 
+  /** The ID of that actor (not an aid_t to save space in memory) */
+  const short aid_;
+
   /* Possible exploration status of an actor transition in a state.
    * Either the checker did not consider the transition, or it was considered and still to do, or considered and
    * done.
    */
   enum class InterleavingType {
     /** This actor transition is not considered by the checker (yet?) */
-    disabled = 0,
+    unknown = 0,
     /** The checker algorithm decided that this actor transitions should be done at some point */
     todo,
     /** The checker algorithm decided that this should be done, but it was done in the meanwhile */
@@ -70,44 +75,42 @@ class ActorState {
   };
 
   /** Exploration control information */
-  InterleavingType state_ = InterleavingType::disabled;
-
-  /** The ID of that actor */
-  const aid_t aid_;
-
-  /** Number of times that the actor was considered to be executed in previous explorations of the state space */
-  unsigned int times_considered_ = 0;
-  /** Maximal amount of times that the actor can be considered for execution in this state.
-   * If times_considered==max_consider, we fully explored that part of the state space */
-  unsigned int max_consider_ = 0;
+  InterleavingType state_ = InterleavingType::unknown;
 
   /** Whether that actor is initially enabled in this state */
   bool enabled_;
 
+  /** Number of times that the actor was considered to be executed in previous explorations of the state space */
+  unsigned char times_considered_ = 0;
+  /** Maximal amount of times that the actor can be considered for execution in this state.
+   * If times_considered==max_consider, we fully explored that part of the state space */
+  unsigned char max_consider_ = 0;
+
 public:
   ActorState(aid_t aid, bool enabled, unsigned int max_consider) : ActorState(aid, enabled, max_consider, {}) {}
 
   ActorState(aid_t aid, bool enabled, unsigned int max_consider, std::vector<std::shared_ptr<Transition>> transitions)
-      : pending_transitions_(std::move(transitions)), aid_(aid), max_consider_(max_consider), enabled_(enabled)
+      : pending_transitions_(std::move(transitions)), aid_(aid), enabled_(enabled), max_consider_(max_consider)
   {
+    pending_transitions_.shrink_to_fit();
   }
 
-  unsigned int do_consider()
+  unsigned char do_consider()
   {
     if (max_consider_ <= times_considered_ + 1)
       mark_done();
     return times_considered_++;
   }
-  unsigned int get_max_considered() const { return max_consider_; }
-  unsigned int get_times_considered() const { return times_considered_; }
-  unsigned int get_times_not_considered() const { return max_consider_ - times_considered_; }
+  unsigned char get_max_considered() const { return max_consider_; }
+  unsigned char get_times_considered() const { return times_considered_; }
+  unsigned char get_times_not_considered() const { return max_consider_ - times_considered_; }
   bool has_more_to_consider() const { return get_times_not_considered() > 0; }
-  aid_t get_aid() const { return aid_; }
+  short get_aid() const { return aid_; }
 
   /* returns whether the actor is marked as enabled in the application side */
   bool is_enabled() const { return enabled_; }
   /* returns whether the actor is marked as disabled by the exploration algorithm */
-  bool is_disabled() const { return this->state_ == InterleavingType::disabled; }
+  bool is_unknown() const { return this->state_ == InterleavingType::unknown; }
   bool is_done() const { return this->state_ == InterleavingType::done; }
   bool is_todo() const { return this->state_ == InterleavingType::todo; }
   /** Mark that we should try executing this process at some point in the future of the checker algorithm */
@@ -141,22 +144,29 @@ public:
     // The formula satisfies both of the above conditions:
     //
     // > std::clamp(times_considered_, 0u, max_consider_ - 1)
-    return get_transition(std::clamp(times_considered_, 0u, max_consider_ - 1));
+    return get_transition(times_considered_ > max_consider_ - 1 ? max_consider_ - 1 : times_considered_);
   }
 
-  std::shared_ptr<Transition> get_transition(unsigned times_considered) const
+  std::shared_ptr<Transition> get_transition(unsigned char times_considered) const
   {
+    xbt_assert(
+        times_considered < this->pending_transitions_.size() || _sg_mc_debug,
+        "You must activate --cfg=model-check/debug:true to see the debug and error messages involving transitions.");
     xbt_assert(times_considered < this->pending_transitions_.size(),
-               "Actor %ld does not have a state available transition with `times_considered = %u`,\n"
-               "yet one was asked for",
-               aid_, times_considered);
+               "Actor %d does not have a state available transition with `times_considered = %u`,\n"
+               "yet one was asked for. Pending transitions are: %s",
+               aid_, times_considered,
+               std::accumulate(
+                   pending_transitions_.begin(), pending_transitions_.end(), std::string(),
+                   [](std::string a, std::shared_ptr<Transition> b) { return std::move(a) + ';' + b->to_string(); })
+                   .c_str());
     return this->pending_transitions_[times_considered];
   }
 
-  void set_transition(std::shared_ptr<Transition> t, unsigned times_considered)
+  void set_transition(std::shared_ptr<Transition> t, unsigned char times_considered)
   {
     xbt_assert(times_considered < this->pending_transitions_.size(),
-               "Actor %ld does not have a state available transition with `times_considered = %u`, "
+               "Actor %d does not have a state available transition with `times_considered = %u`, "
                "yet one was attempted to be set",
                aid_, times_considered);
     this->pending_transitions_[times_considered] = std::move(t);
