@@ -1,4 +1,4 @@
-/* Copyright (c) 2015-2023. The SimGrid Team. All rights reserved.          */
+/* Copyright (c) 2015-2025. The SimGrid Team. All rights reserved.          */
 
 /* This program is free software; you can redistribute it and/or modify it
  * under the terms of the license (GNU LGPL) which comes with this package. */
@@ -20,9 +20,10 @@ class ActorJoinTransition : public Transition {
   aid_t target_;
 
 public:
-  ActorJoinTransition(aid_t issuer, int times_considered, std::stringstream& stream);
+  ActorJoinTransition(aid_t issuer, int times_considered, mc::Channel& channel);
   std::string to_string(bool verbose) const override;
   bool depends(const Transition* other) const override;
+  bool can_be_co_enabled(const Transition* other) const override;
   bool reversible_race(const Transition* other) const override;
 
   bool get_timeout() const { return timeout_; }
@@ -33,12 +34,26 @@ public:
 class ActorSleepTransition : public Transition {
 
 public:
-  ActorSleepTransition(aid_t issuer, int times_considered, std::stringstream& stream);
+  ActorSleepTransition(aid_t issuer, int times_considered, mc::Channel& channel);
   std::string to_string(bool verbose) const override;
   bool depends(const Transition* other) const override;
   bool reversible_race(const Transition* other) const override;
 };
 
+class ActorCreateTransition : public Transition {
+  aid_t child_;
+
+public:
+  ActorCreateTransition(aid_t issuer, int times_considered, mc::Channel& channel);
+  std::string to_string(bool verbose) const override;
+  bool depends(const Transition* other) const override;
+  bool can_be_co_enabled(const Transition* other) const override;
+  bool reversible_race(const Transition* other) const override;
+
+  /** ID of the created actor */
+  aid_t get_child() const { return child_; }
+};
+
 } // namespace simgrid::mc
 
 #endif
