@@ -1,21 +1,23 @@
-/* Copyright (c) 2015-2023. The SimGrid Team. All rights reserved.          */
+/* Copyright (c) 2015-2025. The SimGrid Team. All rights reserved.          */
 
 /* This program is free software; you can redistribute it and/or modify it
  * under the terms of the license (GNU LGPL) which comes with this package. */
 
 #include "src/mc/transition/TransitionObjectAccess.hpp"
+#include "src/mc/remote/Channel.hpp"
 #include "xbt/asserts.h"
-#include "xbt/log.h"
 #include <xbt/string.hpp>
 
 namespace simgrid::mc {
 
-ObjectAccessTransition::ObjectAccessTransition(aid_t issuer, int times_considered, std::stringstream& stream)
+ObjectAccessTransition::ObjectAccessTransition(aid_t issuer, int times_considered, mc::Channel& channel)
     : Transition(Type::OBJECT_ACCESS, issuer, times_considered)
 {
-  short s;
-  xbt_assert(stream >> s >> objaddr_ >> objname_ >> file_ >> line_);
-  access_type_ = static_cast<simgrid::mc::ObjectAccessType>(s);
+  access_type_ = static_cast<simgrid::mc::ObjectAccessType>(channel.unpack<short>());
+  objaddr_     = channel.unpack<void*>();
+  objname_     = channel.unpack<std::string>();
+  file_        = channel.unpack<std::string>();
+  line_        = channel.unpack<int>();
 }
 std::string ObjectAccessTransition::to_string(bool verbose) const
 {
@@ -41,8 +43,8 @@ bool ObjectAccessTransition::depends(const Transition* o) const
   if (o->aid_ == aid_)
     return true;
 
-  if (const auto* other = dynamic_cast<const ObjectAccessTransition*>(o))
-    return objaddr_ == other->objaddr_; // dependent only if it's an access to the same object
+  if (o->type_ == Type::OBJECT_ACCESS) // dependent only if it's an access to the same object
+    return objaddr_ == static_cast<const ObjectAccessTransition*>(o)->objaddr_;
   return false;
 }
 
