@@ -1,4 +1,4 @@
-/* Copyright (c) 2015-2023. The SimGrid Team. All rights reserved.          */
+/* Copyright (c) 2015-2025. The SimGrid Team. All rights reserved.          */
 
 /* This program is free software; you can redistribute it and/or modify it
  * under the terms of the license (GNU LGPL) which comes with this package. */
@@ -16,7 +16,7 @@ class RandomTransition : public Transition {
 
 public:
   std::string to_string(bool verbose) const override;
-  RandomTransition(aid_t issuer, int times_considered, std::stringstream& stream);
+  RandomTransition(aid_t issuer, int times_considered, mc::Channel& channel);
   bool depends(const Transition* other) const override
   {
     if (other->type_ < type_)
