@@ -1,4 +1,4 @@
-/* Copyright (c) 2006-2023. The SimGrid Team. All rights reserved.          */
+/* Copyright (c) 2006-2025. The SimGrid Team. All rights reserved.          */
 
 /* This program is free software; you can redistribute it and/or modify it
  * under the terms of the license (GNU LGPL) which comes with this package. */
@@ -13,9 +13,14 @@
 
 #include "src/kernel/EngineImpl.hpp"
 #include "src/kernel/actor/ActorImpl.hpp"
+#include "src/kernel/actor/SimcallObserver.hpp"
 #include "src/kernel/resource/HostImpl.hpp"
 #include "src/mc/mc.h"
 #include "src/mc/mc_replay.hpp"
+#include "xbt/asserts.h"
+#include "xbt/asserts.hpp"
+#include "xbt/backtrace.hpp"
+#include "xbt/log.h"
 
 #include <algorithm>
 
@@ -71,9 +76,15 @@ ActorPtr Actor::start(const std::function<void()>& code)
 
 ActorPtr Actor::create(const std::string& name, s4u::Host* host, const std::function<void()>& code)
 {
-  const kernel::actor::ActorImpl* self = kernel::actor::ActorImpl::self();
-  kernel::actor::ActorImpl* actor =
-      kernel::actor::simcall_answered([self, &name, host, &code] { return self->init(name, host)->start(code); });
+  kernel::actor::ActorImpl* self = kernel::actor::ActorImpl::self();
+  kernel::actor::ActorCreateSimcall observer{self};
+  kernel::actor::ActorImpl* actor = kernel::actor::simcall_answered(
+      [self, &name, host, &code, &observer] {
+        auto child = self->init(name, host)->start(code);
+        observer.set_child(child->get_pid());
+        return child;
+      },
+      &observer);
 
   return actor->get_iface();
 }
@@ -81,9 +92,7 @@ ActorPtr Actor::create(const std::string& name, s4u::Host* host, const std::func
 ActorPtr Actor::create(const std::string& name, s4u::Host* host, const std::string& function,
                        std::vector<std::string> args)
 {
-  const simgrid::kernel::actor::ActorCodeFactory& factory =
-      simgrid::kernel::EngineImpl::get_instance()->get_function(function);
-  return create(name, host, factory(std::move(args)));
+  return s4u::Engine::get_instance()->add_actor(name, host, function, std::move(args));
 }
 
 void intrusive_ptr_add_ref(const Actor* actor)
@@ -112,7 +121,7 @@ void Actor::join(double timeout) const
   const kernel::actor::ActorImpl* target = pimpl_;
   kernel::actor::ActorJoinSimcall observer{issuer, get_impl(), timeout};
 
-  kernel::actor::simcall_blocking(
+  kernel::actor::simcall_blocking<void>(
       [issuer, target, timeout] {
         if (target->wannadie()) {
           // The joined actor is already finished, just wake up the issuer right away
@@ -215,13 +224,16 @@ void Actor::suspend()
   kernel::actor::ActorImpl* target = pimpl_;
   s4u::Actor::on_suspend(*this);
   s4u::Actor::on_this_suspend(*this);
-  kernel::actor::simcall_blocking([issuer, target]() {
-    target->suspend();
-    if (target != issuer) {
-      /* If we are suspending ourselves, then just do not finish the simcall now */
-      issuer->simcall_answer();
-    }
-  });
+  kernel::actor::ActorSuspendSimcall observer(issuer);
+  kernel::actor::simcall_blocking<void>(
+      [issuer, target]() {
+        target->suspend();
+        if (target != issuer) {
+          /* If we are suspending ourselves, then just do not finish the simcall now */
+          issuer->simcall_answer();
+        }
+      },
+      &observer);
 }
 
 void Actor::resume()
@@ -333,7 +345,7 @@ void sleep_for(double duration)
   Actor::on_sleep(*issuer->get_ciface());
   issuer->get_ciface()->on_this_sleep(*issuer->get_ciface());
 
-  kernel::actor::simcall_blocking(
+  kernel::actor::simcall_blocking<void>(
       [issuer, duration]() {
         if (MC_is_active() || MC_record_replay_is_active()) {
           // MC_process_clock_add(issuer, duration); // BUG: Makes the exploration loop
@@ -390,25 +402,25 @@ ExecPtr exec_init(double flops_amount)
 ExecPtr exec_init(const std::vector<s4u::Host*>& hosts, const std::vector<double>& flops_amounts,
                   const std::vector<double>& bytes_amounts)
 {
-  xbt_assert(not hosts.empty(), "Your parallel executions must span over at least one host.");
-  xbt_assert(hosts.size() == flops_amounts.size() || flops_amounts.empty(),
-             "Host count (%zu) does not match flops_amount count (%zu).", hosts.size(), flops_amounts.size());
-  xbt_assert(hosts.size() * hosts.size() == bytes_amounts.size() || bytes_amounts.empty(),
-             "bytes_amounts must be a matrix of size host_count * host_count (%zu*%zu), but it's of size %zu.",
-             hosts.size(), hosts.size(), bytes_amounts.size());
+  xbt_enforce(not hosts.empty(), "Your parallel executions must span over at least one host.");
+  xbt_enforce(hosts.size() == flops_amounts.size() || flops_amounts.empty(),
+              "Host count (%zu) does not match flops_amount count (%zu).", hosts.size(), flops_amounts.size());
+  xbt_enforce(hosts.size() * hosts.size() == bytes_amounts.size() || bytes_amounts.empty(),
+              "bytes_amounts must be a matrix of size host_count * host_count (%zu*%zu), but it's of size %zu.",
+              hosts.size(), hosts.size(), bytes_amounts.size());
   /* Check that we are not mixing VMs and PMs in the parallel task */
   bool is_a_vm = (nullptr != dynamic_cast<VirtualMachine*>(hosts.front()));
-  xbt_assert(std::all_of(hosts.begin(), hosts.end(),
-                         [is_a_vm](s4u::Host* elm) {
-                           bool tmp_is_a_vm = (nullptr != dynamic_cast<VirtualMachine*>(elm));
-                           return is_a_vm == tmp_is_a_vm;
-                         }),
-             "parallel_execute: mixing VMs and PMs is not supported (yet).");
+  xbt_enforce(std::all_of(hosts.begin(), hosts.end(),
+                          [is_a_vm](s4u::Host* elm) {
+                            bool tmp_is_a_vm = (nullptr != dynamic_cast<VirtualMachine*>(elm));
+                            return is_a_vm == tmp_is_a_vm;
+                          }),
+              "parallel_execute: mixing VMs and PMs is not supported (yet).");
   /* checking for infinite values */
-  xbt_assert(std::all_of(flops_amounts.begin(), flops_amounts.end(), [](double elm) { return std::isfinite(elm); }),
-             "flops_amounts comprises infinite values!");
-  xbt_assert(std::all_of(bytes_amounts.begin(), bytes_amounts.end(), [](double elm) { return std::isfinite(elm); }),
-             "flops_amounts comprises infinite values!");
+  xbt_enforce(std::all_of(flops_amounts.begin(), flops_amounts.end(), [](double elm) { return std::isfinite(elm); }),
+              "flops_amounts comprises infinite values!");
+  xbt_enforce(std::all_of(bytes_amounts.begin(), bytes_amounts.end(), [](double elm) { return std::isfinite(elm); }),
+              "flops_amounts comprises infinite values!");
 
   return Exec::init()->set_flops_amounts(flops_amounts)->set_bytes_amounts(bytes_amounts)->set_hosts(hosts);
 }
@@ -430,6 +442,10 @@ aid_t get_ppid()
 {
   return simgrid::kernel::actor::ActorImpl::self()->get_ppid();
 }
+Engine* get_engine()
+{
+  return simgrid::s4u::Engine::get_instance();
+}
 
 std::string get_name()
 {
@@ -450,9 +466,7 @@ Host* get_host()
 void suspend()
 {
   kernel::actor::ActorImpl* self = simgrid::kernel::actor::ActorImpl::self();
-  s4u::Actor::on_suspend(*self->get_ciface());
-  self->get_ciface()->on_this_suspend(*self->get_ciface());
-  kernel::actor::simcall_blocking([self] { self->suspend(); });
+  self->get_iface()->suspend();
 }
 
 void exit()
@@ -512,6 +526,12 @@ void sg_actor_start_(sg_actor_t actor, xbt_main_func_t code, int argc, const cha
   actor->start(function);
 }
 
+void sg_actor_start_voidp(sg_actor_t actor, void_f_pvoid_t code, void* param)
+{
+  simgrid::kernel::actor::ActorCode function;
+  actor->start(code, param);
+}
+
 sg_actor_t sg_actor_create_(const char* name, sg_host_t host, xbt_main_func_t code, int argc, const char* const* argv)
 {
   simgrid::kernel::actor::ActorCode function = simgrid::xbt::wrap_main(code, argc, argv);
@@ -589,11 +609,6 @@ const char* sg_actor_get_property_value(const_sg_actor_t actor, const char* name
   return actor->get_property(name);
 }
 
-/**
- * @brief Return the list of properties
- *
- * This function returns all the parameters associated with an actor
- */
 xbt_dict_t sg_actor_get_properties(const_sg_actor_t actor)
 {
   xbt_assert(actor != nullptr, "Invalid parameter: First argument must not be nullptr");
@@ -606,6 +621,26 @@ xbt_dict_t sg_actor_get_properties(const_sg_actor_t actor)
   }
   return as_dict;
 }
+const char** sg_actor_get_property_names(const_sg_actor_t actor, int* size)
+{
+  const std::unordered_map<std::string, std::string>* props = actor->get_properties();
+
+  if (props == nullptr) {
+    if (size)
+      *size = 0;
+    return nullptr;
+  }
+
+  const char** res = (const char**)xbt_malloc(sizeof(char*) * (props->size() + 1));
+  if (size)
+    *size = props->size();
+  int i = 0;
+  for (auto const& [key, _] : *props)
+    res[i++] = key.c_str();
+  res[i] = nullptr;
+
+  return res;
+}
 
 /**
  * @brief Suspend the actor.
@@ -724,22 +759,14 @@ void sg_actor_sleep_until(double wakeup_time)
   simgrid::s4u::this_actor::sleep_until(wakeup_time);
 }
 
-sg_actor_t sg_actor_attach(const char* name, void* data, sg_host_t host, xbt_dict_t properties)
+sg_actor_t sg_actor_attach_pthread(const char* name, void* data, sg_host_t host)
 {
-  xbt_assert(host != nullptr, "Invalid parameters: host and code params must not be nullptr");
-  std::unordered_map<std::string, std::string> props;
-  xbt_dict_cursor_t cursor = nullptr;
-  char* key;
-  char* value;
-  xbt_dict_foreach (properties, cursor, key, value)
-    props[key] = value;
-  xbt_dict_free(&properties);
+  xbt_assert(host != nullptr, "Invalid parameters: host must not be nullptr");
 
   /* Let's create the actor: SIMIX may decide to start it right now, even before returning the flow control to us */
   simgrid::kernel::actor::ActorImpl* actor = nullptr;
   try {
     actor = simgrid::kernel::actor::ActorImpl::attach(name, data, host).get();
-    actor->set_properties(props);
   } catch (simgrid::HostFailureException const&) {
     xbt_die("Could not attach");
   }
@@ -747,6 +774,10 @@ sg_actor_t sg_actor_attach(const char* name, void* data, sg_host_t host, xbt_dic
   simgrid::s4u::this_actor::yield();
   return actor->get_ciface();
 }
+sg_actor_t sg_actor_attach(const char* name, void* data, sg_host_t host, xbt_dict_t properties)
+{
+  return sg_actor_attach_pthread(name, data, host);
+}
 
 void sg_actor_detach()
 {
