@@ -1,4 +1,4 @@
-/* Copyright (c) 2018-2023. The SimGrid Team. All rights reserved.          */
+/* Copyright (c) 2018-2025. The SimGrid Team. All rights reserved.          */
 
 /* This program is free software; you can redistribute it and/or modify it
  * under the terms of the license (GNU LGPL) which comes with this package. */
@@ -11,7 +11,8 @@
 #include "src/kernel/actor/SynchroObserver.hpp"
 #include "src/mc/mc_replay.hpp"
 
-XBT_LOG_NEW_DEFAULT_SUBCATEGORY(s4u_barrier, s4u, "S4U barrier");
+XBT_LOG_NEW_SUBCATEGORY(s4u_synchro, s4u, "S4U synchronization objects");
+XBT_LOG_NEW_DEFAULT_SUBCATEGORY(s4u_barrier, s4u_synchro, "S4U barrier");
 
 namespace simgrid::s4u {
 
@@ -47,8 +48,11 @@ int Barrier::wait()
   } else { // Do it in one simcall only
     kernel::activity::BarrierAcquisitionImpl* acqui = nullptr; // unused here, but must be typed to pick the right ctor
     kernel::actor::BarrierObserver observer{issuer, mc::Transition::Type::BARRIER_WAIT, acqui};
-    return kernel::actor::simcall_blocking([issuer, this] { pimpl_->acquire_async(issuer)->wait_for(issuer, -1); },
-                                           &observer);
+    return kernel::actor::simcall_blocking<bool>([issuer, &observer, this] {
+      auto p = pimpl_->acquire_async(issuer);
+      p->wait_for(issuer, -1);
+      observer.set_result(p->get_barrier()->was_last());
+    }, &observer);
   }
 }
 
