@@ -1,9 +1,8 @@
-/* Copyright (c) 2006-2023. The SimGrid Team. All rights reserved.          */
+/* Copyright (c) 2006-2025. The SimGrid Team. All rights reserved.          */
 
 /* This program is free software; you can redistribute it and/or modify it
  * under the terms of the license (GNU LGPL) which comes with this package. */
 
-#include <cmath>
 #include <simgrid/Exception.hpp>
 #include <simgrid/comm.h>
 #include <simgrid/s4u/ActivitySet.hpp>
@@ -13,41 +12,80 @@
 
 #include "src/kernel/activity/CommImpl.hpp"
 #include "src/kernel/actor/ActorImpl.hpp"
-#include "src/kernel/actor/SimcallObserver.hpp"
+#include "src/kernel/actor/WaitTestObserver.hpp"
 #include "src/mc/mc.h"
 #include "src/mc/mc_replay.hpp"
 
+#include <boost/core/demangle.hpp>
+#include <cmath>
+
 XBT_LOG_NEW_DEFAULT_SUBCATEGORY(s4u_comm, s4u_activity, "S4U asynchronous communications");
 
 namespace simgrid::s4u {
 xbt::signal<void(Comm const&)> Comm::on_send;
 xbt::signal<void(Comm const&)> Comm::on_recv;
 
+template <> xbt::signal<void(Comm&)> Activity_T<Comm>::on_veto             = xbt::signal<void(Comm&)>();
+template <> xbt::signal<void(Comm const&)> Activity_T<Comm>::on_start      = xbt::signal<void(Comm const&)>();
+template <> xbt::signal<void(Comm const&)> Activity_T<Comm>::on_completion = xbt::signal<void(Comm const&)>();
+template <> xbt::signal<void(Comm const&)> Activity_T<Comm>::on_suspend    = xbt::signal<void(Comm const&)>();
+template <> xbt::signal<void(Comm const&)> Activity_T<Comm>::on_resume     = xbt::signal<void(Comm const&)>();
+template <> void Activity_T<Comm>::fire_on_start() const
+{
+  on_start(static_cast<const Comm&>(*this));
+}
+template <> void Activity_T<Comm>::fire_on_completion() const
+{
+  on_completion(static_cast<const Comm&>(*this));
+}
+template <> void Activity_T<Comm>::fire_on_suspend() const
+{
+  on_suspend(static_cast<const Comm&>(*this));
+}
+template <> void Activity_T<Comm>::fire_on_resume() const
+{
+  on_resume(static_cast<const Comm&>(*this));
+}
+template <> void Activity_T<Comm>::fire_on_veto()
+{
+  on_veto(static_cast<Comm&>(*this));
+}
+template <> void Activity_T<Comm>::on_start_cb(const std::function<void(Comm const&)>& cb)
+{
+  on_start.connect(cb);
+}
+template <> void Activity_T<Comm>::on_completion_cb(const std::function<void(Comm const&)>& cb)
+{
+  on_completion.connect(cb);
+}
+template <> void Activity_T<Comm>::on_suspend_cb(const std::function<void(Comm const&)>& cb)
+{
+  on_suspend.connect(cb);
+}
+template <> void Activity_T<Comm>::on_resume_cb(const std::function<void(Comm const&)>& cb)
+{
+  on_resume.connect(cb);
+}
+template <> void Activity_T<Comm>::on_veto_cb(const std::function<void(Comm&)>& cb)
+{
+  on_veto.connect(cb);
+}
+
+void Comm::fire_on_completion_for_real() const
+{
+  Activity_T<Comm>::fire_on_completion();
+}
+void Comm::fire_on_this_completion_for_real() const
+{
+  Activity_T<Comm>::fire_on_this_completion();
+}
+
 CommPtr Comm::set_copy_data_callback(const std::function<void(kernel::activity::CommImpl*, void*, size_t)>& callback)
 {
   copy_data_function_ = callback;
   return this;
 }
 
-void Comm::copy_buffer_callback(kernel::activity::CommImpl* comm, void* buff,
-                                size_t buff_size) // XBT_ATTRIB_DEPRECATED_v338
-{
-  XBT_DEBUG("Copy the data over");
-  memcpy(comm->dst_buff_, buff, buff_size);
-  if (comm->is_detached()) { // if this is a detached send, the source buffer was duplicated by SMPI sender to make the
-                             // original buffer available to the application ASAP
-    xbt_free(buff);
-    comm->src_buff_ = nullptr;
-  }
-}
-
-void Comm::copy_pointer_callback(kernel::activity::CommImpl* comm, void* buff,
-                                 size_t buff_size) // XBT_ATTRIB_DEPRECATED_v338
-{
-  xbt_assert((buff_size == sizeof(void*)), "Cannot copy %zu bytes: must be sizeof(void*)", buff_size);
-  *(void**)(comm->dst_buff_) = buff;
-}
-
 Comm::~Comm()
 {
   if (state_ == State::STARTED && not detached_ &&
@@ -98,10 +136,12 @@ void Comm::send(kernel::actor::ActorImpl* sender, const Mailbox* mbox, double ta
     simgrid::kernel::actor::CommIsendSimcall observer(sender, mbox->get_impl(), task_size, rate,
                                                       static_cast<unsigned char*>(src_buff), src_buff_size, match_fun,
                                                       nullptr, copy_data_fun, data, false, "Isend");
-    simgrid::kernel::actor::simcall_blocking([&observer, timeout] {
-      simgrid::kernel::activity::ActivityImplPtr comm = simgrid::kernel::activity::CommImpl::isend(&observer);
-      comm->wait_for(observer.get_issuer(), timeout);
-    });
+    simgrid::kernel::actor::simcall_blocking<void>(
+        [&observer, timeout] {
+          simgrid::kernel::activity::ActivityImplPtr comm = simgrid::kernel::activity::CommImpl::isend(&observer);
+          comm->wait_for(observer.get_issuer(), timeout);
+        },
+        &observer);
   }
 }
 
@@ -138,13 +178,15 @@ void Comm::recv(kernel::actor::ActorImpl* receiver, const Mailbox* mbox, void* d
       throw simgrid::TimeoutException(XBT_THROW_POINT, "Timeouted");
     }
     comm = nullptr;
-  } else {
+  } else { // Non-MC path
     simgrid::kernel::actor::CommIrecvSimcall observer(receiver, mbox->get_impl(), static_cast<unsigned char*>(dst_buff),
                                                       dst_buff_size, match_fun, copy_data_fun, data, rate, "Irecv");
-    simgrid::kernel::actor::simcall_blocking([&observer, timeout] {
-      simgrid::kernel::activity::ActivityImplPtr comm = simgrid::kernel::activity::CommImpl::irecv(&observer);
-      comm->wait_for(observer.get_issuer(), timeout);
-    });
+    simgrid::kernel::actor::simcall_blocking<void>(
+        [&observer, timeout] {
+          simgrid::kernel::activity::ActivityImplPtr comm = simgrid::kernel::activity::CommImpl::irecv(&observer);
+          comm->wait_for(observer.get_issuer(), timeout);
+        },
+        &observer);
   }
 }
 
@@ -341,10 +383,10 @@ Comm* Comm::do_start()
                                              rate_,
                                              static_cast<unsigned char*>(src_buff_),
                                              src_buff_size_,
-                                             match_fun_,
-                                             clean_fun_,
+                                             nullptr,
+                                             get_clean_function(),
                                              copy_data_function_,
-                                             get_data<void>(),
+                                             nullptr,
                                              detached_,
                                              "Isend"};
     pimpl_ = kernel::actor::simcall_answered([&observer] { return kernel::activity::CommImpl::isend(&observer); },
@@ -357,9 +399,9 @@ Comm* Comm::do_start()
                                              mailbox_->get_impl(),
                                              static_cast<unsigned char*>(dst_buff_),
                                              &dst_buff_size_,
-                                             match_fun_,
+                                             nullptr,
                                              copy_data_function_,
-                                             get_data<void>(),
+                                             nullptr,
                                              rate_,
                                              "Irecv"};
     pimpl_ = kernel::actor::simcall_answered([&observer] { return kernel::activity::CommImpl::irecv(&observer); },
@@ -385,17 +427,7 @@ Comm* Comm::do_start()
   return this;
 }
 
-Comm* Comm::detach()
-{
-  xbt_assert(state_ == State::INITED || state_ == State::STARTING,
-             "You cannot use %s() once your communication is %s (not implemented)", __func__, get_state_str());
-  xbt_assert(dst_buff_ == nullptr && dst_buff_size_ == 0, "You can only detach sends, not recvs");
-  detached_ = true;
-  start();
-  return this;
-}
-
-ssize_t Comm::test_any(const std::vector<CommPtr>& comms) // XBT_ATTRIB_DEPRECATED_v339
+ssize_t Comm::test_any(const std::vector<CommPtr>& comms) // XBT_ATTRIB_DEPRECATED_v401
 {
   std::vector<kernel::activity::ActivityImpl*> ractivities(comms.size());
   std::transform(begin(comms), end(comms), begin(ractivities), [](const CommPtr& act) { return act->pimpl_.get(); });
@@ -434,14 +466,14 @@ Comm* Comm::wait_for(double timeout)
       } else if (src_buff_ != nullptr) {
         on_send(*this);
         on_this_send(*this);
-        send(sender_, mailbox_, remains_, rate_, src_buff_, src_buff_size_, match_fun_, copy_data_function_,
+        send(sender_, mailbox_, remains_, rate_, src_buff_, src_buff_size_, nullptr, copy_data_function_,
              get_data<void>(), timeout);
 
       } else { // Receiver
         on_recv(*this);
         on_this_recv(*this);
-        recv(receiver_, mailbox_, dst_buff_, &dst_buff_size_, match_fun_, copy_data_function_, get_data<void>(),
-             timeout, rate_);
+        recv(receiver_, mailbox_, dst_buff_, &dst_buff_size_, nullptr, copy_data_function_, get_data<void>(), timeout,
+             rate_);
       }
       break;
     case State::STARTED:
@@ -470,7 +502,7 @@ Comm* Comm::wait_for(double timeout)
   return this;
 }
 
-ssize_t Comm::deprecated_wait_any_for(const std::vector<CommPtr>& comms, double timeout) // XBT_ATTRIB_DEPRECATED_v339
+ssize_t Comm::deprecated_wait_any_for(const std::vector<CommPtr>& comms, double timeout) // XBT_ATTRIB_DEPRECATED_v401
 {
   if (comms.empty())
     return -1;
@@ -495,14 +527,14 @@ ssize_t Comm::deprecated_wait_any_for(const std::vector<CommPtr>& comms, double
   return -1;
 }
 
-void Comm::wait_all(const std::vector<CommPtr>& comms) // XBT_ATTRIB_DEPRECATED_v339
+void Comm::wait_all(const std::vector<CommPtr>& comms) // XBT_ATTRIB_DEPRECATED_v401
 {
   // TODO: this should be a simcall or something
   for (const auto& comm : comms)
     comm->wait();
 }
 
-size_t Comm::wait_all_for(const std::vector<CommPtr>& comms, double timeout) // XBT_ATTRIB_DEPRECATED_v339
+size_t Comm::wait_all_for(const std::vector<CommPtr>& comms, double timeout) // XBT_ATTRIB_DEPRECATED_v401
 {
   if (timeout < 0.0) {
     for (const auto& comm : comms)
@@ -563,7 +595,7 @@ sg_error_t sg_comm_wait_for(sg_comm_t comm, double timeout)
   return status;
 }
 
-void sg_comm_wait_all(sg_comm_t* comms, size_t count) // XBT_ATTRIB_DEPRECATED_v339
+void sg_comm_wait_all(sg_comm_t* comms, size_t count) // XBT_ATTRIB_DEPRECATED_v401
 {
   simgrid::s4u::ActivitySet as;
   for (size_t i = 0; i < count; i++)
@@ -572,7 +604,7 @@ void sg_comm_wait_all(sg_comm_t* comms, size_t count) // XBT_ATTRIB_DEPRECATED_v
   as.wait_all();
 }
 
-ssize_t sg_comm_wait_any(sg_comm_t* comms, size_t count) // XBT_ATTRIB_DEPRECATED_v339
+ssize_t sg_comm_wait_any(sg_comm_t* comms, size_t count) // XBT_ATTRIB_DEPRECATED_v401
 {
   std::vector<simgrid::s4u::CommPtr> s4u_comms;
   for (size_t i = 0; i < count; i++)
@@ -586,7 +618,7 @@ ssize_t sg_comm_wait_any(sg_comm_t* comms, size_t count) // XBT_ATTRIB_DEPRECATE
   return pos;
 }
 
-ssize_t sg_comm_wait_any_for(sg_comm_t* comms, size_t count, double timeout) // XBT_ATTRIB_DEPRECATED_v339
+ssize_t sg_comm_wait_any_for(sg_comm_t* comms, size_t count, double timeout) // XBT_ATTRIB_DEPRECATED_v401
 {
   std::vector<simgrid::s4u::CommPtr> s4u_comms;
   for (size_t i = 0; i < count; i++)
