@@ -1,11 +1,11 @@
-/* Copyright (c) 2019-2023. The SimGrid Team. All rights reserved.          */
+/* Copyright (c) 2019-2025. The SimGrid Team. All rights reserved.          */
 
 /* This program is free software; you can redistribute it and/or modify it
  * under the terms of the license (GNU LGPL) which comes with this package. */
 
 #include <simgrid/s4u/Disk.hpp>
 #include <simgrid/s4u/Io.hpp>
-#include <simgrid/simix.hpp>
+#include <simgrid/simcall.hpp>
 
 #include "src/kernel/resource/DiskImpl.hpp"
 
@@ -16,8 +16,11 @@ template class xbt::Extendable<s4u::Disk>;
 namespace s4u {
 
 xbt::signal<void(Disk&)> Disk::on_creation;
-xbt::signal<void(Disk const&)> Disk::on_destruction;
 xbt::signal<void(Disk const&)> Disk::on_onoff;
+xbt::signal<void(Disk const&)> Disk::on_read_bandwidth_change;
+xbt::signal<void(Disk const&)> Disk::on_write_bandwidth_change;
+xbt::signal<void(kernel::resource::DiskAction&, kernel::resource::Action::State)> Disk::on_io_state_change;
+xbt::signal<void(Disk const&)> Disk::on_destruction;
 
 const std::string& Disk::get_name() const
 {
@@ -68,6 +71,36 @@ Host* Disk::get_host() const
   return pimpl_->get_host();
 }
 
+void Disk::turn_on()
+{
+  if (not is_on()) {
+    kernel::actor::simcall_answered([this] {
+      this->pimpl_->turn_on();
+      on_onoff(*this);
+      on_this_onoff(*this);
+    });
+  }
+}
+
+void Disk::turn_off()
+{
+  if (is_on()) {
+    kernel::actor::simcall_answered([this] {
+      this->pimpl_->turn_off();
+
+      on_onoff(*this);
+      on_this_onoff(*this);
+    });
+  }
+}
+
+bool Disk::is_on() const
+{
+  return this->pimpl_->is_on();
+}
+
+
+
 const std::unordered_map<std::string, std::string>* Disk::get_properties() const
 {
   return pimpl_->get_properties();
