@@ -1,25 +1,45 @@
 /* s4u::Engine Simulation Engine and global functions. */
 
-/* Copyright (c) 2006-2023. The SimGrid Team. All rights reserved.          */
+/* Copyright (c) 2006-2025. The SimGrid Team. All rights reserved.          */
 
 /* This program is free software; you can redistribute it and/or modify it
  * under the terms of the license (GNU LGPL) which comes with this package. */
 
+
 #include <simgrid/kernel/routing/NetPoint.hpp>
 #include <simgrid/modelchecker.h>
 #include <simgrid/s4u/Engine.hpp>
 
+#include "simgrid/forward.h"
+#include "simgrid/kernel/routing/DijkstraZone.hpp"
+#include "simgrid/kernel/routing/EmptyZone.hpp"
+#include "simgrid/kernel/routing/FloydZone.hpp"
+#include "simgrid/kernel/routing/FullZone.hpp"
+#include "simgrid/kernel/routing/StarZone.hpp"
+#include "simgrid/kernel/routing/VivaldiZone.hpp"
+#include "simgrid/kernel/routing/WifiZone.hpp"
 #include "src/instr/instr_private.hpp"
 #include "src/kernel/EngineImpl.hpp"
+#include "src/kernel/actor/ActorImpl.hpp"
 #include "src/kernel/resource/HostImpl.hpp"
 #include "src/kernel/resource/NetworkModel.hpp"
 #include "src/kernel/resource/SplitDuplexLinkImpl.hpp"
 #include "src/kernel/resource/StandardLinkImpl.hpp"
 #include "src/mc/mc.h"
 #include "src/mc/mc_replay.hpp"
+#include "src/simgrid/module.hpp"
+#include "xbt/asserts.h"
 #include "xbt/config.hpp"
 
+#if HAVE_PAPI
+#include "src/simgrid/sg_config.hpp"
+#include "src/smpi/include/smpi_actor.hpp"
+#include <papi.h>
+#endif
+
 #include <algorithm>
+#include <boost/algorithm/string/classification.hpp>
+#include <boost/algorithm/string/split.hpp>
 #include <string>
 
 XBT_LOG_NEW_CATEGORY(s4u, "Log channels of the S4U (SimGrid for you) interface");
@@ -30,12 +50,15 @@ static simgrid::kernel::actor::ActorCode maestro_code;
 namespace simgrid::s4u {
 xbt::signal<void()> Engine::on_platform_creation;
 xbt::signal<void()> Engine::on_platform_created;
+xbt::signal<void()> Engine::on_platform_sealed;
 xbt::signal<void()> Engine::on_simulation_start;
 xbt::signal<void()> Engine::on_simulation_end;
 xbt::signal<void(double)> Engine::on_time_advance;
 xbt::signal<void(void)> Engine::on_deadlock;
 
 Engine* Engine::instance_ = nullptr; /* This singleton is awful, but I don't see no other solution right now. */
+bool Engine::shutdown_ongoing_ =
+    false; // set to true just before the final shutdown, to break dependency loops in that area
 
 void Engine::initialize(int* argc, char** argv)
 {
@@ -62,6 +85,7 @@ Engine::Engine(int* argc, char** argv) : pimpl_(new kernel::EngineImpl())
 
 Engine::~Engine()
 {
+  shutdown_ongoing_ = true;
   kernel::EngineImpl::shutdown();
   Engine::instance_ = nullptr;
 }
@@ -95,6 +119,130 @@ double Engine::get_clock()
   }
 }
 
+const char* Engine::get_context_factory_name() const
+{
+  return pimpl_->context_factory_->get_name();
+}
+
+std::vector<long long> Engine::get_papi_counters()
+{
+#if HAVE_PAPI
+  if (not smpi_cfg_papi_events_file().empty()) {
+    papi_counter_t& counter_data = smpi_process()->papi_counters();
+    int event_set                = smpi_process()->papi_event_set();
+
+    if (PAPI_NULL != event_set) {
+      std::vector<long long> event_values(counter_data.size());
+      auto ret = PAPI_read(event_set, &event_values[0]);
+
+      const char* errname = "unknown";
+      switch (ret) {
+        case PAPI_EINVAL:
+          errname = "PAPI EINVAL";
+          break;
+        case PAPI_ESYS:
+          errname = "PAPI ESYS";
+          break;
+        case PAPI_ENOEVST:
+          errname = "PAPI ENOEVST";
+          break;
+      }
+      xbt_assert(ret == PAPI_OK, "Could not read PAPI counters. Got a %s error (%d).", errname, ret);
+
+      for (unsigned i = 0; i < counter_data.size(); i++) {
+        event_values[i] += counter_data[i].second;
+      }
+
+      return event_values;
+    }
+    return std::vector<long long>(0);
+  } else {
+    return std::vector<long long>(0);
+  }
+#else
+    return std::vector<long long>(0);
+#endif
+}
+
+int Engine::papi_get_num_counters()
+{
+#if HAVE_PAPI
+  papi_counter_t& counter_data = smpi_process()->papi_counters();
+  return counter_data.size();
+#else
+    xbt_die("papi_get_num_counters() cannot be used when PAPI is not compiled in.");
+#endif
+}
+
+void Engine::papi_start()
+{
+#if HAVE_PAPI
+  if (not smpi_cfg_papi_events_file().empty()) {
+    int event_set = smpi_process()->papi_event_set();
+    // PAPI_start sets everything to 0! See man(3) PAPI_start
+    if (PAPI_LOW_LEVEL_INITED == PAPI_is_initialized() && PAPI_NULL != event_set) {
+      auto ret = PAPI_start(event_set);
+
+      const char* errname = "unknown";
+      switch (ret) {
+        case PAPI_EINVAL:
+          errname = "PAPI EINVAL";
+          break;
+        case PAPI_ESYS:
+          errname = "PAPI ESYS";
+          break;
+        case PAPI_ENOEVST:
+          errname = "PAPI ENOEVST";
+          break;
+        case PAPI_ECNFLCT:
+          errname = "PAPI ECNFLCT";
+          break;
+          /* ignore PAPI_EISRUN */
+      }
+      xbt_assert(ret == PAPI_OK || ret == PAPI_EISRUN, "Could not start PAPI counters. Got a %s error (%d).", errname,
+                 ret);
+    }
+  }
+#endif
+}
+
+void Engine::papi_stop(){
+#if HAVE_PAPI
+    if (not smpi_cfg_papi_events_file().empty()) {
+
+        int event_set                       = smpi_process()->papi_event_set();
+        papi_counter_t& counter_data        = smpi_process()->papi_counters();
+        std::vector<long long> event_values = Engine::get_papi_counters();
+
+        if (PAPI_LOW_LEVEL_INITED == PAPI_is_initialized() && PAPI_NULL != event_set){
+            auto ret = PAPI_stop(event_set, &event_values[0]);
+
+            const char* errname = "unknown";
+            switch( ret ){
+            case PAPI_EINVAL:
+                errname = "PAPI EINVAL";
+                break;
+            case PAPI_ESYS:
+                errname = "PAPI ESYS";
+                break;
+            case PAPI_ENOEVST:
+                errname = "PAPI ENOEVST";
+                break;
+            case PAPI_ECNFLCT:
+                errname = "PAPI ECNFLCT";
+                break;
+                /* ignore PAPI_ENOTRUN */
+            }
+            xbt_assert(ret == PAPI_OK || ret == PAPI_ENOTRUN, "Could not stop PAPI counters. Got a %s error (%d).", errname, ret );
+
+            if( ret != PAPI_ENOTRUN )
+                for (unsigned int i = 0; i < counter_data.size(); i++)
+                    counter_data[i].second += event_values[i];
+        }
+    }
+#endif
+}
+
 void Engine::add_model(std::shared_ptr<kernel::resource::Model> model,
                        const std::vector<kernel::resource::Model*>& dependencies)
 {
@@ -105,6 +253,26 @@ const std::vector<simgrid::kernel::resource::Model*>& Engine::get_all_models() c
 {
   return pimpl_->get_all_models();
 }
+ActorPtr Engine::add_actor(const std::string& name, s4u::Host* host, const std::function<void()>& code)
+{
+  kernel::actor::ActorImpl* self = kernel::actor::ActorImpl::self();
+  kernel::actor::ActorCreateSimcall observer{self};
+  kernel::actor::ActorImpl* actor = kernel::actor::simcall_answered(
+      [self, &name, host, &code, &observer] {
+        auto child = self->init(name, host)->start(code);
+        observer.set_child(child->get_pid());
+        return child;
+      },
+      &observer);
+
+  return actor->get_iface();
+}
+ActorPtr Engine::add_actor(const std::string& name, s4u::Host* host, const std::string& function,
+                           std::vector<std::string> args)
+{
+  const simgrid::kernel::actor::ActorCodeFactory& factory = pimpl_->get_function(function);
+  return add_actor(name, host, factory(std::move(args)));
+}
 
 void Engine::load_platform(const std::string& platf) const
 {
@@ -282,12 +450,13 @@ void Engine::register_default(const std::function<void(int, char**)>& code)
 }
 void Engine::register_default(const kernel::actor::ActorCodeFactory& code)
 {
-  simgrid::kernel::actor::simcall_answered([this, &code]() { pimpl_->register_default(code); });
+  register_function("", code);
 }
 
 void Engine::register_function(const std::string& name, const kernel::actor::ActorCodeFactory& code)
 {
-  simgrid::kernel::actor::simcall_answered([this, name, &code]() { pimpl_->register_function(name, code); });
+  simgrid::kernel::actor::simcall_answered(
+      [this, name, &code]() { name.empty() ? pimpl_->register_default(code) : pimpl_->register_function(name, code); });
 }
 
 /** Load a deployment file and launch the actors that it contains
@@ -325,6 +494,31 @@ std::vector<Host*> Engine::get_filtered_hosts(const std::function<bool(Host*)>&
   return hosts;
 }
 
+std::vector<Host*> Engine::get_hosts_from_MPI_hostfile(const std::string& hostfile) const
+{
+  std::vector<Host*> hosts;
+  std::ifstream in(hostfile.c_str());
+  xbt_assert(in, "Cannot open the MPI hostfile: %s", hostfile.c_str());
+  std::string str;
+  while (std::getline(in, str)) {
+    if (not str.empty()) {
+      std::vector<std::string> tokens;
+      boost::split(tokens, str, boost::is_any_of(":"));
+      auto host = host_by_name_or_null(tokens[0]);
+      xbt_assert(host, "Host '%s' doesn't exist in this platform", tokens[0].c_str());
+      if (tokens.size() > 1)
+        for (int i = 0; i < std::stoi(tokens[1]); i++)
+          hosts.emplace_back(host);
+      else
+          hosts.emplace_back(host);
+    }
+  }
+  xbt_assert(not hosts.empty(), "The hostfile '%s' is empty", hostfile.c_str());
+  std::sort(hosts.begin(), hosts.end(), [](const auto* h1, const auto* h2) { return h1->get_name() < h2->get_name(); });
+
+  return hosts;
+}
+
 /** @brief Find a host from its name.
  *
  *  @throw std::invalid_argument if the searched host does not exist.
@@ -449,6 +643,10 @@ size_t Engine::get_actor_count() const
 {
   return pimpl_->get_actor_count();
 }
+aid_t Engine::get_actor_max_pid() const
+{
+  return kernel::actor::ActorImpl::get_maxpid() - 1; // maxpid_ is post-incremented
+}
 
 std::vector<ActorPtr> Engine::get_all_actors() const
 {
@@ -490,13 +688,35 @@ void Engine::track_vetoed_activities(std::set<Activity*>* vetoed_activities) con
 {
   Activity::set_vetoed_activities(vetoed_activities);
 }
-
 /** @brief Retrieve the root netzone, containing all others */
 s4u::NetZone* Engine::get_netzone_root() const
 {
-  if (pimpl_->netzone_root_)
-    return pimpl_->netzone_root_->get_iface();
-  return nullptr;
+  if (not pimpl_->netzone_root_) {
+    /* The platform is getting populated with netzones. It's time to finish the initialization.
+     *
+     * Without globals and with current model description init functions (see module.hpp), we need
+     * the root netzone to exist when creating the models.
+     */
+    pimpl_->netzone_root_ = new kernel::routing::FullZone("_world_");
+
+    simgrid::s4u::Engine::on_platform_creation();
+
+    /* Initialize the models. That must be done after we got all config, and before we need the models.
+     * That is, after the last <config> tag, if any, and before the first of cluster|peer|zone|trace|trace_cb
+     *
+     * I'm not sure for <trace> and <trace_cb>, there may be a bug here
+     * (FIXME: check it out by creating a file beginning with one of these tags)
+     * but cluster and peer come down to zone creations, so putting this verification here is correct.
+     */
+    simgrid_host_models().init_from_flag_value();
+    simgrid_vm_model_init_HL13();
+
+    /* HACK: direct access to the global controlling the level of configuration to prevent
+     * any further config now that we created some real content */
+    _sg_cfg_init_status = 2;
+  }
+
+  return pimpl_->netzone_root_->get_iface();
 }
 /** @brief Set the root netzone, containing all others. Once set, it cannot be changed. */
 void Engine::set_netzone_root(const s4u::NetZone* netzone)
@@ -505,6 +725,17 @@ void Engine::set_netzone_root(const s4u::NetZone* netzone)
   pimpl_->netzone_root_ = netzone->get_impl();
 }
 
+std::vector<NetZone*> Engine::get_all_netzones() const
+{
+  std::vector<kernel::routing::NetZoneImpl*> netzone_pimpl_list = {pimpl_->netzone_root_};
+  std::vector<NetZone*> netzone_list;
+  get_filtered_netzones_recursive(pimpl_->netzone_root_->get_iface(), &netzone_pimpl_list);
+  for (auto const& n : netzone_pimpl_list) {
+    netzone_list.push_back(n->get_iface());
+  }
+  return netzone_list;
+}
+
 static NetZone* netzone_by_name_recursive(NetZone* current, const std::string& name)
 {
   if (current->get_name() == name)
@@ -632,7 +863,25 @@ double simgrid_get_clock()
 {
   return simgrid::s4u::Engine::get_clock();
 }
-
+void simgrid_get_papi_counters(long long* tab)
+{
+  std::vector<long long> cnt = simgrid::s4u::Engine::get_papi_counters();
+  for (size_t i = 0; i < cnt.size(); i++) {
+    tab[i] = cnt[i];
+  }
+}
+int simgrid_papi_get_num_counters()
+{
+  return simgrid::s4u::Engine::papi_get_num_counters();
+}
+void simgrid_papi_start()
+{
+  return simgrid::s4u::Engine::papi_start();
+}
+void simgrid_papi_stop()
+{
+  return simgrid::s4u::Engine::papi_stop();
+}
 void simgrid_set_maestro(void (*code)(void*), void* data)
 {
   maestro_code = std::bind(code, data);
