@@ -1,9 +1,9 @@
-/* Copyright (c) 2006-2023. The SimGrid Team. All rights reserved.          */
+/* Copyright (c) 2006-2025. The SimGrid Team. All rights reserved.          */
 
 /* This program is free software; you can redistribute it and/or modify it
  * under the terms of the license (GNU LGPL) which comes with this package. */
 
-#include "simgrid/simix.hpp"
+#include "simgrid/simcall.hpp"
 #include <simgrid/Exception.hpp>
 #include <simgrid/exec.h>
 #include <simgrid/s4u/ActivitySet.hpp>
@@ -17,6 +17,51 @@
 XBT_LOG_NEW_DEFAULT_SUBCATEGORY(s4u_exec, s4u_activity, "S4U asynchronous executions");
 
 namespace simgrid::s4u {
+template <> xbt::signal<void(Exec&)> Activity_T<Exec>::on_veto             = xbt::signal<void(Exec&)>();
+template <> xbt::signal<void(Exec const&)> Activity_T<Exec>::on_start      = xbt::signal<void(Exec const&)>();
+template <> xbt::signal<void(Exec const&)> Activity_T<Exec>::on_completion = xbt::signal<void(Exec const&)>();
+template <> xbt::signal<void(Exec const&)> Activity_T<Exec>::on_suspend    = xbt::signal<void(Exec const&)>();
+template <> xbt::signal<void(Exec const&)> Activity_T<Exec>::on_resume     = xbt::signal<void(Exec const&)>();
+template <> void Activity_T<Exec>::fire_on_start() const
+{
+  on_start(static_cast<const Exec&>(*this));
+}
+template <> void Activity_T<Exec>::fire_on_completion() const
+{
+  on_completion(static_cast<const Exec&>(*this));
+}
+template <> void Activity_T<Exec>::fire_on_suspend() const
+{
+  on_suspend(static_cast<const Exec&>(*this));
+}
+template <> void Activity_T<Exec>::fire_on_resume() const
+{
+  on_resume(static_cast<const Exec&>(*this));
+}
+template <> void Activity_T<Exec>::fire_on_veto()
+{
+  on_veto(static_cast<Exec&>(*this));
+}
+template <> void Activity_T<Exec>::on_start_cb(const std::function<void(Exec const&)>& cb)
+{
+  on_start.connect(cb);
+}
+template <> void Activity_T<Exec>::on_completion_cb(const std::function<void(Exec const&)>& cb)
+{
+  on_completion.connect(cb);
+}
+template <> void Activity_T<Exec>::on_suspend_cb(const std::function<void(Exec const&)>& cb)
+{
+  on_suspend.connect(cb);
+}
+template <> void Activity_T<Exec>::on_resume_cb(const std::function<void(Exec const&)>& cb)
+{
+  on_resume.connect(cb);
+}
+template <> void Activity_T<Exec>::on_veto_cb(const std::function<void(Exec&)>& cb)
+{
+  on_veto.connect(cb);
+}
 
 Exec::Exec(kernel::activity::ExecImplPtr pimpl)
 {
@@ -37,10 +82,12 @@ ExecPtr Exec::init()
 Exec* Exec::do_start()
 {
   kernel::actor::simcall_answered([this] {
-    (*boost::static_pointer_cast<kernel::activity::ExecImpl>(pimpl_))
-        .set_name(get_name())
-        .set_tracing_category(get_tracing_category())
-        .start();
+    auto pimpl = boost::static_pointer_cast<kernel::activity::ExecImpl>(pimpl_);
+    pimpl->set_name(get_name());
+    pimpl->set_tracing_category(get_tracing_category());
+    if (detached_)
+      pimpl->detach();
+    pimpl->start();
   });
 
   if (suspended_)
@@ -52,7 +99,7 @@ Exec* Exec::do_start()
   return this;
 }
 
-ssize_t Exec::deprecated_wait_any_for(const std::vector<ExecPtr>& execs, double timeout) // XBT_ATTRIB_DEPRECATED_v339
+ssize_t Exec::deprecated_wait_any_for(const std::vector<ExecPtr>& execs, double timeout) // XBT_ATTRIB_DEPRECATED_v401
 {
   if (execs.empty())
     return -1;
@@ -71,10 +118,6 @@ ssize_t Exec::deprecated_wait_any_for(const std::vector<ExecPtr>& execs, double
   return -1;
 }
 
-/** @brief change the execution bound
- * This means changing the maximal amount of flops per second that it may consume, regardless of what the host may
- * deliver. Currently, this cannot be changed once the exec started.
- */
 ExecPtr Exec::set_bound(double bound)
 {
   xbt_assert(state_ == State::INITED || state_ == State::STARTING,
@@ -85,12 +128,6 @@ ExecPtr Exec::set_bound(double bound)
   return this;
 }
 
-/** @brief  Change the execution priority, don't you think?
- *
- * An execution with twice the priority will get twice the amount of flops when the resource is shared.
- * The default priority is 1.
- *
- * Currently, this cannot be changed once the exec started. */
 ExecPtr Exec::set_priority(double priority)
 {
   xbt_assert(state_ == State::INITED || state_ == State::STARTING,
@@ -152,9 +189,6 @@ ExecPtr Exec::set_thread_count(int thread_count)
   return this;
 }
 
-/** @brief Retrieve the host on which this activity takes place.
- *  If it runs on more than one host, only the first host is returned.
- */
 Host* Exec::get_host() const
 {
   return static_cast<kernel::activity::ExecImpl*>(pimpl_.get())->get_host();
@@ -169,9 +203,6 @@ int Exec::get_thread_count() const
   return static_cast<kernel::activity::ExecImpl*>(pimpl_.get())->get_thread_count();
 }
 
-/** @brief Change the host on which this activity takes place.
- *
- * The activity cannot be terminated already (but it may be started). */
 ExecPtr Exec::set_host(Host* host)
 {
   xbt_assert(state_ == State::INITED || state_ == State::STARTING || state_ == State::STARTED,
@@ -238,10 +269,6 @@ double Exec::get_remaining() const
         [this]() { return boost::static_pointer_cast<kernel::activity::ExecImpl>(pimpl_)->get_remaining(); });
 }
 
-/** @brief Returns the ratio of elements that are still to do
- *
- * The returned value is between 0 (completely done) and 1 (nothing done yet).
- */
 double Exec::get_remaining_ratio() const
 {
   if (is_parallel())
@@ -336,7 +363,7 @@ sg_error_t sg_exec_wait_for(sg_exec_t exec, double timeout)
   return status;
 }
 
-ssize_t sg_exec_wait_any(sg_exec_t* execs, size_t count) // XBT_ATTRIB_DEPRECATED_v339
+ssize_t sg_exec_wait_any(sg_exec_t* execs, size_t count) // XBT_ATTRIB_DEPRECATED_v401
 {
   std::vector<simgrid::s4u::ExecPtr> s4u_execs;
   for (size_t i = 0; i < count; i++)
@@ -350,7 +377,7 @@ ssize_t sg_exec_wait_any(sg_exec_t* execs, size_t count) // XBT_ATTRIB_DEPRECATE
   return pos;
 }
 
-ssize_t sg_exec_wait_any_for(sg_exec_t* execs, size_t count, double timeout) // XBT_ATTRIB_DEPRECATED_v339
+ssize_t sg_exec_wait_any_for(sg_exec_t* execs, size_t count, double timeout) // XBT_ATTRIB_DEPRECATED_v401
 {
   std::vector<simgrid::s4u::ExecPtr> s4u_execs;
   for (size_t i = 0; i < count; i++)
