@@ -1,4 +1,4 @@
-/* Copyright (c) 2006-2023. The SimGrid Team. All rights reserved.          */
+/* Copyright (c) 2006-2025. The SimGrid Team. All rights reserved.          */
 
 /* This program is free software; you can redistribute it and/or modify it
  * under the terms of the license (GNU LGPL) which comes with this package. */
@@ -13,7 +13,8 @@
 #include <simgrid/s4u/VirtualMachine.hpp>
 #include <xbt/parse_units.hpp>
 
-#include "simgrid/simix.hpp"
+#include "simgrid/s4u/Actor.hpp"
+#include "simgrid/simcall.hpp"
 #include "src/kernel/resource/HostImpl.hpp"
 #include "src/kernel/resource/StandardLinkImpl.hpp"
 #include "src/kernel/resource/VirtualMachineImpl.hpp"
@@ -337,6 +338,11 @@ Host* Host::set_factor_cb(const std::function<CpuFactorCb>& cb)
   kernel::actor::simcall_object_access(pimpl_, [this, &cb] { pimpl_cpu_->set_factor_cb(cb); });
   return this;
 }
+Host* Host::set_cpu_factor_cb(const std::function<double(Host&, double)>& cb)
+{
+  kernel::actor::simcall_object_access(pimpl_, [this, &cb] { pimpl_cpu_->set_cpu_factor_cb(cb); });
+  return this;
+}
 
 Host* Host::set_coordinates(const std::string& coords)
 {
@@ -349,16 +355,21 @@ std::vector<Disk*> Host::get_disks() const
   return this->pimpl_->get_disks();
 }
 
-Disk* Host::create_disk(const std::string& name, double read_bandwidth, double write_bandwidth)
+Disk* Host::get_disk_by_name(const std::string& name) const
+{
+  return this->pimpl_->get_disk_by_name(name);
+}
+
+Disk* Host::add_disk(const std::string& name, double read_bandwidth, double write_bandwidth)
 {
   return kernel::actor::simcall_answered([this, &name, read_bandwidth, write_bandwidth] {
-    auto* disk = pimpl_->create_disk(name, read_bandwidth, write_bandwidth);
-    pimpl_->add_disk(disk);
+    auto* disk = pimpl_->add_disk(name, read_bandwidth, write_bandwidth);
+    pimpl_->register_disk(disk);
     return disk;
   });
 }
 
-Disk* Host::create_disk(const std::string& name, const std::string& read_bandwidth, const std::string& write_bandwidth)
+Disk* Host::add_disk(const std::string& name, const std::string& read_bandwidth, const std::string& write_bandwidth)
 {
   double d_read;
   try {
@@ -372,12 +383,11 @@ Disk* Host::create_disk(const std::string& name, const std::string& read_bandwid
   } catch (const simgrid::ParseError&) {
     throw std::invalid_argument("Impossible to create disk: " + name + ". Invalid write bandwidth: " + write_bandwidth);
   }
-  return create_disk(name, d_read, d_write);
+  return add_disk(name, d_read, d_write);
 }
-
-void Host::add_disk(const Disk* disk)
+void Host::register_disk(const Disk* disk)
 {
-  kernel::actor::simcall_answered([this, disk] { this->pimpl_->add_disk(disk); });
+  kernel::actor::simcall_answered([this, disk] { this->pimpl_->register_disk(disk); });
 }
 
 void Host::remove_disk(const std::string& disk_name)
@@ -498,7 +508,10 @@ void sg_host_get_disks(const_sg_host_t host, unsigned int* disk_count, sg_disk_t
   *disks                      = xbt_new(sg_disk_t, list.size());
   std::copy(begin(list), end(list), *disks);
 }
-
+sg_disk_t sg_host_get_disk_by_name(const_sg_host_t host, const char* name)
+{
+  return host->get_disk_by_name(name);
+}
 // =========== user-level functions ===============
 // ================================================
 /** @brief Returns the total speed of a host */
@@ -611,6 +624,26 @@ xbt_dict_t sg_host_get_properties(const_sg_host_t host)
   }
   return as_dict;
 }
+const char** sg_host_get_property_names(const_sg_host_t host, int* size)
+{
+  const std::unordered_map<std::string, std::string>* props = host->get_properties();
+
+  if (props == nullptr) {
+    if (size)
+      *size = 0;
+    return nullptr;
+  }
+
+  const char** res = (const char**)xbt_malloc(sizeof(char*) * (props->size() + 1));
+  if (size)
+    *size = props->size();
+  int i = 0;
+  for (auto const& [key, _] : *props)
+    res[i++] = key.c_str();
+  res[i] = nullptr;
+
+  return res;
+}
 
 /** @ingroup m_host_management
  * @brief Returns the value of a given host property
@@ -643,6 +676,22 @@ void sg_host_get_route(const_sg_host_t from, const_sg_host_t to, xbt_dynar_t lin
   for (auto const& link : vlinks)
     xbt_dynar_push(links, &link);
 }
+const_sg_link_t* sg_host_get_route_links(const_sg_host_t from, const_sg_host_t to, int* size)
+{
+  std::vector<simgrid::s4u::Link*> vlinks;
+  from->route_to(to, vlinks, nullptr);
+
+  const_sg_link_t* res = (const_sg_link_t*)xbt_malloc(sizeof(const_sg_link_t) * (vlinks.size() + 1));
+  if (size)
+    *size = vlinks.size();
+  int i = 0;
+  for (auto const& link : vlinks)
+    res[i++] = link;
+  res[i] = nullptr;
+
+  return res;
+}
+
 /**
  * @brief Find the latency of the route between two hosts
  *
@@ -681,17 +730,26 @@ void sg_host_sendto(sg_host_t from, sg_host_t to, double byte_amount)
   simgrid::s4u::Comm::sendto(from, to, byte_amount);
 }
 
-/** @brief Return the list of actors attached to a host.
- *
- * @param host a host
- * @param whereto a dynar in which we should push actors living on that host
- */
 void sg_host_get_actor_list(const_sg_host_t host, xbt_dynar_t whereto)
 {
   auto const actors = host->get_all_actors();
   for (auto const& actor : actors)
     xbt_dynar_push(whereto, &actor);
 }
+const_sg_actor_t* sg_host_get_actors(const_sg_host_t host, int* size)
+{
+  std::vector<simgrid::s4u::ActorPtr> actors = host->get_all_actors();
+
+  const_sg_actor_t* res = (const_sg_actor_t*)xbt_malloc(sizeof(const_sg_link_t) * (actors.size() + 1));
+  if (size)
+    *size = actors.size();
+  int i = 0;
+  for (auto actor : actors)
+    res[i++] = actor.get();
+  res[i] = nullptr;
+
+  return res;
+}
 
 sg_host_t sg_host_self()
 {
