@@ -1,9 +1,10 @@
-/* Copyright (c) 2015-2023. The SimGrid Team. All rights reserved.          */
+/* Copyright (c) 2015-2025. The SimGrid Team. All rights reserved.          */
 
 /* This program is free software; you can redistribute it and/or modify it
  * under the terms of the license (GNU LGPL) which comes with this package. */
 
-#include "simgrid/simix.hpp"
+#include "simgrid/s4u/Engine.hpp"
+#include "simgrid/simcall.hpp"
 #include <simgrid/Exception.hpp>
 #include <simgrid/kernel/routing/NetPoint.hpp>
 #include <simgrid/vm.h>
@@ -32,14 +33,6 @@ void VmHostExt::ensureVmExtInstalled()
     EXTENSION_ID = Host::extension_create<VmHostExt>();
 }
 
-VirtualMachine::VirtualMachine(const std::string& name, s4u::Host* physical_host, int core_amount,
-                               size_t ramsize) // XBT_ATTRIB_DEPRECATED_v336
-    : Host(new kernel::resource::VirtualMachineImpl(name, this, physical_host, core_amount, ramsize))
-    , pimpl_vm_(dynamic_cast<kernel::resource::VirtualMachineImpl*>(Host::get_impl()))
-{
-  physical_host->get_impl()->create_vm(name, this);
-}
-
 VirtualMachine::VirtualMachine(kernel::resource::VirtualMachineImpl* impl)
     : Host(impl), pimpl_vm_(dynamic_cast<kernel::resource::VirtualMachineImpl*>(Host::get_impl()))
 {
@@ -83,7 +76,7 @@ void VirtualMachine::destroy()
 
   if (not this_actor::is_maestro() && this_actor::get_host() == this) {
     XBT_VERB("Launch another actor on physical host %s to destroy my own VM: %s", get_pm()->get_cname(), get_cname());
-    simgrid::s4u::Actor::create(get_name() + "-vm_destroy", get_pm(), destroy_code);
+    simgrid::s4u::Engine::get_instance()->add_actor(get_name() + "-vm_destroy", get_pm(), destroy_code);
     simgrid::s4u::this_actor::yield();
     XBT_CRITICAL("I should be dead now!");
     DIE_IMPOSSIBLE;
