@@ -1,4 +1,4 @@
-/* Copyright (c) 2018-2023. The SimGrid Team. All rights reserved.          */
+/* Copyright (c) 2018-2025. The SimGrid Team. All rights reserved.          */
 
 /* This program is free software; you can redistribute it and/or modify it
  * under the terms of the license (GNU LGPL) which comes with this package. */
@@ -30,17 +30,4 @@ void ForcefulKillException::do_throw()
 {
   throw ForcefulKillException();
 }
-
-bool ForcefulKillException::try_n_catch(const std::function<void()>& try_block) // XBT_ATTRIB_DEPRECATED_v338
-{
-  bool res;
-  try {
-    try_block();
-    res = true;
-  } catch (ForcefulKillException const& e) {
-    XBT_DEBUG("Caught a ForcefulKillException: %s", e.what());
-    res = false;
-  }
-  return res;
-}
 } // namespace simgrid
