@@ -1,4 +1,4 @@
-/* Copyright (c) 2007-2023. The SimGrid Team. All rights reserved.          */
+/* Copyright (c) 2007-2025. The SimGrid Team. All rights reserved.          */
 
 /* This program is free software; you can redistribute it and/or modify it
  * under the terms of the license (GNU LGPL) which comes with this package. */
@@ -18,6 +18,8 @@
 #include <getopt.h>
 
 #include "src/smpi/include/smpi_actor.hpp"
+#include "xbt/log.h"
+#include "xbt/xbt_os_time.h"
 
 #include <cerrno>
 #include <cmath>
@@ -84,11 +86,7 @@ void smpi_bench_begin()
 
 #if HAVE_PAPI
   if (not smpi_cfg_papi_events_file().empty()) {
-    int event_set = smpi_process()->papi_event_set();
-    // PAPI_start sets everything to 0! See man(3) PAPI_start
-    if (PAPI_LOW_LEVEL_INITED == PAPI_is_initialized() && event_set)
-      xbt_assert(PAPI_start(event_set) == PAPI_OK,
-                 "Could not start PAPI counters (TODO: this needs some proper handling).");
+      simgrid_papi_start();
   }
 #endif
   xbt_os_threadtimer_start(smpi_process()->timer());
@@ -121,14 +119,7 @@ void smpi_bench_end()
    * our PAPI counters for this process.
    */
   if (not smpi_cfg_papi_events_file().empty()) {
-    papi_counter_t& counter_data        = smpi_process()->papi_counters();
-    int event_set                       = smpi_process()->papi_event_set();
-    std::vector<long long> event_values(counter_data.size());
-
-    if (event_set)
-      xbt_assert(PAPI_stop(event_set, &event_values[0]) == PAPI_OK, "Could not stop PAPI counters.");
-    for (unsigned int i = 0; i < counter_data.size(); i++)
-      counter_data[i].second += event_values[i];
+      simgrid_papi_stop();
   }
 #endif
 
@@ -390,7 +381,7 @@ void smpi_sample_iter(int global, const char* file, const char* tag)
   data.sum_pow2 += period * period;
   double n       = data.count;
   data.mean      = data.sum / n;
-  data.relstderr = sqrt((data.sum_pow2 / n - data.mean * data.mean) / n) / data.mean;
+  data.relstderr = sqrt((data.sum_pow2 / n) - (data.mean * data.mean)) / data.mean;
 
   XBT_DEBUG("Average mean after %d steps is %f, relative standard error is %f (sample was %f)",
             data.count, data.mean, data.relstderr, period);
@@ -487,3 +478,51 @@ int smpi_getopt (int argc,  char *const *argv,  const char *options)
 pid_t smpi_getpid(){
   return static_cast<pid_t>(simgrid::s4u::this_actor::get_pid());
 }
+
+#define SIZE 2000
+
+double smpi_autobench()
+{
+  XBT_INFO("You requested an automatic benchmark of this machine. This may take some time.");
+
+  // Allocate matrices
+  double* A = (double*)malloc(SIZE * SIZE * sizeof(double));
+  double* B = (double*)malloc(SIZE * SIZE * sizeof(double));
+  double* C = (double*)malloc(SIZE * SIZE * sizeof(double));
+
+  // Fill in matrix values
+  for (int i = 0; i < SIZE * SIZE; i++) {
+    A[i] = 2.0 * i / (SIZE * SIZE);
+    B[i] = -1.0 * i / (SIZE * SIZE);
+    C[i] = 0.0;
+  }
+
+  xbt_os_timer_t timer = xbt_os_timer_new();
+
+  // Multiply the matrices
+  xbt_os_threadtimer_start(timer);
+  for (int i = 0; i < SIZE; i++)
+    for (int k = 0; k < SIZE; k++)
+      for (int j = 0; j < SIZE; j++)
+        C[i * SIZE + j] += A[i * SIZE + k] * B[k * SIZE + j];
+  xbt_os_threadtimer_stop(timer);
+
+  // Compute the sum (to avoid compiler optimization)
+  double sum = 0;
+  for (int i = 0; i < SIZE * SIZE; i++)
+    sum += C[i];
+
+  double elapsed     = xbt_os_timer_elapsed(timer);
+  double number_flop = (3.0 * SIZE * SIZE * SIZE + SIZE * SIZE);
+
+  free(A);
+  free(B);
+  free(C);
+
+  // Print the wall-clock time and the sum (to avoid compiler optimization)
+  XBT_INFO("Automatic calibration ended after %lf seconds (for about 24Gflop). Use --cfg=smpi/host-speed:%.3lfGf next "
+           "time on this machine. (ignorable sum: %d)",
+           elapsed, number_flop / elapsed / (1000 * 1000 * 1000), (int)sum);
+
+  return number_flop / elapsed;
+}
\ No newline at end of file
