@@ -1,4 +1,4 @@
-/* Copyright (c) 2007-2023. The SimGrid Team. All rights reserved.          */
+/* Copyright (c) 2007-2025. The SimGrid Team. All rights reserved.          */
 
 /* This program is free software; you can redistribute it and/or modify it
  * under the terms of the license (GNU LGPL) which comes with this package. */
@@ -79,6 +79,8 @@ XBT_LOG_NEW_DEFAULT_SUBCATEGORY(smpi_kernel, smpi, "Logging specific to SMPI (ke
   // this default name is used there as well (when the <actor> tag is generated).
   static const std::string smpi_default_instance_name("smpirun");
 
+  static std::vector<std::string> smpi_executables;
+
   static simgrid::config::Flag<std::string>
       smpi_hostfile("smpi/hostfile",
                     "Classical MPI hostfile containing list of machines to dispatch "
@@ -392,7 +394,7 @@ static int visit_libs(struct dl_phdr_info* info, size_t, void* data)
 }
 #endif
 
-static void smpi_init_privatization_dlopen(const std::string& executable)
+static void smpi_init_privatization_dlopen(const std::string& executable, bool use_default = true)
 {
   // Prepare the copy of the binary (get its size)
   struct stat fdin_stat;
@@ -423,67 +425,78 @@ static void smpi_init_privatization_dlopen(const std::string& executable)
     }
   }
 
-  simgrid::s4u::Engine::get_instance()->register_default([executable, fdin_size](std::vector<std::string> args) {
-    return simgrid::kernel::actor::ActorCode([executable, fdin_size, args = std::move(args)] {
-      static std::size_t rank = 0;
-      // Copy the dynamic library:
-      simgrid::xbt::Path path(executable);
-      std::string target_executable = simgrid::config::get_value<std::string>("smpi/tmpdir") + "/" +
-          path.get_base_name() + "_" + std::to_string(getpid()) + "_" + std::to_string(rank) + ".so";
+  std::string function_name = "";
 
-      smpi_copy_file(executable, target_executable, fdin_size);
-      // if smpi/privatize-libs is set, duplicate pointed lib and link each executable copy to a different one.
-      std::vector<std::string> target_libs;
-      for (auto const& libpath : privatize_libs_paths) {
-        // if we were given a full path, strip it
-        size_t index = libpath.find_last_of("/\\");
-        std::string libname;
-        if (index != std::string::npos)
-          libname = libpath.substr(index + 1);
+  if (!use_default) {
+    function_name = executable;
+  }
 
-        if (not libname.empty()) {
-          // load the library to add it to the local libs, to get the absolute path
-          struct stat fdin_stat2;
-          stat(libpath.c_str(), &fdin_stat2);
-          off_t fdin_size2 = fdin_stat2.st_size;
+  simgrid::s4u::Engine::get_instance()->register_function(
+      function_name,
+      static_cast<simgrid::kernel::actor::ActorCodeFactory>([executable, fdin_size](std::vector<std::string> args) {
+        return simgrid::kernel::actor::ActorCode([executable, fdin_size, args = std::move(args)] {
+          static std::size_t rank = 0;
+          // Copy the dynamic library:
+          simgrid::xbt::Path path(executable);
+          std::string target_executable = simgrid::config::get_value<std::string>("smpi/tmpdir") + "/" +
+                                          path.get_base_name() + "_" + std::to_string(getpid()) + "_" +
+                                          std::to_string(rank) + ".so";
 
-          // Copy the dynamic library, the new name must be the same length as the old one
-          // just replace the name with 7 digits for the rank and the rest of the name.
-          auto pad                   = std::min<size_t>(7, libname.length());
-          std::string target_libname = std::string(pad - std::to_string(rank).length(), '0') + std::to_string(rank) + libname.substr(pad);
-          std::string target_lib = simgrid::config::get_value<std::string>("smpi/tmpdir") + "/" + target_libname;
-          target_libs.push_back(target_lib);
-          XBT_DEBUG("copy lib %s to %s, with size %lld", libpath.c_str(), target_lib.c_str(), (long long)fdin_size2);
-          smpi_copy_file(libpath, target_lib, fdin_size2);
+          smpi_copy_file(executable, target_executable, fdin_size);
+          // if smpi/privatize-libs is set, duplicate pointed lib and link each executable copy to a different one.
+          std::vector<std::string> target_libs;
+          for (auto const& libpath : privatize_libs_paths) {
+            // if we were given a full path, strip it
+            size_t index = libpath.find_last_of("/\\");
+            std::string libname;
+            if (index != std::string::npos)
+              libname = libpath.substr(index + 1);
 
-          std::string sedcommand = "sed -i -e 's/" + libname + "/" + target_libname + "/g' " + target_executable;
-          int status             = system(sedcommand.c_str());
-          xbt_assert(status == 0, "error while applying sed command %s \n", sedcommand.c_str());
-        }
-      }
+            if (not libname.empty()) {
+              // load the library to add it to the local libs, to get the absolute path
+              struct stat fdin_stat2;
+              stat(libpath.c_str(), &fdin_stat2);
+              off_t fdin_size2 = fdin_stat2.st_size;
 
-      rank++;
-      // Load the copy and resolve the entry point:
-      void* handle    = dlopen(target_executable.c_str(), RTLD_LAZY | RTLD_LOCAL | WANT_RTLD_DEEPBIND);
-      int saved_errno = errno;
-      if (not simgrid::config::get_value<bool>("smpi/keep-temps")) {
-        unlink(target_executable.c_str());
-        for (const std::string& target_lib : target_libs)
-          unlink(target_lib.c_str());
-      }
-      xbt_assert(handle != nullptr,
-                 "dlopen failed: %s (errno: %d -- %s).\nError: Did you compile the program with a SMPI-specific "
-                 "compiler (spmicc or friends)?",
-                 dlerror(), saved_errno, strerror(saved_errno));
+              // Copy the dynamic library, the new name must be the same length as the old one
+              // just replace the name with 7 digits for the rank and the rest of the name.
+              auto pad = std::min<size_t>(7, libname.length());
+              std::string target_libname =
+                  std::string(pad - std::to_string(rank).length(), '0') + std::to_string(rank) + libname.substr(pad);
+              std::string target_lib = simgrid::config::get_value<std::string>("smpi/tmpdir") + "/" + target_libname;
+              target_libs.push_back(target_lib);
+              XBT_DEBUG("copy lib %s to %s, with size %lld", libpath.c_str(), target_lib.c_str(),
+                        (long long)fdin_size2);
+              smpi_copy_file(libpath, target_lib, fdin_size2);
 
-      smpi_entry_point_type entry_point = smpi_resolve_function(handle);
-      xbt_assert(entry_point, "Could not resolve entry point. Does your program contain a main() function?");
-      smpi_run_entry_point(entry_point, executable, args);
-    });
-  });
+              std::string sedcommand = "sed -i -e 's/" + libname + "/" + target_libname + "/g' " + target_executable;
+              int status             = system(sedcommand.c_str());
+              xbt_assert(status == 0, "error while applying sed command %s \n", sedcommand.c_str());
+            }
+          }
+
+          rank++;
+          // Load the copy and resolve the entry point:
+          void* handle    = dlopen(target_executable.c_str(), RTLD_LAZY | RTLD_LOCAL | WANT_RTLD_DEEPBIND);
+          int saved_errno = errno;
+          if (not simgrid::config::get_value<bool>("smpi/keep-temps")) {
+            unlink(target_executable.c_str());
+            for (const std::string& target_lib : target_libs)
+              unlink(target_lib.c_str());
+          }
+          xbt_assert(handle != nullptr,
+                     "dlopen failed: %s (errno: %d -- %s).\nError: Did you compile the program with a SMPI-specific "
+                     "compiler (spmicc or friends)?",
+                     dlerror(), saved_errno, strerror(saved_errno));
+
+          smpi_entry_point_type entry_point = smpi_resolve_function(handle);
+          xbt_assert(entry_point, "Could not resolve entry point. Does your program contain a main() function?");
+          smpi_run_entry_point(entry_point, executable, args);
+        });
+      }));
 }
 
-static void smpi_init_privatization_no_dlopen(const std::string& executable)
+static void smpi_init_privatization_no_dlopen(const std::string& executable, bool use_default = true)
 {
   if (smpi_cfg_privatization() == SmpiPrivStrategies::MMAP)
     smpi_prepare_global_memory_segment();
@@ -498,19 +511,27 @@ static void smpi_init_privatization_no_dlopen(const std::string& executable)
   if (smpi_cfg_privatization() == SmpiPrivStrategies::MMAP)
     smpi_backup_global_memory_segment();
 
+  std::string function_name = "";
+
+  if (!use_default) {
+    function_name = executable;
+  }
+
   // Execute the same entry point for each simulated process:
-  simgrid::s4u::Engine::get_instance()->register_default([entry_point, executable](std::vector<std::string> args) {
-    return simgrid::kernel::actor::ActorCode([entry_point, executable, args = std::move(args)] {
-      if (smpi_cfg_privatization() == SmpiPrivStrategies::MMAP) {
-        simgrid::smpi::ActorExt* ext = smpi_process();
-        /* Now using the segment index of this process  */
-        ext->set_privatized_region(smpi_init_global_memory_segment_process());
-        /* Done at the process's creation */
-        smpi_switch_data_segment(simgrid::s4u::Actor::self());
-      }
-      smpi_run_entry_point(entry_point, executable, args);
-    });
-  });
+  simgrid::s4u::Engine::get_instance()->register_function(
+      function_name,
+      static_cast<simgrid::kernel::actor::ActorCodeFactory>([entry_point, executable](std::vector<std::string> args) {
+        return simgrid::kernel::actor::ActorCode([entry_point, executable, args = std::move(args)] {
+          if (smpi_cfg_privatization() == SmpiPrivStrategies::MMAP) {
+            simgrid::smpi::ActorExt* ext = smpi_process();
+            /* Now using the segment index of this process  */
+            ext->set_privatized_region(smpi_init_global_memory_segment_process());
+            /* Done at the process's creation */
+            smpi_switch_data_segment(simgrid::s4u::Actor::self());
+          }
+          smpi_run_entry_point(entry_point, executable, args);
+        });
+      }));
 }
 
 int smpi_main(const char* executable, int argc, char* argv[])
@@ -659,3 +680,28 @@ void smpi_exit(int res){
   simgrid::s4u::this_actor::exit();
   THROW_IMPOSSIBLE;
 }
+
+void SMPI_executable_init(const std::string& executable)
+{
+  if (std::find(smpi_executables.begin(), smpi_executables.end(), executable) != smpi_executables.end())
+    return;
+
+  XBT_DEBUG("Initialize SMPI for %s", executable.c_str());
+  simgrid::s4u::Engine* engine = simgrid::s4u::Engine::get_instance();
+
+  if (smpi_executables.empty())
+    sg_storage_file_system_init();
+  engine->set_default_comm_data_copy_callback(smpi_comm_copy_buffer_callback);
+
+  xbt_assert(not MC_is_active() || smpi_cfg_privatization() != SmpiPrivStrategies::MMAP,
+             "Please use the dlopen privatization schema when model-checking SMPI code");
+
+  if (smpi_cfg_privatization() == SmpiPrivStrategies::DLOPEN)
+    smpi_init_privatization_dlopen(executable, false);
+  else if (smpi_cfg_privatization() == SmpiPrivStrategies::MMAP)
+    xbt_die("Privatization with MMAP is not supported");
+  else
+    smpi_init_privatization_no_dlopen(executable, false);
+
+  smpi_executables.push_back(executable);
+}
\ No newline at end of file
