@@ -1,4 +1,4 @@
-/* Copyright (c) 2005-2023. The SimGrid Team. All rights reserved.          */
+/* Copyright (c) 2005-2025. The SimGrid Team. All rights reserved.          */
 
 /* This program is free software; you can redistribute it and/or modify it
  * under the terms of the license (GNU LGPL) which comes with this package. */
@@ -15,6 +15,10 @@
 #include <cstdlib>
 #include <sstream>
 
+#if HAVE_STD_STACKTRACE
+#include <stacktrace>
+#endif
+
 #if HAVE_BOOST_STACKTRACE_BACKTRACE
 #define BOOST_STACKTRACE_USE_BACKTRACE
 #include <boost/stacktrace.hpp>
@@ -34,7 +38,40 @@ void xbt_backtrace_display_current()
 namespace simgrid::xbt {
 
 class BacktraceImpl {
-#if HAVE_BOOST_STACKTRACE_BACKTRACE || HAVE_BOOST_STACKTRACE_ADDR2LINE
+#if HAVE_STD_STACKTRACE
+  std::stacktrace st = std::stacktrace::current();
+
+public:
+  std::string resolve() const
+  {
+    std::stringstream ss;
+
+    int frame_count = 0;
+    bool print      = false;
+
+    for (const auto& frame : st) {
+      const std::string frame_name = frame.description();
+
+      if (print) {
+        if (frame_name.rfind("simgrid::xbt::MainFunction", 0) == 0 ||
+            frame_name.rfind("simgrid::kernel::context::Context::operator()()", 0) == 0 ||
+            frame_name.rfind("auto sthread_create::{lambda") == 0)
+          break;
+        ss << "  ->  #" << frame_count++ << " ";
+        if (xbt_log_no_loc) // Don't display file source and line if so
+          ss << (frame_name.empty() ? "(debug info not found and log:no_loc activated)" : frame_name) << "\n";
+        else
+          ss << frame << "\n";
+        if (frame_name == "main")
+          break;
+      } else if (frame_name ==
+                 "std::shared_ptr<simgrid::xbt::BacktraceImpl> std::make_shared<simgrid::xbt::BacktraceImpl>()") {
+        print = true;
+      }
+    }
+    return ss.str();
+  }
+#elif HAVE_BOOST_STACKTRACE_BACKTRACE || HAVE_BOOST_STACKTRACE_ADDR2LINE
   const boost::stacktrace::stacktrace st;
 
 public:
@@ -57,14 +94,6 @@ public:
           ss << (frame_name.empty() ? "(debug info not found and log:no_loc activated)" : frame_name) << "\n";
         else
           ss << frame << "\n";
-        // If we are displaying the user side of a simcall, remove the crude details of context switching
-        if (frame_name.find("simgrid::kernel::actor::simcall_answered") != std::string::npos ||
-            frame_name.find("simgrid::kernel::actor::simcall_blocking") != std::string::npos ||
-            frame_name.find("simcall_run_answered") != std::string::npos ||
-            frame_name.find("simcall_run_blocking") != std::string::npos) {
-          frame_count = 0;
-          ss.str(""); // This is how you clear a stringstream in C++. clear() is something else :'(
-        }
         if (frame_name == "main")
           break;
       } else {
@@ -95,7 +124,8 @@ void Backtrace::display() const
   std::fprintf(stderr, "Backtrace (displayed in actor %s%s):\n%s\n",
                simgrid::s4u::Actor::is_maestro() ? "maestro" : sg_actor_self_get_name(),
                (xbt_log_no_loc ? " -- short trace because of --log=no_loc" : ""),
-               backtrace.empty() ? "(backtrace not set -- did you install Boost.Stacktrace?)" : backtrace.c_str());
+               backtrace.empty() ? "(the backtrace is empty -- please use a C++23 compiler or install Boost.Stacktrace)"
+                                 : backtrace.c_str());
 }
 
 } // namespace simgrid::xbt
