@@ -1,4 +1,4 @@
-/* Copyright (c) 2010-2023. The SimGrid Team. All rights reserved.          */
+/* Copyright (c) 2010-2025. The SimGrid Team. All rights reserved.          */
 
 /* This program is free software; you can redistribute it and/or modify it
  * under the terms of the license (GNU LGPL) which comes with this package. */
@@ -10,22 +10,23 @@ static void test_link_off_helper(double delay)
 {
   const double start = simgrid::s4u::Engine::get_clock();
 
-  simgrid::s4u::ActorPtr receiver = simgrid::s4u::Actor::create("receiver", all_hosts[1], [&start]() {
-    assert_exit(true, 9);
-    std::array<double, 5> milestone{{0.5, 3.5, 4.5, 7.5, 9.0}};
-    for (double& m : milestone)
-      m += start;
-    for (int i = 0; i < 4; i++) {
-      simgrid::s4u::this_actor::sleep_until(milestone[i]);
-      REQUIRE_NETWORK_FAILURE({
-        INFO("get(" << ('A' + i) << ")");
-        simgrid::s4u::Mailbox::by_name("mb")->get<int>();
+  simgrid::s4u::ActorPtr receiver =
+      simgrid::s4u::Engine::get_instance()->add_actor("receiver", all_hosts[1], [&start]() {
+        assert_exit(true, 9);
+        std::array<double, 5> milestone{{0.5, 3.5, 4.5, 7.5, 9.0}};
+        for (double& m : milestone)
+          m += start;
+        for (int i = 0; i < 4; i++) {
+          simgrid::s4u::this_actor::sleep_until(milestone[i]);
+          REQUIRE_NETWORK_FAILURE({
+            INFO("get(" << ('A' + i) << ")");
+            simgrid::s4u::Mailbox::by_name("mb")->get<int>();
+          });
+        }
+        simgrid::s4u::this_actor::sleep_until(milestone[4]);
       });
-    }
-    simgrid::s4u::this_actor::sleep_until(milestone[4]);
-  });
 
-  simgrid::s4u::ActorPtr sender = simgrid::s4u::Actor::create("sender", all_hosts[2], [&start]() {
+  simgrid::s4u::ActorPtr sender = simgrid::s4u::Engine::get_instance()->add_actor("sender", all_hosts[2], [&start]() {
     assert_exit(true, 9);
     int data            = 42;
     std::array<double, 5> milestone{{1.5, 2.5, 5.5, 6.5, 9.0}};
@@ -61,50 +62,53 @@ static void test_link_off_helper(double delay)
 static simgrid::s4u::ActorPtr sender_basic(bool& ending_boolean, bool expected_success, double duration,
                                            double delay = -1.0)
 {
-  return simgrid::s4u::Actor::create("sender", all_hosts[1], [&ending_boolean, expected_success, duration, delay]() {
-    assert_exit(expected_success, duration);
-    // Encapsulate the payload in a std::unique_ptr so that it is correctly free'd if/when the sender is killed during
-    // its communication (thanks to RAII).  The pointer is then released when the communication is over.
-    std::unique_ptr<char, decltype(&xbt_free_f)> payload(xbt_strdup("toto"), &xbt_free_f);
-    if (delay > 0.0) {
-      simgrid::s4u::this_actor::sleep_for(delay / 2.0);
-      auto comm = simgrid::s4u::Mailbox::by_name("mb")->put_init(payload.get(), 5000);
-      simgrid::s4u::this_actor::sleep_for(delay / 2.0);
-      comm->wait();
-    } else {
-      simgrid::s4u::Mailbox::by_name("mb")->put(payload.get(), 5000);
-    }
-    payload.release();
-    ending_boolean = true;
-  });
+  return simgrid::s4u::Engine::get_instance()->add_actor(
+      "sender", all_hosts[1], [&ending_boolean, expected_success, duration, delay]() {
+        assert_exit(expected_success, duration);
+        // Encapsulate the payload in a std::unique_ptr so that it is correctly free'd if/when the sender is killed
+        // during its communication (thanks to RAII).  The pointer is then released when the communication is over.
+        std::unique_ptr<char, decltype(&xbt_free_f)> payload(xbt_strdup("toto"), &xbt_free_f);
+        if (delay > 0.0) {
+          simgrid::s4u::this_actor::sleep_for(delay / 2.0);
+          auto comm = simgrid::s4u::Mailbox::by_name("mb")->put_init(payload.get(), 5000);
+          simgrid::s4u::this_actor::sleep_for(delay / 2.0);
+          comm->wait();
+        } else {
+          simgrid::s4u::Mailbox::by_name("mb")->put(payload.get(), 5000);
+        }
+        payload.release();
+        ending_boolean = true;
+      });
 }
 static simgrid::s4u::ActorPtr receiver_basic(bool& ending_boolean, bool expected_success, double duration,
                                              double delay = -1.0)
 {
-  return simgrid::s4u::Actor::create("receiver", all_hosts[2], [&ending_boolean, expected_success, duration, delay]() {
-    assert_exit(expected_success, duration);
-    char* payload;
-    if (delay > 0.0) {
-      simgrid::s4u::this_actor::sleep_for(delay / 2.0);
-      auto comm = simgrid::s4u::Mailbox::by_name("mb")->get_init()->set_dst_data(reinterpret_cast<void**>(&payload),
-                                                                                 sizeof(void*));
-      simgrid::s4u::this_actor::sleep_for(delay / 2.0);
-      comm->wait();
-    } else {
-      payload = simgrid::s4u::Mailbox::by_name("mb")->get<char>();
-    }
-    xbt_free(payload);
-    ending_boolean = true;
-  });
+  return simgrid::s4u::Engine::get_instance()->add_actor(
+      "receiver", all_hosts[2], [&ending_boolean, expected_success, duration, delay]() {
+        assert_exit(expected_success, duration);
+        char* payload;
+        if (delay > 0.0) {
+          simgrid::s4u::this_actor::sleep_for(delay / 2.0);
+          auto comm = simgrid::s4u::Mailbox::by_name("mb")->get_init()->set_dst_data(reinterpret_cast<void**>(&payload),
+                                                                                     sizeof(void*));
+          simgrid::s4u::this_actor::sleep_for(delay / 2.0);
+          comm->wait();
+        } else {
+          payload = simgrid::s4u::Mailbox::by_name("mb")->get<char>();
+        }
+        xbt_free(payload);
+        ending_boolean = true;
+      });
 }
 static simgrid::s4u::ActorPtr sender_dtach(bool& ending_boolean, bool expected_success, double duration)
 {
-  return simgrid::s4u::Actor::create("sender", all_hosts[1], [&ending_boolean, expected_success, duration]() {
-    assert_exit(expected_success, duration);
-    char* payload = xbt_strdup("toto");
-    simgrid::s4u::Mailbox::by_name("mb")->put_init(payload, 1000)->detach();
-    ending_boolean = true;
-  });
+  return simgrid::s4u::Engine::get_instance()->add_actor(
+      "sender", all_hosts[1], [&ending_boolean, expected_success, duration]() {
+        assert_exit(expected_success, duration);
+        char* payload = xbt_strdup("toto");
+        simgrid::s4u::Mailbox::by_name("mb")->put_init(payload, 1000)->detach();
+        ending_boolean = true;
+      });
 }
 
 TEST_CASE("Activity lifecycle: comm activities")
@@ -208,7 +212,7 @@ TEST_CASE("Activity lifecycle: comm activities")
 
     simgrid::s4u::ActorPtr sender = sender_basic(send_done, false, 2);
 
-    simgrid::s4u::Actor::create("receiver", all_hosts[2], [&recv_done]() {
+    simgrid::s4u::Engine::get_instance()->add_actor("receiver", all_hosts[2], [&recv_done]() {
       assert_exit(true, 2);
       REQUIRE_NETWORK_FAILURE({
         char* payload = simgrid::s4u::Mailbox::by_name("mb")->get<char>();
@@ -238,9 +242,9 @@ TEST_CASE("Activity lifecycle: comm activities")
     bool in_catch_after_on_exit  = false;
     bool send_done               = false;
 
-    simgrid::s4u::ActorPtr receiver =
-        simgrid::s4u::Actor::create("receiver", all_hosts[1], [&in_on_exit, &returned_from_main,
-                                                               &in_catch_before_on_exit, &in_catch_after_on_exit]() {
+    simgrid::s4u::ActorPtr receiver = simgrid::s4u::Engine::get_instance()->add_actor(
+        "receiver", all_hosts[1],
+        [&in_on_exit, &returned_from_main, &in_catch_before_on_exit, &in_catch_after_on_exit]() {
           assert_exit(false, 1);
           try {
             simgrid::s4u::Mailbox::by_name("mb")->get<int>();
@@ -254,12 +258,13 @@ TEST_CASE("Activity lifecycle: comm activities")
 
     receiver->on_exit([&in_on_exit](bool) { in_on_exit = true; });
 
-    simgrid::s4u::ActorPtr sender = simgrid::s4u::Actor::create("sender", all_hosts[2], [&send_done]() {
-      assert_exit(true, 1);
-      int data = 42;
-      REQUIRE_NETWORK_FAILURE(simgrid::s4u::Mailbox::by_name("mb")->put(&data, 100000));
-      send_done = true;
-    });
+    simgrid::s4u::ActorPtr sender =
+        simgrid::s4u::Engine::get_instance()->add_actor("sender", all_hosts[2], [&send_done]() {
+          assert_exit(true, 1);
+          int data = 42;
+          REQUIRE_NETWORK_FAILURE(simgrid::s4u::Mailbox::by_name("mb")->put(&data, 100000));
+          send_done = true;
+        });
 
     simgrid::s4u::this_actor::sleep_for(1);
     receiver->get_host()->turn_off();
@@ -309,7 +314,7 @@ TEST_CASE("Activity lifecycle: comm activities")
   BEGIN_SECTION("comm turn link off during wait_any")
   {
     XBT_INFO("try to communicate with communicating link turned off during wait_any");
-    simgrid::s4u::ActorPtr receiver = simgrid::s4u::Actor::create("receiver", all_hosts[1], []() {
+    simgrid::s4u::ActorPtr receiver = simgrid::s4u::Engine::get_instance()->add_actor("receiver", all_hosts[1], []() {
       assert_exit(true, 2);
       int* data;
       simgrid::s4u::CommPtr comm = simgrid::s4u::Mailbox::by_name("mb")->get_async<int>(&data);
@@ -317,7 +322,7 @@ TEST_CASE("Activity lifecycle: comm activities")
       REQUIRE_NETWORK_FAILURE(pending_comms.wait_any());
     });
 
-    simgrid::s4u::ActorPtr sender = simgrid::s4u::Actor::create("sender", all_hosts[2], []() {
+    simgrid::s4u::ActorPtr sender = simgrid::s4u::Engine::get_instance()->add_actor("sender", all_hosts[2], []() {
       assert_exit(true, 2);
       int data = 42;
       REQUIRE_NETWORK_FAILURE(simgrid::s4u::Mailbox::by_name("mb")->put(&data, 100000));
