@@ -1,4 +1,4 @@
-/* Copyright (c) 2017-2023. The SimGrid Team. All rights reserved.          */
+/* Copyright (c) 2017-2025. The SimGrid Team. All rights reserved.          */
 
 /* This program is free software; you can redistribute it and/or modify it
  * under the terms of the license (GNU LGPL) which comes with this package. */
@@ -24,16 +24,17 @@ static void dummy_daemon()
 
 static void autostart()
 {
+  simgrid::s4u::Engine& e  = *simgrid::s4u::this_actor::get_engine();
   simgrid::s4u::Host* host = simgrid::s4u::Host::by_name("Fafard");
 
   XBT_INFO("starting a dummy process on %s", host->get_cname());
-  simgrid::s4u::ActorPtr dummy_actor = simgrid::s4u::Actor::create("Dummy", host, dummy);
+  simgrid::s4u::ActorPtr dummy_actor = e.add_actor("Dummy", host, dummy);
   dummy_actor->on_exit([](bool failed) { XBT_INFO("Dummy actor %s.", failed ? "failed" : "terminating"); });
   dummy_actor->set_auto_restart(true);
   dummy_actor->on_exit([](bool) { XBT_INFO("On_exit callback set after autorestart"); });
 
   XBT_INFO("starting a daemon process on %s", host->get_cname());
-  simgrid::s4u::ActorPtr daemon_actor = simgrid::s4u::Actor::create("Daemon", host, dummy_daemon);
+  simgrid::s4u::ActorPtr daemon_actor = e.add_actor("Daemon", host, dummy_daemon);
   daemon_actor->on_exit([](bool failed) { XBT_INFO("Daemon actor %s.", failed ? "failed" : "terminating"); });
   daemon_actor->daemonize()->set_auto_restart(true);
   daemon_actor->on_exit([](bool) { XBT_INFO("On_exit callback set after autorestart"); });
@@ -55,7 +56,7 @@ int main(int argc, char* argv[])
   simgrid::s4u::Engine e(&argc, argv);
   e.load_platform(argv[1]);
 
-  simgrid::s4u::Actor::create("Autostart", e.host_by_name("Tremblay"), autostart);
+  e.add_actor("Autostart", e.host_by_name("Tremblay"), autostart);
 
   e.run();
   XBT_INFO("Simulation time %g", simgrid::s4u::Engine::get_clock());
