@@ -1,4 +1,4 @@
-/* Copyright (c) 2017-2023. The SimGrid Team. All rights reserved.          */
+/* Copyright (c) 2017-2025. The SimGrid Team. All rights reserved.          */
 
 /* This program is free software; you can redistribute it and/or modify it
  * under the terms of the license (GNU LGPL) which comes with this package. */
@@ -82,6 +82,12 @@ static void streamer(size_t size)
   clock = sg4::Engine::get_clock();
   sg4::Io::streamto(bob, bob_disk, bob, bob_disk, size);
   XBT_INFO("    Total : %.6f seconds", sg4::Engine::get_clock() - clock);
+
+  XBT_INFO("[Bob -> Alice] Streaming \"from memory to disk of size 0\" (no read)");
+  clock = sg4::Engine::get_clock();
+  sg4::Io::streamto(bob, nullptr, alice, alice_disk, 0);
+  XBT_INFO("    Total : %.6f seconds", sg4::Engine::get_clock() - clock);
+
 }
 
 static void background_send() {
@@ -102,21 +108,21 @@ int main(int argc, char** argv)
   sg4::Engine e(&argc, argv);
 
   /* simple platform containing 2 hosts and 2 disks */
-  auto* zone = sg4::create_full_zone("");
-  auto* bob  = zone->create_host("bob", 1e6);
-  auto* alice  = zone->create_host("alice", 1e6);
+  auto* zone   = e.get_netzone_root();
+  auto* bob  = zone->add_host("bob", 1e6);
+  auto* alice  = zone->add_host("alice", 1e6);
 
-  auto* link = zone->create_link("link", "2MBps")->set_latency("50us");
+  auto* link = zone->add_link("link", "2MBps")->set_latency("50us");
   zone->add_route(bob, alice, {link});
 
-  bob->create_disk("bob_disk", "1MBps", "500kBps");
-  alice->create_disk("alice_disk", "4MBps", "4MBps");
+  bob->add_disk("bob_disk", "1MBps", "500kBps");
+  alice->add_disk("alice_disk", "4MBps", "4MBps");
 
   zone->seal();
 
-  sg4::Actor::create("streamer", bob, streamer, 4e6);
-  sg4::Actor::create("background send", bob, background_send);
-  sg4::Actor::create("background recv", alice, background_recv);
+  e.add_actor("streamer", bob, streamer, 4e6);
+  e.add_actor("background send", bob, background_send);
+  e.add_actor("background recv", alice, background_recv);
 
   e.run();
 
