@@ -1,4 +1,4 @@
-/* Copyright (c) 2009-2023. The SimGrid Team. All rights reserved.          */
+/* Copyright (c) 2009-2025. The SimGrid Team. All rights reserved.          */
 
 /* This program is free software; you can redistribute it and/or modify it
  * under the terms of the license (GNU LGPL) which comes with this package. */
@@ -38,10 +38,10 @@ int main(int argc, char* argv[])
 
   simgrid::s4u::Actor::kill_all();
 
-  simgrid::s4u::Actor::create("sendpid", e.host_by_name("Tremblay"), sendpid);
-  simgrid::s4u::Actor::create("sendpid", e.host_by_name("Tremblay"), sendpid);
-  simgrid::s4u::Actor::create("sendpid", e.host_by_name("Tremblay"), sendpid);
-  simgrid::s4u::Actor::create("killall", e.host_by_name("Tremblay"), killall);
+  e.add_actor("sendpid", e.host_by_name("Tremblay"), sendpid);
+  e.add_actor("sendpid", e.host_by_name("Tremblay"), sendpid);
+  e.add_actor("sendpid", e.host_by_name("Tremblay"), sendpid);
+  e.add_actor("killall", e.host_by_name("Tremblay"), killall);
 
   e.run();
 
