@@ -9,7 +9,7 @@ set -ex
 echo "XXXXXXXXXXXXXXXX Install APT dependencies"
 SUDO="" # to ease the local testing
 $SUDO apt-get -y update
-$SUDO apt-get -y install python-is-python3 python3-setuptools libboost-dev libeigen3-dev
+$SUDO apt-get -y install python-is-python3 python3-setuptools libboost-dev autoconf gfortran libblas-dev liblapack-dev 
 $SUDO apt-get -y install --only-upgrade ca-certificates
 
 echo "XXXXXXXXXXXXXXXX build and test BigDFT (git version)"
@@ -22,10 +22,6 @@ export PATH=$PWD/simgrid-dev/smpi_script/bin/:$PATH
 export LD_LIBRARY_PATH=$PWD/simgrid-dev/lib/:$LD_LIBRARY_PATH
 export JHBUILD_RUN_AS_ROOT=1
 
-#workaround issue with ntpoly 3.0.0
-sed -i 's|repository type="tarball" name="ntpoly" href="https://github.com/william-dawson/NTPoly/archive/"|repository type="git" name="ntpoly" href="https://github.com/william-dawson/"|' ../modulesets/hpc-upstream.modules
-sed -i 's|module="ntpoly-v3.0.0.tar.gz"|module="ntpoly"|' ../modulesets/hpc-upstream.modules
-
 ../Installer.py autogen -y
 
 ../Installer.py -f ../../tools/jenkins/gfortran-simgrid.rc -y build
